############################################################################
#Name: ArmaturePhysics
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################
#
#This module is the top level engine in the Armature physics package (except for the user parameter script and the GUI).
#It contains three types of functions.
#
#Intitialization functions: These will be run as the GUI is starting up. If you add for example a hold bone to the blender armature while the GUI is
#							active, you need to shut down the GUI and start it up again for the holdBone to be recognized.
#
#MainEvolve(): The main function of evolution both for pure object evolution, pure dynamical evolution and
#			   the combination of the two.
#
#Monitor(): For monitoring the net apparent force and torque acting on the object, in comparison with the force and torque coming from 
#			target forcebone forces and torques. The ouput is in the form of the scale and rotation of the pins and arrows typically to be found in
#			Blender layer 17. Arrows are for the apparent forces and torques, pins are for the target forces and torques. Red color means force, cyan means torque.
#			The torque is the torque about the object center of mass. Forces are normalized so that a force lentght of one blender unit corresponds to the
#			magnitude of gravitatinal force of the whole object. Torques are normalized so that a torque length of one blender unit corresponds to the
#			magnitude of the torque of the gravitatinal force of the whole object times one blender unit (one meter).
#			Comment: Any net force from hold bones are not included in the monitoring of the target forces currently.
#			This should be simple to add though (during monitoring), by treating hold bones with no parent as force bones.
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from Blender import *
from Blender.Mathutils import Vector, Matrix  #For Vectors and matrices.
import time

######### Taking in the Scenery ####################################################

print ""
print "----------------------------------"
print "++++++++++++++++++++++++++++++++++"
Window.EditMode(0)		#Sets object mode - otherwise things do not works as they should
EPSILON=1e-10			#A limit below wich numbers may be considered zero

########################### Importing modules  #####################################

import ArmaturePhysicsGraphics as APG
reload(APG)

######## For development phase  ############################

import ArmaturePhysicsTools
reload(ArmaturePhysicsTools)
import ArmaturePhysicsObjects
reload(ArmaturePhysicsObjects)
import ArmaturePhysicsBoneDictClasses
reload(ArmaturePhysicsBoneDictClasses)
import ArmaturePhysicsBoneClasses 
reload(ArmaturePhysicsBoneClasses)

######### Loading modules ###################################

from ArmaturePhysicsTools import sign, MagnitudeVec, ModifyDefaultDict
from ArmaturePhysicsTools import AddSimpleBones, getSpringVec, getDampVec, GetFramelist, GetDt
from ArmaturePhysicsTools import RotVec4Vecs, RotateMat, SetRotMat

from ArmaturePhysicsObjects import PhysObjectClass
from ArmaturePhysicsBoneDictClasses import PhysBoneDictClass

from ArmaturePhysicsBoneDictClasses import InternalPhysBoneDictClass	
from ArmaturePhysicsBoneDictClasses import DynBoneDictClass	
from ArmaturePhysicsBoneDictClasses import SpringBoneDictClass	
from ArmaturePhysicsBoneDictClasses import DynBoneBranchesClass	


from ArmaturePhysicsBoneClasses import PhysBoneClass
from ArmaturePhysicsBoneClasses import InternalPhysBoneClass	#Inherites from PhysBoneClass
from ArmaturePhysicsBoneClasses import DynBoneClass				#Inherites from InternalPhysBoneClass
from ArmaturePhysicsBoneClasses import SpringBoneClass			#Inherites from DynBoneClass
from ArmaturePhysicsBoneClasses import SpringBoneRubberClass	#Inherites from SpringBoneClass
from ArmaturePhysicsBoneClasses import SpringBoneHingeClass		#Inherites from SpringBoneClass

########## Creating some graphical objects for monitoring and forces #############################

APG.SetupForceGraphics()	
Redraw()					#Shows arrows immediately (if the layers in question are selected as visible)

###########################################################################################################################
########################### Initializing hold bones  ######################################################################

def InitializeHoldBones(PhysOb,PhysBones):
	#The argument PhysBones is only used to determine whether the hold bone affects the entire object by a net force or not.
	#If the hold bone parent is in PhysBones - no net force is assumed.
		
	# Looking for hold bones in the armature, storing exsting targets in a dictionary HoldBoneDict 
	
	HoldBoneDict={}
	for item in PhysOb.pose.items():
		name=item[0]
		if name[0:4]=='Hold':
			
			targetName=name[4:]
			
			targetExists=False
			for item2 in PhysOb.pose.items():
				name2=item2[0]
				if name2==targetName:
					targetExists=True
			
			if targetExists:
				HoldBoneDict[name]=name[4:]			#The name of the targeted bone.
			else:
				Draw.PupMenu(name + ": Hold bone's target not found in armature. Omitting hold bone.")
		
	# Adding holdbones to branches 
	
	PhysOb.HoldBoneDict={}			#This dict is just used to keep track of if there are any hold bones at all (not implemented for dynamicFast flag)
	
	counter=0
	for branch in PhysOb.DynBoneBranches.BoneDictList:
		branch.HoldBoneDict={}
			
		for name in HoldBoneDict:
			targetName=HoldBoneDict[name]
			
			if targetName in branch.Bones or PhysOb.dynAndObjectFlag:		#Even if the branch is empty, the object can still have physical bones with hold bones constraints, hence the "or dynAndObjectFlag".
				PhysOb.HoldBoneDict[name]=name[4:]
				counter+=1
				
				parent=PhysOb.arm_data.bones[name].parent
				
				#Setting up the components of the HoldBone dict for the branch in question:
				#[target holdbone, hold bone parent,     hold bone location, initialDeltaToHold vector, scaleY, non dyn parent name, non dyn parentMat].				
				
				if parent==None:
					branch.HoldBoneDict[name]=[targetName,None,None,None,None,None,None]		
				elif not parent.name in branch.Bones:
					branch.HoldBoneDict[name]=[targetName,None,None,None,None,parent.name,1]
				else:
					branch.HoldBoneDict[name]=[targetName,parent.name,None,None,None,None,None]
					print "Holdbone parent in branch. Holdbone will give no net force on the entire object (only internal forces)."
					
					#Comment: Currently the parentMat slot is not used for the last case (the stored bone matrix of the parent is used in stead).
					#Really, one should make a hold bone class of course, since this list is very non-readable.
		
	print "Hold bones: ",counter
				
###########################################################################################################################
########################### Initializing torque dictionaries ###############################################################

def InitializeTorques(PhysOb):
	#Intialization of torque bones, that work as musle torques at bone joints.
	#Note that these provide only an internal joint torque, no net torque.
	
	TorqueBones={}
	for item in PhysOb.pose.items():
		name=item[0]
		if name[0:6]=='Torque':
			length=PhysOb.arm_data.bones[name].length
			parent=PhysOb.arm_data.bones[name].parent
			try:
				torqueparent=parent.name
			except:
				torqueparent=None
			
			try:
				torqueparentParent=PhysOb.arm_data.bones[parent.name].parent.name
			except:
				torqueparentParent=None

			TorqueBones[name]={ 'length': length,'torqueparent':torqueparent, 'torqueparentParent':torqueparentParent}

	PhysOb.TorqueBones=TorqueBones
	
	print "Torque bones: ", len(TorqueBones)

	
###########################################################################################################################
########################### Initializing force dictionaries ###############################################################

def InitializeForces(PhysOb):
	#Initialization of force bones and force armature force bones.
	#Unlike the Torque bones, these give net forces. 
		
	######################### Looking for force bones in the object itself
	
	ForceBones={}
	for item in PhysOb.pose.items():
		name=item[0]
		
		if name[0:5]=='Force':
			length=PhysOb.arm_data.bones[name].length
			parent=PhysOb.arm_data.bones[name].parent
			try:
				forceparent=parent.name
			except:
				forceparent=None
			
			ForceBones[name]={ 'length': length,'forceparent':forceparent}
	
	PhysOb.ForceBones=ForceBones
	print "Force bones: ", len(ForceBones)
	
	#########################  External force armatures  ##################################################################

	ForceArmatureDict={}
	for ob in Scene.GetCurrent().objects:
		if ob.name[0:5]=='Force' and ob.type=='Armature':		#Comment: One could update to ..=='Force' and ob.name[-len(PhysOb.name),-1]==PhysOb.name. That way we can use different force armatures for different characters.
			ForceArmatureDict[ob.name]={}
			
			for item in ob.getPose().bones.items():
				name=item[0]
				length=ob.getData().bones[name].length
				if name[0:5]=='Force':
					ForceArmatureDict[ob.name][name]={ 'length': length}  
	
	PhysOb.ForceArmatureDict=ForceArmatureDict
	print "Force armatures: ", len(ForceArmatureDict)
	
###########################################################################################################################
################## Initializing main object armature  #####################################################################

def AddBlenderArmatureLinks(PhysOb,ArmatureName):	
	
	Window.EditMode(0)			#Sets object mode. Generally makes things work better. Comment: It may already be set, but it is not time consuming to set it again if so.
	
	try:
		arm_ob=Object.Get(ArmatureName)
	except:
		Draw.PupMenu('object "' + ArmatureName + '" not found. Quitting')
		10/0
	
	PhysOb.arm_ob=		arm_ob
	PhysOb.arm_mat=		arm_ob.matrixWorld		
	PhysOb.arm_data=	arm_ob.getData() 			#Used to get rest length of bones
	PhysOb.pose=		arm_ob.getPose().bones		#Takes into account final transforms, just a pointer
	
	####### Dynamical bones and actions ##################################################
	
	# If there is no action adds an action. Necessary for adding keys to bones during dynamic evolution.
	
	act = arm_ob.getAction()
	if not act: 							# Add a pose action if we dont have one
		print "Creating new action"
		act = Armature.NLA.NewAction()
		act.setActive(arm_ob)		


#############################################################################################################			
#############################################################################################################
########################## Setting up initial bone dictionaries etc   #######################################
	
def InitializeBoneStructure(inParameterDict):		
	Window.EditMode(0)			#Sets object mode. Generally makes things work better. Comment: It may already be set, but it is not time consuming to set it again if so.
	
	################################################################################
	######################## Variable settings #####################################
		
	ParameterDict={
		
		'ArmatureName':'Armature',
		'g':10,
		'dynamicFastFlag':False,
		'DeltaT':1.0/25,
		'gravity':True,
		'dampMode':2,
		'objectBody': False,		#True: Object evolution is made by translating and rotating the armature object.
									#False: Object evolution is made by by translating and rotating bodybone (in object pose space).
		'bodyBoneName':'BODYBONE',
		'WarningFlag': True,			#True: Warnings are printed to standard out
		'branchFlag':False,				#True: Put a new separate boneGroup (branch) whenever a dynamical/spring bone has no dynamical/spring parent.
		'dynAndObjectFlag': False,
		'dynInterpolateAccFlag': False,	#True: Rotational acceleration (omega_dot) is used in the interpolation step for the dynamical bones initial rotations.
	
		############ The bone dictionaries #########################################
		
		'BoneDict':	{},
		'SimpleBoneDict':{},
		'DynamicBoneDict':{},
		'SpringBoneDict':{}	
			}		

	# Replacing default values and making local variables of the modified ParameterDict keys.
	
	ModifyDefaultDict(ParameterDict,inParameterDict,'InitializeBoneStructure')	#Substitutes parameters from inParameterDict to ParameterDict, if the type matches the type of the original ParameterDict

	for varname in ParameterDict:
		string=varname +"=ParameterDict['" + varname + "']"
		exec(string) 

	#Initiating the base object for bones et cetera.
	
	######################################################################################
	############# Blender specific initiation  ###########################################
	
	PhysOb=PhysObjectClass()
	AddBlenderArmatureLinks(PhysOb,ArmatureName)
			
	arm_data=PhysOb.arm_data		#For brevity
	pose=PhysOb.pose				#For brevity
	
	
	######################################################################################
	########### A check-up  (setting a flag) #############################################
	
	PhysOb.forbidObject=False		#A flag indicating that, if objectBody=False and no valid bodyNone is given, no object evolution will be possible. 
	PhysOb.existBodyBone=False		#A flag to indicatre if a valid bodyBone exists.
	if objectBody==False:			#Then we need to insure that there is a bodybone
		PhysOb.existBodyBone=True
		try:		
			arm_data.bones[bodyBoneName].length		#Just to check if the bodyBone is in the armature.
		except:
			objectBody=True				#This needs to be set even to make pure dynamical evolution.
			PhysOb.forbidObject=True	#When this is true - object evolution will be dissalowed.
			print "BODYBONE: "+bodyBoneName+" not found in armature. Since also objectBody=False, object evolution will not work."
			PhysOb.existBodyBone=False
		
	#############################################################################################################
	############# Initiating dictionaries of physical bones, and other initiations  #############################
	
	try:
		AddSimpleBones(SimpleBoneDict,BoneDict,arm_data)		#Writes to BoneDict (overwrites if bone already exists)
	except NameError:
		do=None
	
	#Note that PhysBones contains every DynBone, and DynBones contains every SpringBone
	#Of either Hinge or Rubber type. So for example, PhysBones will not in general only include bones of class PhysBone
	#but also of the classes that inherite from PhysBone.
			
	#Initiating bone dictionaries
	
	PhysBones={}
	DynBones={}
	SpringBones={}
	PhysObjectBones=[]		#Initializing a list containing all the bones that are counted as concerns the object mass, center of mass and angular momentum
	
	for name in BoneDict:
		
		#Default settings 
		
		dict={'mass':1,	'masscenter':Vector(0,0.5,0), 'inertiamatrix':Matrix([1,0,0],[0,1,0],[0,0,1]), "Object": True}
		inDict=BoneDict[name]	
		ModifyDefaultDict(dict,inDict,'Bone initialization loop')		#Comment: slightly suboptimal to send the bones that come from SimpleBones (they should be alright already)
		
		if dict['Object']:
			PhysObjectBones.append(name)
		
		try:		
			arm_data.bones[name].length
		except:
			print "Physical bone",name,"not found in armature: Omitting"
			continue
		
		### Updating the the bone dictionaries with linked bone instances ################################	
		
		okFlag=True
		if objectBody==False and PhysOb.existBodyBone and name==bodyBoneName and (name in DynamicBoneDict or name in SpringBoneDict):
			okFlag=False
			Draw.PupMenu("Active BodyBone found in dynamic/spring bones. Removed from dynamic/spring bones.")
			
		if (name in DynamicBoneDict or name in SpringBoneDict) and okFlag:
			if name in SpringBoneDict:
				
				###### Spring bone specific settings
				
				if 'type' in SpringBoneDict[name]:
					if SpringBoneDict[name]['type']=='Hinge':
						b=SpringBoneHingeClass()
					else:
						b=SpringBoneRubberClass()		#So this is the default
				else:
					b=SpringBoneRubberClass()			#If there is no type - the type is set to default
				
				###### Setting the damping mode
				
				if 'rubberNewDampFlag' in SpringBoneDict[name]:
					if SpringBoneDict[name]['rubberNewDampFlag']:
						b.rubberNewDampFlag=True
					else:
						b.rubberNewDampFlag=False	#The old method of damping based on kind of internal damping in a (waterhose) rubber joint.
				else:
					b.rubberNewDampFlag=True		#Letting new method overrule old if nothing is said.

				#Setting up the spring vector
				
				if 'springVecSimple' in SpringBoneDict[name]:
					if 'springVec' in SpringBoneDict[name]:
						print name, ": Overwriting standard springVec"
					simpleVec=SpringBoneDict[name]['springVecSimple']
					b.springVec=getSpringVec(simpleVec,dict['inertiamatrix'],dict['masscenter'],dict['mass'],DeltaT)
				elif 'springVecElastic' in SpringBoneDict[name]:
					b.springVec=getSpringVecElastic(SpringBoneDict[name]['springVecElastic'],arm_data.bones[name])
				else:
					b.springVec=SpringBoneDict[name]['springVec']
				
				#Setting up the damp vector
				
				if 'dampVecSimple' in SpringBoneDict[name]:
					if 'dampVec' in SpringBoneDict[name]:
						print name, ": Overwriting standard dampVec"
					simpleVec=SpringBoneDict[name]['dampVecSimple']
					b.dampVec=getDampVec(simpleVec,dict['inertiamatrix'],dict['masscenter'],dict['mass'],DeltaT)
				else:
					b.dampVec=	SpringBoneDict[name]['dampVec']
				
				#General
				
				if 'offsetTorqueFlag' in SpringBoneDict[name]:			
					b.offsetTorqueFlag=SpringBoneDict[name]['offsetTorqueFlag']
					
				b.springChildren=[]
				list=arm_data.bones[name].children
				for childbone in list:
					if  (childbone.name in SpringBoneDict) and (   (childbone.name in BoneDict) or (childbone.name in SimpleBoneDict)   ):
						b.springChildren.append(childbone.name)
							
				SpringBones[name]=b
				
			else:
				b=DynBoneClass()
				
				
			# Adding joint constraints. And stuff related to "keepKeyedRotations"
			# Example. If constrainedVec=Vector(1,0,1) then all dynamic rotations except about the bone local y-axis is frozen.				
			
			b.hasConstraint=False
			b.hasFreedom=True
			b.isRigid=False
			
			
			vec=None			
			if (name in SpringBoneDict) and ('constrainedVec' in SpringBoneDict[name]):
				vec=SpringBoneDict[name]['constrainedVec']		
			elif (name in DynamicBoneDict) and ('constrainedVec' in DynamicBoneDict[name]):
				vec=DynamicBoneDict[name]['constrainedVec']
			else:
				vec=None
				b.freeIndexes=[0,1,2]
				b.constrainedIndexes=[]
			
			
			keepKeyedRotations=False	
			if (name in SpringBoneDict) and ('keepKeyedRotations' in SpringBoneDict[name]):
				if SpringBoneDict[name]['keepKeyedRotations']:
					vec=Vector(1,1,1)
					keepKeyedRotations=True
						
			elif (name in DynamicBoneDict) and ('keepKeyedRotations' in DynamicBoneDict[name]):
				if DynamicBoneDict[name]['keepKeyedRotations']:
					vec=Vector(1,1,1)
					keepKeyedRotations=True
			
			b.keepKeyedRotations=keepKeyedRotations 
			
			#Comment: Note that if the keepKeyedRotations option is true,
			#the constrainedVec is overwritten to be (1,1,1) regardless of other settings of constrainedVec.


			if not vec==None:
				b.freeIndexes=[]
				b.constrainedIndexes=[]
				
				for i in [0,1,2]:
					if vec[i]<0.5:			#Just to deal with any numerical issues.			
						b.freeIndexes.append(i)
					else:
						b.constrainedIndexes.append(i)
				
				if b.constrainedIndexes!=[]: 
					b.hasConstraint=True
				
				if b.freeIndexes==[]:
					b.hasFreedom=False
			
			
			if not keepKeyedRotations and b.hasFreedom==False:
				b.isRigid=True
			
			###### Dyn bone specific settings
			
			b.length=arm_data.bones[name].length
			b.restLocalMatrix=arm_data.bones[name].matrix['BONESPACE']		#The restspace matrix (3x3) relative to parent (constant matrix)
	
			if not pose[name].parent==None:
				b.parent=pose[name].parent.name
			
			b.dynChildren=[] 									#Making a real copy. Deep copying the entire b does not work.		
			list=arm_data.bones[name].children
			for childbone in list:
				if  (childbone.name in DynamicBoneDict or childbone.name in SpringBoneDict) and childbone.name in BoneDict:					
					b.dynChildren.append(childbone.name)
			
			DynBones[name]=b
		else:
			b=PhysBoneClass()
			
			
		PhysBones[name]=b
			
		###### Setting some properties common to all PhysBones ####################
									
		b.Ob=PhysOb
		b.mass=dict['mass']
		b.name=name
		b.inertiamatrix=dict['inertiamatrix']	
		b.crel=dict['masscenter']
		
		if b.mass==0:
			Draw.PupMenu(name+': bone mass is zero. Quitting.')
			print name,': bone mass is zero. Quitting.'
			10/0
	
	#############################################################################################################	
	####################################  Checkups  #############################################################
	
	if PhysOb.existBodyBone and dynAndObjectFlag and len(DynBones)>0:
		if not bodyBoneName in PhysObjectBones:
			print "Warning: When there are dynamical bones and dynAndObjectFlag=True, having a bodyBone that is not a physical object bone, can be a bad idea."
			#Comment: For example consider a bodyBone and a single dynamical bone that is the child of the bodyBone.
			#Suppose further that there is no spring tension at the head of the dynamical bone.
			#Then rotating the bodybone about the head of the dynamical bone has no physical relevance. 
			#When solving a linear matrix system for the rotation of the bodyBone - it is then no wonder if things go to heck.


	for name in DynBones:
		ncon=PhysOb.pose[name].constraints.__len__()
		if not DynBones[name].keepKeyedRotations:
			for index in range(ncon):
				con=PhysOb.pose[name].constraints.__getitem__(index)
				if con.influence>EPSILON:	
					if DynBones[name].freeIndexes==[0,1,2]:
						Draw.PupMenu('Non-zero Blender constraints on unconstrained dynamical bone' + name + '. Fix and get back. Quitting')
						print "Warning: ", name, " has nonzero blender constraints. Unless these match the dynamical constraints things can go to heck."
						10/0
					else:
						print "Warning: ", name, " has nonzero blender constraints. Unless these match the dynamical constraints things can go to heck."
				else:
					print "Warning: ", name, " has blender constraints. So long as the influence remains at 0 throughout the simulation, this is no problem however."

	
	if dynAndObjectFlag and branchFlag:
		branchFlag=False
		Draw.PupMenu("Changing branchFlag to false since dynAndObjectFlag is True.")

	
	existOffset=False
	for name in SpringBones:
		if SpringBones[name].offsetTorqueFlag:
			existOffset=True	
	if existOffset and not gravity:
		Draw.PupMenu("gravity=False at initialization, 'offsetTorqueFlag'=True will have no effect.")
				
	
	for name in BoneDict:
		if not ((name in PhysObjectBones) or (name in DynBones)):
			print "Bone:",name," is currently not playing any role in the evolution"
	
	
	for name in SpringBoneDict:
		if name not in BoneDict:
			print "SpringBone",name,"not found in BoneDict - has been omitted"
	
	
	for name in DynamicBoneDict:
		if name not in BoneDict:
			print "DynamicBone",name,"not found in BoneDict - has been omitted"

	##### Checking that the inertia matrix is not too small
	
	for name in DynBones:
		b=DynBones[name]
			
		for mi in [0,1,2]:
			if b.inertiamatrix[mi][mi]<EPSILON:
				print name, " :small inertia matrix component."
	
	##### Checking to see that there are no dynamic bones upstream of non-dynamic bones
	
	for name in PhysBones:
		pbone=arm_data.bones[name].parent
		if not pbone==None:
			pname=pbone.name
		else:
			pname=None
		
		if (pname in DynBones) and (not name in DynBones):
			Draw.PupMenu('Currently dynamic bones may not have physical, non-dynamic children.')
			Draw.PupMenu('Tip: Add the physical bones to DynamicBoneDict and set constrainedVec=Vector(1,1,1). Quitting.')
			print "The current algorithm can not deal correctly with with having dynamic bones"
			print "upstream of non-dynamic physical bones. Quitting"
			10/0
			#When dynamicFastFlag is False, one could in principle allow for this simply by letting all
			#physical bones be of the dynamical type, but one just skips the evolution part (the equation solving)
			#for those pseudo-dynamical bones. This way one would get the correct transport of joint forces in
			#the chain of bones. This seems reasonably simple to set up. It lacks a bit of elegance and efficiency
			#because within any connected goup of pseudo-dynamical bones, there really is no need to consider the 
			#internal forces within that group - only at the 'borders' of the group. 
			#(Unless of course we are also interested in muscle monitoring of the pseudo group).
			#An idea would be to treat the whole pseudo-group as a (non-rigid) bone.
			#From one time step to another, one gets dL/dt and dV/dt for the whole group,
			#and deduces the border forces from that kind of. But note that
			#if there are several branches within the pseudo-group, that each have real dynamical bones 
			#as children - then it is a bit more complex than simply viewing the whole group as a bone,
			#since at the end of the day we need to set the rotation of the dynamical children with respect
			#to each of their actual parents - and not relative to a single group-bone. But actually
			#dealing with this is probably just a matter of a couple of matrix multiplications.
			#So, yeah - it should be doable, but it is not implemented.
			#
			#	
			#When dynamicFastFlag is True, it is less easy to implement the feature.
			#For this case, the bone downstream of a dynamical bone
			#is asumed to move as if hinged (in the Blender sense of hinge), as one varies the
			#rotation of the dynamical bone by some smal angle. But if the downstream bone
			#is not a dynamical bone, but just a physics bone, then the angle between the bones should
			#remain fixed (like a rigid joint) as one rotates the dynamic parent.
			#So the child is inherently not allowed to be hinged. There are workarounds, one could treat any connected
			#group of physical non-dynamical bones as a non-rigid single bone, as discussed above,
			#but there is kind of much to consider before one gets that working (and it will never work
			#with IK-chains and stuff). 
	
	
	
	##### Crude check of numerical instability related to too high spring stiffness, or high damping 
	##### For dampMode=2 - this is irrelevant however, hence one can shut it down using the WarningFlag option.
	
	if WarningFlag:				
		for name in SpringBones:
			b=SpringBones[name]
			
			kmax_advicevec=Vector(0,0,0)
			bmax_advicevec=Vector(0,0,0)
			flag1=False
			flag2=False
			
			for i in [0,1,2]:
				k=b.springVec[i]		#If this is too high - escalating bone jumping can occur
				
				vec=Vector(b.crel)
				vec[i]=0
				s=vec.length			#The distance from the axis of rotation in question to the bone center of mass
				I=b.inertiamatrix[i][i]+b.mass*s*s
				
				kmax_advice=3*I/(DeltaT*DeltaT)
				kmax_advicevec[i]=kmax_advice
				
				if k>kmax_advice:		#The factor of 3 seems to work fairly well
					
					mystring=name+":"+" Num. instab. risk: High springVec["+str(i)+"]/inertMat["+str(i)+"]["+str(i)+"]"
					print mystring
					flag1=True
				
				#Whether it is actually unstable depends also on the downstream bones,
				#but at least this gives some indication
				
				bmax_advice=I/DeltaT
				if b.dampVec[i]>bmax_advice:
					mystring=name+":"+" Num. instab. risk: High dampVec["+str(i)+"]"
					print mystring
					flag2=True
			
				bmax_advicevec[i]=bmax_advice
			
			if flag1:
				string=name+": Max advice springVec="+str(kmax_advicevec)
				print string
				 
			if flag2:
				string=name+": Max advice dampVec="+str(bmax_advicevec)
				print string
	
	#############################################################################################################
	########################### Settings related to spring bones ################################################
	
	# Looking for bones starting with 'Ghost' in the armature of the object itself, and adding them to the SpringBones ghostBone if name matches 
	
	nGhost=0
	poseitems=pose.items()
	for item in poseitems:
		name=item[0]
		if name[0:5]=='Ghost':
			if name[5:] in SpringBones:
				nGhost+=1
				SpringBones[name[5:]].ghostBone=name
			else:
				"Not finding armature ghostBone's corresponding springBone"
	
	print "Ghost bones: ", nGhost
	
	#############################################################################################################
	############## Collecting the bones that are internally affected by ExternalDynForceTorque() ################
	
	InternalPhysBones=InternalPhysBoneDictClass(DynBones,PhysOb)

	#Comment: One could alternatively run ExternalDynForceTorque() for the individual
	#branches I guess. 
	#
	#Comment: There may be more bones than those in DynBones in later versions.
	#Currently this is just used to run SetExternalDynforceTorque
	#The other methods are not called at this stage. They may be in the future though. 
	#If pure internal physbones are parents to dynamical bones, and we want to monitor
	#internal torques for all these bones.
	
	PhysOb.InternalPhysBones=InternalPhysBones
	
	####################################################################################################
	######################### Dividing dynamical bones into branches ###################################	
	
	# Setting up the .allDynChildren list for each dynamical bone.
	# Something similar will be done as an initialization method later as well
	# altough that will be restriced to the bones in the particular boneGroup, and not to all dynamical bones.
	
	for name in DynBones:
		b=DynBones[name]
		b.allDynChildren=[]			#Needs to be done before appending, to unlink the instance
		
		list=arm_data.bones[name].getAllChildren()
		for childbone in list:
			if  childbone.name in DynBones:
				b.allDynChildren.append(childbone.name)
			
	# Separates the DynBones into branches.
	# Takes all the downstream children of any dynamical bone that has no dynamical parent and puts these in a dictionary.
	# All such dictionaries (of separate branches) are collected in a list boneGroupList.
	
	
	#Getting the bones that have nondynamical parents or no parents at all.
	
	topParentList=[]
	for name in DynBones:
		b=DynBones[name]
		if not b.parent in DynBones:
			topParentList.append(name)
	
	#Ordering the list of top parents, so parents come before downstream children
	#This order is preserved for the branch list later, so that
	#evolution of updtream dynamical bones comes before evolution of downstream dynamical bones.
	
	if branchFlag:
		treeList=[]		#Initiating a new list of lists where the the second element says how many dynamical top parents are upstream of the top parent in questions. 
		for name1 in topParentList:
			treeList.append([name1,0])
			for name2 in topParentList:
				list=DynBones[name2].allDynChildren
				if name1 in list:
					treeList[-1][1]+=1
					
		# Sorting the list (there are prettier ways of doing this - in particular in python 2.4 and later.)
		
		max=0
		for item in treeList:
			if item[1]>max:
				max=item[1]
		
		topParentList=[]
		level=-1
		for level in range(0,max+1):	
			for item in treeList:
				if item[1]==level:
					topParentList.append(item[0])
			
	
	# Removing doubles, so that any dynamical children belongs with only one dynamical parent, if branchFlag=True
	
	for name1 in topParentList:
		for name2 in topParentList:
			list=DynBones[name2].allDynChildren		#Just a link. Editing list will affect .allDynChildren
			if name1 in list:
				if not branchFlag:
					Draw.PupMenu('Non-dynamical bone(s) breaks dyn/spring chain. Either add bone(s) to DynamicalBonesDict or set branchFlag=True. Quitting')
					10/0
					#Comment: In principle one can certainly allow for this. In particular if all the breaking (non-dynamical) bones
					#are physical, then they could be treated as dynamical but with keepKeyedRotations=True.
					#Also if they are not physical, they can be treated treated as superlight extensions
					#of the first parent bone. This can also be done,  but for the dynFast=True option 
					#one needs a more serious code addition if one wants to allow for these bones
					#to be non-rigid in relation to the first dynamical parent.
				else:
					print ""
					print "Non-dynamical bone(s) breaks dyn/spring chain. Children will not interact with parents through nondynamical bones."
					#Comment: When childrens mass are small compared to the parents mass,
					#this can work well.
				
				if branchFlag:
					list.remove(name1)
					for name3 in DynBones[name1].allDynChildren:
						list.remove(name3)
	
	
	#Setting up a list of bone groups
	
	PhysOb.gravity=gravity		#For gravity offset torques. This will be set again inside MainEvolve. Comment: note that these will remain if one turns of gravity since this is only in initialization.
	PhysOb.g=g					#For gravity offset torques. Comment: note that these will remain if one turns of gravity since this is only in initialization.
	
	boneGroupList=[]
	
	if not branchFlag:
		
		#Setting up all dynamical bones in one boneGroup if possible
		
		type=None
		for name in DynBones:
			if not type==None:
				if type!=DynBones[name].type:
					Draw.PupMenu('Purely dynamical and spring/damp bones mixed in one boneGroup. Not allowed currently. Quitting.')
					10/0
					
			type=DynBones[name].type
		
		if type=='SpringBone':
			boneGroup=SpringBoneDictClass(DynBones,PhysOb)	
		else:
			boneGroup=DynBoneDictClass(DynBones,PhysOb)
		
		boneGroup.Bones=DynBones
		boneGroupList=[boneGroup]

	else:
		
		#Setting upp several boneGroups, one for every DynBone that has no DynBone parent. 
		
		for name in topParentList:
			b=DynBones[name]
			subDict={}					#Initiating branch dictionary
			subDict[name]=b				#First bone of the branch (the dynamical top parent)
			
			for childName in b.allDynChildren:
				childBone=DynBones[childName]
				if childBone.type!=b.type:
					Draw.PupMenu('Purely dynamical and spring/damp bones mixed in a single branch. Not allowed currently. Quitting.')
					10/0
				
				subDict[childName]=childBone
			
			if isinstance(b,SpringBoneClass):			#Comment: isinstance should be highlighted by editor
				boneGroup=SpringBoneDictClass(subDict,PhysOb)	
			else:
				boneGroup=DynBoneDictClass(subDict,PhysOb)
			
			boneGroup.Bones=subDict
			boneGroupList.append(boneGroup)
	
	
	# Adding boneGroup as reference to the individual bones, and some constraint related issues
		
	for boneGroup in boneGroupList:
		subDict=boneGroup.Bones
		
		# Adding branch as boneGroup for the bones in the branch. Can be used to reference the branch's dampMode from the individual bone methods.
		
		for bonename in subDict:
			boneGroup.Bones[bonename].boneGroup=boneGroup 	#Links the branch object to the individual bone.
		
		# Checking that there are at least one degree of freedom for the entire branch
		
		hasFreedom=False
		hasConstraints=False
		
		for name in boneGroup.Bones:
					
			if boneGroup.Bones[name].freeIndexes!=[]:
				hasFreedom=True
			
			if boneGroup.Bones[name].hasConstraint:
				hasConstraints=True
		
		boneGroup.hasFreedom=hasFreedom
		boneGroup.hasConstraints=hasConstraints
		
		if boneGroupList[0].Bones=={}:
			print "No dynamical bones"
		else:
			if not hasFreedom:
				Draw.PupMenu("No degrees of freedom in dynamic Branch. Keyed states will be preserved even thought they should perhaps be overwritten by rigid joint constraints.")
				#Comment: The thing is that there are matrixes going to be empty and stuff when there are no variations. 
				#Can be fixed so that rigid joints are overwriting existing keys, but for the moment I'll let it be.
				print ""
				print "ALL BONES IN BRANCH ARE FULLY CONSTRAINED. KEYED STATES WILL BE PRESERVED"
			
	print "Bone groups: ",len(boneGroupList)			

	
	#########################################################################################
	#################### The object settings ################################################
	
	M=0
	N=0
	for name in PhysObjectBones:
		N+=1
		M+=PhysBones[name].mass	
	
	print "Dynamical bones:", len(DynBones)
	print "Object bones:", N
	print "Mass: ", M
	
	if M<EPSILON:
		Draw.PupMenu('Zero net mass. Fix and get back. Quitting')
		print 'Zero net mass. Fix and get back. Quitting.'
		10/0
	
	######## BodyBone rest matrix 
	
	if PhysOb.existBodyBone:
		bodyBoneRestMat=Matrix(arm_data.bones[bodyBoneName].matrix['BONESPACE'])	#The restspace matrix (3x3) relative to parent.
		bodyBoneRestMat.resize4x4()
		bodyBoneRestMat[3][0:3]=arm_data.bones[bodyBoneName].head['BONESPACE']
		
		try:	
			bodyBoneRestMat[3][1]+=arm_data.bones[bodyBoneName].parent.length
		except:
			pass
			
		#Comment: The thing is that, if the bodyBone has a parent, the bodyBone restmatrix should be relative
		#to the parent rest space. And the parent space sits at the head of the parent, 
		#whereas the head['bonespace'] gives the vector from the tip (tail) of the parent 
		#to the bodybone. So we must add a vector corresponding to the length of the parent.
	
	######## The bones that are counted as part of the object (affecting object center of mass and angular momentum)
	
	PhysObjectBoneCollection={}
	for name in PhysObjectBones:
		PhysObjectBoneCollection[name]=PhysBones[name]		
	
	
	#########################################################################################
	#########################################################################################
	###########    Transferring variables to PhysOb attributes ##############################
	
	#Settings relating to individual physical bones
	
	PhysOb.bodyBoneName=bodyBoneName
	PhysOb.objectBody=	objectBody				
	
	#Other settings 
	
	PhysOb.M=M
	PhysOb.forceFactor=M*g
	PhysOb.AllPhysBones=PhysBones				#A dictionary of links to all physical bones (including dynamical bones and springbones) connected to the object
	if PhysOb.existBodyBone:
		PhysOb.bodyBoneRestMat=bodyBoneRestMat
	
	PhysOb.deltashift=0.001			#Base: 0.001. For the derivative with respect to shifts. Comment 0.0001 is to small - gives visible numerical errors. And 0.01 can yield errors because the disp is too large. But 0.001 can yield errors (matrix Kmat in IterateLocRot1 gets error of the order of 1/1000) also.
	PhysOb.deltaangleOb=0.001		#Base: 0.001. For the derivative with respect to rotations of object.
	
	#Comment: Not sure whether dynAndObject belongs in the initialization parameter dict (as it is currently)
	#or in the main parameter dict. But for the moment we leave it in initialization. 
	
	PhysOb.dynAndObjectFlag=dynAndObjectFlag
		
	#Adding the bones used to calculate object center of mass and angular momentum
	
	PhysOb.ObjectBones=PhysBoneDictClass()
	PhysOb.ObjectBones.Ob=PhysOb		#Comment: Looks odd, but this circular reference is intentional
	PhysOb.ObjectBones.M=M				#Comment. Both PhysOb.M and ObjectBones.M exists. 
	PhysOb.ObjectBones.Bones=PhysObjectBoneCollection	
		
	#Adding the dynamical branches 
		
	PhysOb.DynBoneBranches=DynBoneBranchesClass()			#Comment: Poor notation .DynBoneBranches. Would be better with dynBoneGroups.
	PhysOb.DynBoneBranches.BoneDictList=boneGroupList		#Comment: Poor notation .BoneDictList. Would be better with .boneGroupList.
	
	for index,branch in enumerate(PhysOb.DynBoneBranches.BoneDictList):
		branch.branchIndex=index
		branch.dynInterpolateAccFlag=dynInterpolateAccFlag
	
	# Initializing forces

	InitializeForces(PhysOb)
	
	# Initializing muscle joint torques
	
	PhysOb.torqueFactor=PhysOb.forceFactor	#So the torque of PhysOb.M*PhysOb.g times one blender unit corresponds to a non-rescaled torque bone of unit length.
	InitializeTorques(PhysOb)

	# Initializing hold bones
	
	InitializeHoldBones(PhysOb,PhysBones)		#Must be called after branches are added.
	
	# Returning the initialized physical object.
	
	return PhysOb


##################################################################################################
##################################################################################################
##################################################################################################

def MainEvolve(inParameterDict,PhysOb):
	#The main function of evolution both for pure object evolution, pure dynamical evolution and
	#the combination of the two.
	
	Window.EditMode(0)				#Sets object mode. Necessary for good operation. It may already be set, but it is not time consuming to set it again.
	initframe=Get('curframe')		#Just to go back to the current frame at the end
	TIME0=time.clock()				#Just for benchmarking	
	
	######################################################################################################
	#################### Variable settings  ##############################################################
	
	ParameterDict={
		
		############ Defining frames et. cetera ####################################

		'FrameStartEnd':[3,10],
		'DeltaT':1.0/25,			#The time interval in between frames - positive even if evolution is backwards in time.
		
		########### Gravity 
		
		'gravity':True,				#True - turns on gravity
		'g':10,						#Earth gravitational acceleration: 9.81 is good, 10 is for testing
		
		########## Steps
		
		'stepsPerFrame':1,			#Number of steps per frame
		
		############ Settings for object evolution
		
		'accuracyC':0.01, 			#Relative time step accuracy (how large the relative discrepancy may be between F and M*d2C/dt2)
		'accuracyL':0.01, 			#Relative time step accuracy (how large the relative discrepancy may be between tau and dL/dt)		
		'accuracyCabs':0.00001, 	#Absolute accuracy, center of mass 
		'accuracyLabs':0.0001, 		#Absolute accuracy, angular momentum

		'maxiter':1,				#Object maximum iteration. Minimum value is 1
		'objectForce':True,			#True: Evolve object position and rotation
		'boneforce':True,			#True: Activates bone force
							
		############ Settings for dynamic bones evolution
		
		'defaultIterationMode':1,	#1 is standard. It will automatically change to 2 for the second iterartion.
		
		'dynamicNiteration':1,		#Number of dynamic iterations. Lowest value: 1, seems to work fine typically. Sometimes 2 works a good del better with dynamicDerivative.
		'dynamicFastFlag':False,	#True: Assumes that the children bones can move as if hinged, giving faster calculation
		'dynamicFlag':True,			#If false - does no dynamic bone evolution 
		'dynamicDerivative':False,	#True: Dynamical evolution performed in the object derivative steps.
									#Very useful if the dynamical bones have more mass and inertia moment
									#than a couple of percent of the rest of the armature.
									#Slow but accurate.
									#Overrules the mode=1 default iteration mode and goes directly to mode=2.
		
		'dampMode':2,				#0 - uses damping from 1.5 frame before current. Fast. Only option for DynamicFastFlag=True.
									#1 - uses damping from 1 frame before current. Not so fast. But most accurate option
									#2 - uses damping from 0.5 frames before current. Not so fast. But probably the most numerically stable option for strong damping
		
		'almostRigidFlag':	False,		#True: Approximates constant eqution system for small oscillations.
		'updateAlmostRigidFlag': False,	#True: Automatically updates constant equation system as evolution starts 
		'plainBranchFlag': False,   	#If the dynamical system is plain, so there are for example no IK constraints, that affect the
										#dynamical evolution. Then (for dynamicFast=False, dampMode=2) one can speed up the evolution by
										#setting plainBranchFlag=True, which insures that no unnecessary damping calculations are performed.
		
		########### Other settings
		
		'monitorCM': True			#Monitors the center of mass as the evolution proceeds
			}
	
	
	# Replacing default values

	ModifyDefaultDict(ParameterDict,inParameterDict,'MainEvolve')	#Substitutes parameters from inParameterDict to ParameterDict, if the type matches the type of the original ParameterDict
	
	# Making local variables of the modified ParameterDict keys.
	
	for varname in ParameterDict:
		string=varname +"=ParameterDict['" + varname + "']"
		exec(string) 

	
	######################################################################################################
	#################### Settings before the evolution starts #############################################
		
	PhysOb.gravity=gravity
	PhysOb.dynamicFlag=dynamicFlag
	PhysOb.boneforce=boneforce
	PhysOb.dynamicDerivative=dynamicDerivative
	
	for branch in PhysOb.DynBoneBranches.BoneDictList:
		branch.dampMode=dampMode
		branch.dynamicNiteration=dynamicNiteration
		branch.dynamicFastFlag=dynamicFastFlag
		branch.plainBranchFlag=plainBranchFlag 
			
		if branch.dynamicFastFlag and branch.hasConstraints:
			Draw.PupMenu("DynamicFastFlag=True cannot (currently) be used when there are dynamical joint constraints. Returning.")
			#Comment: One can fix it so that one can have DynamicFast in combination with totally constrained (user keyed)
			#bones, by treating any connected group of keyed bones as a single bone whose group angular momentum
			#about the group mass is the "bone" spin etc. Note however that the children to this groupBone, will not have 
			#fixed positions - so, presumably one would have to recalculate massvectors at every time step.
			#As concerns partially constrained joints one can, for spring bones, fake the constrained directions of the
			#joint by asserting a large spring/damp component in that direction. 
			#But currently there is no TorqueMatrix/deltalist reduction for the dynamicFastFlag option,
			#so that would have to be set up first. Working with spring vectors to fake constrained joints seems a bit clumsy,
			#although it might work like a charm. For now that will have to rest though. 
			return
		
		if branch.dynamicFastFlag and branch.HoldBoneDict!={}:
			Draw.PupMenu("DynamicFastFlag=True cannot (currently) be used when there hold bones. Returning.")
			return
		
		
	PhysOb.stepsPerFrame=stepsPerFrame
	
	###################################################################################################################
	################## Initializations related to the graphical scene #################################################
	
	# Adding a link to a graphical object that tracks the center of mass
	
	if monitorCM:
		try:
			CM=Object.Get('CM')
			CMdot=Object.Get('CMdot')
			xyVelocity=Object.Get('xyVelocity')
			zVelocity=Object.Get('zVelocity')			
			cmFlag=True
		except:
			Draw.PupMenu("Missing 'CM' related object, center of mass monitoring will be cancelled")	
			cmFlag=False
	
	# Going to (if possible) the linked (faster) scene.
			
	try:
		scn=Scene.GetCurrent()
		ascn=Scene.Get('ArmPhysScene')
		ascn.makeCurrent()
		sceneFlag=True
	except:	
		print "For complex scenes, you can get faster evolution if you create a new linked scene within the same blender file, called 'ArmPhysScene', from which you delete everything except the armatures that concerns this script."
		modDict=APG.DisableMod()
		sceneFlag=False
	
	#Comment: The thing is that by creating a linked scene called ArmPhysScene, and deleting everything
	#except the armatures related to this script, from that scene - one
	#can speed up the evolution (by a lot for many cases). 
	
	
	####################################################################################################################
	####################### Checkups before the evolution starts #######################################################
	
	if PhysOb.forbidObject and objectForce:
		Draw.PupMenu('No bodyBone found in armature, and objectBody=False. Object evolution is disabled. Quitting')
		10/0
		
	if PhysOb.objectBody==False and objectForce:
		if Armature.CONNECTED in PhysOb.arm_data.bones[PhysOb.bodyBoneName].options:
			Draw.PupMenu('The bodyBone is connected to its parent. Disconnect it before proceeding, or choose different bodyBone.  Quitting')
			10/0
		
		ncon=PhysOb.pose[PhysOb.bodyBoneName].constraints.__len__()
		
		for index in range(ncon):
			con=PhysOb.pose[PhysOb.bodyBoneName].constraints.__getitem__(index)
			if con.influence>EPSILON:		
				Draw.PupMenu('Non-zero Blender constraints on bodyBone' + PhysOb.bodyBoneName + '. Fix and get back. Quitting')
				print "Warning: bodyBone", PhysOb.bodyBoneName, " has nonzero blender constraints.  Not a good idea currently. Quitting."
				#In principle one could have constraints for the body bone - but there is currently no support for that.
				10/0
			else:
				print "Warning:  bodyBone", PhysOb.bodyBoneName, " has blender constraints. So long as the influence remains at 0 throughout the simulation, this is no problem however."
		
		
	if objectForce==False and dynamicFlag and stepsPerFrame>1:
				print "Heads up: If the object/bodybone moves (even with constant velocity),"
				print "or if the internal keyed (non-dynamic) parents of the dynamic bones are animated,"
				print "then dynamic evolution (without object evolution) is likely"
				print "more accurate with stepsPerFrame=1."
			
				#Comment: The thing is that when there are several steps in between frames,
				#the object/bodybone is currently effectively at rest relative to world space in between frames,
				#and then at the step before the new frame  - it jumps to its new position, to then 
				#remain at rest again. This jerky motion completely screws up the interpolation guesstimation
				#of the rotation of (at least) the top parent dynamic bones. This can however be handled 
				#by cranking up the dynNiteration setting. Still the effects are quite severe,
				#possibly this is physically reasonable. When the velocities starts getting large
				#it is totally ridiculus to stop the motion of the top parent bone for say 9 tenths of a frame and then kick it
				#into really high velocity (ten times the object velocity) for just a step only to halt it again.
				#The relativity principle is in risk of being screwed certainly.
				#A solution is to linearize the object/bodybone position and rotation in between frames.
				#This works like a charm, but as of yet I don't know how to acces 
				#the object IPO without changing frame (which is slow-ish).
				#Also the exact same arguments apply to when there are internal keyed bones in the armature.
		
	if PhysOb.dynAndObjectFlag and not (dynamicFlag and objectForce):
		PhysOb.dynAndObjectFlag=False
		Draw.PupMenu("Changing dynAndObjectFlag to False since not both dynamicFlag and objectForce are set.")
	
	if PhysOb.dynAndObjectFlag and dynamicFastFlag:
		Draw.PupMenu('dynAndObjectFlag=True is not supported (yet) for dynFastFlag')
		return
	
	if PhysOb.dynAndObjectFlag and almostRigidFlag:
		print 'dynAndObjectFlag=True in combination with almostRigidFlag=True is not extensively tested.'
	
	#Comment: Poor notation. Should change name objectForce to objectFlag. It ooks like a force otherwise.
	
	###################################################################################################
	############## Initiation before the evolution ####################################################		
	
	framelist=GetFramelist(FrameStartEnd,initframe)
	dt0=GetDt(framelist,DeltaT)
	k=framelist[0]
	dt=dt0*1.0
	
	PhysOb.dt0=dt0
	PhysOb.dt=dt
	PhysOb.startFrame=k
	PhysOb.almostRigidFlag=almostRigidFlag
	PhysOb.updateAlmostRigidFlag=updateAlmostRigidFlag
	
	#Some initiation related to bone updates. UpdateBoneList will contain the names of all the bones that need updating.
	
	PhysOb.UpdateBoneList=[]
	for name in PhysOb.AllPhysBones:
		if (objectForce and (name in PhysOb.ObjectBones.Bones)) or (dynamicFlag and (name in PhysOb.InternalPhysBones.Bones)):
			#Comment: InternalPhysBones may be a somewhat larger set than is necessary.
			#Maybe one should just have PhysOb.AllDynBones (a dictionary - not an object) or something.
			PhysOb.UpdateBoneList.append(name)
		
	# Resetting stuff ###########################################
	
	#Explanation: If this module is not reloaded and boneforce was True on the last run of MainEvolve, 
	#but False now - then we must reset the external forces and torques to zero (to avoid writing zero vectors to all bones at all frames - better to do it just once here).
	#Commment: Currently this should not be necessary, because the forces are reset at the onset of SetExternalDynForceTorque.
	#However, it may be that in future versions one only resets those bones with force bones,
	#and since it is fast, we keep the zeroing here just to be safe.
	
	for name in PhysOb.InternalPhysBones.Bones:
		b=PhysOb.InternalPhysBones.Bones[name]
		
		b.forceExternal=Vector(0,0,0)
		b.torqueExternal=Vector(0,0,0)
	
	
	#############################################################################################################
	#############################################################################################################	
	################  Initial conditions of positions velocities etc. ###########################################
	
	# 2 frames before start frame
	
	Set('curframe',k-2*sign(dt))					#If frame does not exist - effectively sets 'curframe' to 1
	PhysOb.iframe=k-2*sign(dt)						#For diagnosis, and to shut of warning flag about big angles for the initial steps
	jloop=0
	PhysOb.jloop=jloop
	
	#Comment: Below, it is slightly redundant to make complete updates - for example updates torques for dynamical bones which is unnecesary here, but never mind
	
	PhysOb.UpdateAppropriateBones()
	
	if objectForce:
		C=PhysOb.ObjectBones.GetC()
		PhysOb.update(C)							#Updates object properties related to the guesstimation of position and rotation for the next step
	
	# 1 frame before start frame
	
	Set('curframe',k-sign(dt))						#If frame does not exist - effectively sets 'curframe' to 1
	PhysOb.iframe=k-sign(dt)						#Just for diagnosis
	jloop=0
	PhysOb.jloop=jloop
	
	if dynamicFlag:									#Comment: Not ideal - can affect the rotation at the start frame. See comment below. 
		PhysOb.DynBoneBranches.SetDynamicKeys()		
	
	if objectForce:
		PhysOb.SetKey()
	
	#Comment: Ideally we could just read the rotations at this frame (store in a bone attribute .quad or whatever).
	#Go to the start frame. Insert keys there. Go back to frame k-sign(dt) and set the rotations that
	#we had written to .quad. But at least this sets some initial keys if the user has not set them,
	#which insures that (by blender IPO intnerpolation) initial conditions are not altered by subsequent dynamic evolution.
	
	#Comment: Below, it is slightly redundant to make complete updates - for example updates torques for dynamical bones which is unnecesary here, but never mind
	
	PhysOb.UpdateAppropriateBones()
	
	if objectForce:
		C0=PhysOb.ObjectBones.GetC()
		PhysOb.update(C0)
	
	Set('curframe',k)
	PhysOb.iframe=k									#For diagnosis
	jloop=0
	PhysOb.jloop=jloop
	
	if dynamicFlag:
		PhysOb.DynBoneBranches.SetDynamicKeys()			
	
	if objectForce:
		PhysOb.SetKey()
	
	if objectForce:
		PhysOb.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
		C=PhysOb.ObjectBones.GetPrelC()
		L=PhysOb.ObjectBones.GetPrelL(C)		
		V=(C-C0)/dt									#Estimated initial velocity midframe before initial frame.
	
	#Setting a flag indicating that first step of dynamic evolution has yet to come.
	
	PhysOb.DynBoneBranches.InitiationBeforeEvolution()
	
	
	#############################################################################################################
	############## The main loop of evolution for both object and dynamical bones ###############################
	
	firstStepFlag=True
	
	for iloop in framelist[1:]:
		
		for jloop in range(stepsPerFrame):
			#Setting up forces and torques on the entire object due to force bones. 
			#Note that we still have not changed frame. And note that the target center of mass is used as the actual center of mass.	
			
			PhysOb.jloop=jloop
			
			if objectForce:
				
				if PhysOb.dynAndObjectFlag and not firstStepFlag:	#At the beginning of the first step - we have yet to make any dynAndObject evolution. So PhysOb.holdWorldForce and PhysOb.holdWorldTorque are undefined (as they should be).
					
					if not PhysOb.HoldBoneDict=={}:
						
						option=False	
						#Comment: One may experiment and set option=True or option=False
						#to get better energy conservation for object evolution when there are hold bones
						#depending on if the almostRigid flag is set or not. In at least one example
						#the False option worked much better, so that is now the default value.
						#The option=True possibility is kept mainly for reference that both options have been considered.
						#The point is that when there are hold bones, there is the option to use the actual C and L, 
						#or the corresponding previous targets for C and L, as the basis for the target C and L for the coming frame.
						#Note that either way, energy conservation is still typically not as good as for pure dynamical evolution.
							
						if option:		
							PhysOb.UpdateLocRot()
							PhysOb.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
							C=PhysOb.ObjectBones.GetPrelC()
							L=PhysOb.ObjectBones.GetPrelL(C)
							
							V1=V+PhysOb.holdWorldForce / PhysOb.M * PhysOb.dt
							V2=(C-PhysOb.C)/dt
							
							V=V1	
						else:					
							C+=PhysOb.holdWorldForce / PhysOb.M * PhysOb.dt * PhysOb.dt
							L+=PhysOb.holdWorldTorque * PhysOb.dt
							V+=PhysOb.holdWorldForce / PhysOb.M * PhysOb.dt
							
					#Comment: Currently, when there are no HoldBones, PhysOb.holdWorldForce is set to zero at the 
					#end of every time step.
					
					#Comment: The thing is that the holdbone forces and torques on the object are not part of 
					#the target object forces and torques for the step one is calculating. They cannot be since they are not yet known.
					#As soon as they are known however, from solving the equation system, we can update the targets to be what they would
					#have been if the hold forces had been known from the start, as regular forces.
					#Later comes an update the targets using the regular external forces and torques.
				
				f,tau=PhysOb.GetForceAndTorque(C)
				
			### Updating before going to current frame
			
			PhysOb.UpdateAppropriateBones()		#This also sets the forces and torques coming from force bones and torque bones
					
			if dynamicFlag:
				PhysOb.DynBoneBranches.UpdateSpringAndDampTorques()
							
			if objectForce:
				PhysOb.update(C)				#For the physics object, updates .loc .mat .v .omega .a and .omegadot according to current arm_mat.
			
			dt=dt0/stepsPerFrame			#Note that this belongs here, because in the initiation (and in the primary updates) dt=dt0
			PhysOb.dt=dt
			
			#Comment: Some of the updates below probably only used when dynAndObjectFlag==True, but never mind for now.
			for branch in PhysOb.DynBoneBranches.BoneDictList:
				for holdName in branch.HoldBoneDict:
					branch.HoldBoneDict[holdName][2]=(PhysOb.pose[holdName].poseMatrix*PhysOb.arm_mat).translationPart()	#Storing hold bone head position (to be used subsequently to calculate the object torque once the hold force is known).
					branch.HoldBoneDict[holdName][4]=PhysOb.pose[holdName].size[1]		#Storing y-scale, which is an indicator of if the hold bone is active or not.
					
					if branch.HoldBoneDict[holdName][5]!=None:		#This is where the parent name is stored if the parent is not a dynamical bone. 
						parentName=branch.HoldBoneDict[holdName][5]
						branch.HoldBoneDict[holdName][6]=(PhysOb.pose[parentName].poseMatrix*PhysOb.arm_mat).rotationPart()
			
			############# Monitoring center of mass, velocity etc.  ################################
			
			#Comment: Note that monitoring is done after the effects of any hold bone forces (and corresponding torques)  are included with the targets - as is proper.
			#Note also that the target values are used, not the "actual" values that are used if one calls the monitor function (without evolution).
			
			if monitorCM and jloop==0:
				if objectForce and cmFlag:
					
					# The small center of mass indicator
					
					CMdot.loc=C
					CMdot.insertIpoKey(4)					#4 means LOCROTSIZE
					
					# The vertical velocity gauge
					
					vFactor=10		#Rescale factor for the velocity indicators 
				
					zVelocity.loc=C
					zVelocity.size=Vector(1,1,V[2]/vFactor)	#Target velocity, midframe before curent - will do for now at least.
					zVelocity.insertIpoKey(4)			#4 means LOCROTSIZE
					
					# The horisontal velocity gauge
					
					mat=Matrix([1,0,0],[0,1,0],[0,0,1])
					hVelocity=V-Vector(0,0,V[2])
					rotvec=RotVec4Vecs(Vector(1,0,0), hVelocity)
					newmat=RotateMat(mat,rotvec)
					
					xyMat=xyVelocity.matrixWorld
					SetRotMat(xyMat,newmat)
					xyVelocity.setMatrix(xyMat)
					xyVelocity.loc=C
					xyVelocity.size=Vector(hVelocity.length/vFactor,1,1)	#Midframe before curent - will do for now at least.
					xyVelocity.insertIpoKey(4)			#4 means LOCROTSIZE
					
					# The cross-hair indicator of center of mass and velocity scale
					
					CM.loc=C							#Using the target C rather than GetC() is fast of course.
					CMMat=CM.matrixWorld
					SetRotMat(CMMat,newmat)
					CM.setMatrix(CMMat)
					CM.insertIpoKey(4)					#4 means LOCROTSIZE
					
			#########################################################
			####### New frame #######################################
			
			if jloop==0:				
				PhysOb.iframe=iloop
				Set('curframe',PhysOb.iframe)					#Changing frame. This automatically updates arm_mat	
			else:
				do=None
			
			# Setting the preliminary rotation and location of the object, and dynamical bones
			
			if objectForce:
				sendMat=PhysOb.InterpolateMat()			#This is the guesstimated world space object/bodybone 3-matrix, based on previous frames
				PhysOb.SetMat(sendMat)					#Preliminary setting setting of object/bodybone rotation matrix
				
				
				#Using hold forces from two steps before current, to guesstimate the force from one step before current, to improve the interpolation.
				
				fHold=Vector(0,0,0)
				if PhysOb.dynAndObjectFlag and not firstStepFlag:
					try:
						fHold=PhysOb.holdWorldForce		#For the first step this is undefined, and if there are no holdBones as well.
					except:
						pass
				
				#Interpolating to find a good starting configuratin for the subsequent equation solving.
				
				PhysOb.SetLoc(PhysOb.RelInterpolateLoc(sendMat,C+(V+(f+fHold) /PhysOb.M*dt)*dt) )	#Preliminary setting of arm_mat.translationPart() based on previous frames.
				
					
			if dynamicFlag:
				PhysOb.DynBoneBranches.SetInterpolatedRot()					#Dynamical bones: sets rotation based on previous frames	
				if PhysOb.objectBody:
					PhysOb.DynBoneBranches.SetDynamicKeys()					#Keys needed despite possibly being just an iteration - otherwise PhysOb.UpdateLocRot destroys the current pose, which is needed (possibly) as the initial pose for the next iteration. 
					#Comment: when objectBody is False, UpdateLocRot() will not destroy non-keyed bone rotations. Then there is no need to key them here.
		
			################### Evolving the object 
			
			if objectForce:
				
				firstStepFlag=False
				
				#Updating targets (excluding any effects of hold bones which are yet to be calculated if present)
				
				V+=f/PhysOb.M*dt
				C+=V*dt
				L+=dt*tau						#The target angular momentum, about the center of mass, at the midstep before frame iframe 	
				
				if PhysOb.dynAndObjectFlag:
					PhysOb.targetC=Vector(C)
					PhysOb.targetL=Vector(L)
					
				#Settings related to error margins for the whole object
		
				GoodEnough= False
				GoingWorseL=False
				GoingWorseC=False
				
				errLmax=max(  accuracyLabs,  MagnitudeVec(dt*tau*accuracyL)  )	
				if errLmax<100*EPSILON:						#To insure that the error limits are not tighter than the numerical accuracy allows. The factor of 100 is just a guess of what might be appropriate.
					errLmax=100*EPSILON
				
				errCmax=max(  accuracyCabs,  MagnitudeVec(f*dt*dt*1.0/PhysOb.M*accuracyC)  )
				if errCmax<100*EPSILON:						#To insure that the error limits are not tighter than the numerical accuracy allows. The factor of 100 is just a guess of what might be appropriate.
					errCmax=100*EPSILON
				
				#Resetting mode and iter
				
				mode=defaultIterationMode
				iter=0
				
				###########################################################################
				######### On-frame iteration of object loc rot and dynamical bones ######## 
						
				while iter<(maxiter-0.5) and GoodEnough==False:
					
					iter=iter+1
					
					if not PhysOb.dynAndObjectFlag:
						if mode==1 and not dynamicDerivative:
							PhysOb.IterateLocRot1(C,L)		#Moves and rotates entire object to try and achieve the target C and L, no backreaction from dynamical bones though.
						else:
							PhysOb.IterateLocRot2(C,L)		#Moves and rotates entire object to try and achieve the target C and L, includes backreaction from dynamical bones if dynamicDerivative is True
							
						PhysOb.UpdateLocRot() 				#Should be necessary here.
					
					if dynamicFlag:
						PhysOb.DynBoneBranches.EvolveDynamics()					#Dynamical bones: sets rotation based on previous frames	
						if PhysOb.objectBody:
							PhysOb.DynBoneBranches.SetDynamicKeys()						#Keys needed despite possibly being just an iteration - otherwise PhysOb.UpdateLocRot destroys the current pose, which is needed (possibly) as the initial pose for the next iteration. 
							#Comment: when objectBody is False, UpdateLocRot() will not destroy non-keyed bone rotations. Then there is no need to key them here.
												
					if maxiter>1:
						
						PhysOb.UpdateLocRot()	
						PhysOb.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
						PrelC=PhysOb.ObjectBones.GetPrelC()
						PrelL=PhysOb.ObjectBones.GetPrelL(PrelC)
		
						errL=MagnitudeVec(PrelL-L)
						errC=MagnitudeVec(PrelC-C)
				
						# Checking convergence. No point in keeping iterating after the numerically achievable accuracy is reached.
						
						if iter>1:
							if errL>errLprev:
								GoingWorseL=True
							if errC>errCprev:
								GoingWorseC=True
								
						errLprev=errL     
						errCprev=errC
						
						# Checking to see if a new iteration is worthwhile
						 
						if (errL<errLmax  and errC<errCmax) or (GoingWorseL and GoingWorseC):
							if (GoingWorseL and GoingWorseC):
								print "Iteration no longer converging"
							GoodEnough=True
						else:
							print 'Frame', PhysOb.iframe, 'Iterating extra, Changing mode',iter-1
							mode=2		#This may be faster when there are constraints acting, or heavy dynamic bones
				
				if iter==maxiter and GoodEnough==False:
					print PhysOb.iframe, "Iteration may not have converged"
				
				PhysOb.UpdateLocRot()
				PhysOb.SetKey()
				
			###### No object force - then just evolve the dynamical bones 
			
			if not objectForce:   #No object force - then do the dynamics here
				if dynamicFlag:			
					PhysOb.DynBoneBranches.EvolveDynamics()
						
					if PhysOb.objectBody:
						PhysOb.DynBoneBranches.SetDynamicKeys()
						#Comment: when objectBody is False, UpdateLocRot() will not destroy non-keyed bone rotations. Then there is no need to key them here.
					
		##### End of in between frame steps
		
		if dynamicFlag and not PhysOb.objectBody:
			PhysOb.DynBoneBranches.SetDynamicKeys()
			
			
	###### The end of the evolution #################################
	
	if sceneFlag:			#Going back to the regular, slow scene
		scn.makeCurrent()
	else:
		APG.EnableMod(modDict)			
		
	Set('curframe',initframe)
	Redraw()
	print "Done in", time.clock()-TIME0, "s"	
			

########################################################################################
########################################################################################
##### Monitoring Object Force/Torque and Bones Force/Torque about center of mass #######

def Monitor(inParameterDict,PhysOb):
	#For monitoring the net apparent force and torque acting on the object, in comparison with the force and torque coming from 
	#target forcebone forces and torques. The ouput is in the form of the scale and rotation of the pins and arrows typically to be found in
	#Blender layer 17. Arrows are for the apparent forces and torques, pins are for the target forces and torques. Red color means force, cyan means torque.
	#The torque is the torque about the object center of mass. Forces are normalized so that a force lentght of one blender unit corresponds to the
	#magnitude of gravitatinal force of the whole object. Torques are normalized so that a torque length of one blender unit corresponds to the
	#magnitude of the torque of the gravitatinal force of the whole object times one blender unit (one meter).
	#Comment: Any net force from hold bones are not included in the monitoring of the target forces currently.
	#This should be simple to add though (during monitoring), by treating hold bones with no parent as force bones.

	TIME0=time.clock()		#Just for benchmarking	
	
	#############################################################################
	#################### Variable settings  #####################################
	
	ParameterDict={
		'monitorForceAtC':False,
		'DeltaT':1.0/25,
		'g':10,
		'gravity':True,
		'torqueFactor':1,
		'FrameStartEnd':[3,10],
		'boneforce':True
			}
		
	# Replacing default values and making local variables of the modified ParameterDict keys.
	
	ModifyDefaultDict(ParameterDict,inParameterDict,'Monitor')	#Substitutes parameters from inParameterDict to ParameterDict, if the type matches the type of the original ParameterDict
	
	for varname in ParameterDict:
		string=varname +"=ParameterDict['" + varname + "']"
		exec(string) 
	
	# Other settings
	
	if FrameStartEnd[0]>FrameStartEnd[1]:
		dt=-DeltaT
	else:
		dt=DeltaT
		
	initframe=Get('curframe')
	
	framelist=GetFramelist(FrameStartEnd,initframe)
	dt=GetDt(framelist,DeltaT)
	k=framelist[0]
	
	# Some object settings 
	
	PhysOb.boneforce=boneforce 
	PhysOb.dt=dt
	PhysOb.gravity=gravity
	PhysOb.g=g
	PhysOb.stepsPerFrame=1	
	
	########## Setting up the objects
	
	ForceArrow=Object.Get('F_Arrow')
	TorqueArrow=Object.Get('T_Arrow')
	F_Target=Object.Get('F_Target')
	T_Target=Object.Get('T_Target')
	CM=Object.Get('CM')
	CMdot=Object.Get('CMdot')
	xyVelocity=Object.Get('xyVelocity')
	zVelocity=Object.Get('zVelocity')
	
	CM.xRay=True
	xyVelocity.xRay=True
	zVelocity.xRay=True
	
	######## Moving to the faster Scene
	
	try:
		scn=Scene.GetCurrent()
		ascn=Scene.Get('ArmPhysScene')
		ascn.makeCurrent()
		sceneFlag=True
	except:	
		print "For complex scenes you can get much faster evolution if you create a new linked scene within the same blender file, called 'ArmPhysScene', from which you delete everything except the armatures that concerns this script."
		modDict=APG.DisableMod()
		sceneFlag=False
	
	########## Initialize - probably initializing more than is necesary - but never mind
	
	cm=[]
	Llist=[]
	boneforcelist=[]
	bonetorque=[]
	
	Set('curframe',k-2*sign(dt))			
	PhysOb.iframe=k-2*sign(dt)				#For diagnosis
	
	PhysOb.ObjectBones.UpdatePhysBones()				
	C=PhysOb.ObjectBones.GetC()
	PhysOb.update(C)							
	
	Set('curframe',k-sign(dt))				
	PhysOb.iframe=k-sign(dt)				#For diagnosis
	
	#Starting the frame before the startframe
	
	PhysOb.ObjectBones.UpdatePrelMatAndC()	#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
	C=PhysOb.ObjectBones.GetPrelC()
	L=PhysOb.ObjectBones.GetPrelL(C)
	
	cm.append(C)
	Llist.append(L)
	boneforcelist.append(Vector(0,0,0))		#Just a dummy in the start
 	bonetorque.append(Vector(0,0,0))		#Just a dummy in the start	
	
	PhysOb.ObjectBones.UpdatePhysBones()
	PhysOb.update(C)
	
	for i in framelist:
		Set('curframe',i)
		PhysOb.iframe=i
	
		PhysOb.ObjectBones.UpdatePrelMatAndC()	#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
		C=PhysOb.ObjectBones.GetPrelC()
		L=PhysOb.ObjectBones.GetPrelL(C)

		F,Tau=PhysOb.GetForceAndTorque(C)		#The force and torque coming from force bones and force bone armatures
		
		cm.append(C)
		Llist.append(L)
		boneforcelist.append(F)
		bonetorque.append(Tau)
		
		PhysOb.ObjectBones.UpdatePhysBones()
		PhysOb.update(C)
		
	#So now we have C,L,boneforcelist,bonetorque in a lists starting one frame before the first frame (and two frames before the first frame to be affected by evolution)
	
	########## Calculate torques and forces  ###################
	
	torque=[Vector([0,0,0])]			#A dummy in the start
	force=[Vector([0,0,0])]				#A dummy in the start
	
	newlist=range(len(cm))	
	for i in newlist[1:-1]:
		torque.append((Llist[i+1]-Llist[i])/dt)
		
		if gravity:
			grav=g
		else:
			grav=0
		
		force.append(PhysOb.M/(dt*dt)*(cm[i-1]-2*cm[i]+cm[i+1])+grav*PhysOb.M*Vector(0,0,1))
	
	#So now we have force and torque as lists starting one frame before the first frame
	
	########## Set the arrow and disc locations, length and directions for all monitor frames 
	
	invForceFactor=1.0/PhysOb.forceFactor
	torquefac=invForceFactor*torqueFactor
	
	for j in framelist[:-1]:
		
		Set('curframe',j)
		i=j-framelist[0]+1		#+1 is to crop away the dummies at the frame before the first frame
		
		APG.SetArrow(force[i]*invForceFactor,Vector(0,0,0),ForceArrow)	
		ForceArrow.insertIpoKey(4)							#Integer 4 means LOCROTSIZE
		
		APG.SetArrow(torque[i]*torquefac,Vector(0,0,0),TorqueArrow)
		TorqueArrow.insertIpoKey(4)							
		
		#Setting up the disc for the (target) bone force
		
		APG.SetArrow(  (boneforcelist[i] + grav*PhysOb.M*Vector(0,0,1) )*invForceFactor,Vector(0,0,0),F_Target)	
		F_Target.insertIpoKey(4)						
		
		#Setting up the disc for the (target) bone torque
		
		APG.SetArrow( bonetorque[i]*torquefac ,Vector(0,0,0),T_Target)	
		T_Target.insertIpoKey(4)
		
		#Set the center of mass
		
		CM.loc=cm[i]
		CM.insertIpoKey(4)
	
	######### Finishing monitoring
	
	if sceneFlag:			#Going back to the regular, slow scene
		scn.makeCurrent()
	else:
		APG.EnableMod(modDict)			
			
	Set('curframe',initframe)
	Redraw()
	print "Monitoring from", framelist[0], "to",framelist[-1]
	print "Done in", time.clock()-TIME0, "s"	
	