############################################################################
#Name: ArmaturePhysicsBoneClasses
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This modules contains class definitions for various levels of INDIVIDUAL physical bones:
#
#PhysBoneClass: The basic physical bone class. Contains attributes and methods for 
#				evolving an armature object (consisting of several PhysBones) as a whole (using F=dP/dt and tau=dL/dt).
#
#InternalPhysBoneClass: Inherites from PhysBoneClass. Also contains attributes and methods for 
#				monitoring internal joint forces and torques (between multiple InternalPhysBones) within an armature object.
#				For example this can be used to monitor the muscle forces necessary to create a specific user defined armature pose evolution.
#
#DynBoneClass: Inherites from InternalPhysBoneClass. Also contains attributes and methods for 
#				dynamical evolution of chains (branches) of joined DynBones within an armature.
#				The bone joints can have constrained directions of rotation. 
#
#SpringBoneClass: Inherites from DynBoneClass. Also contains attributes and methods for 
#				including spring and damping torques at the joints between joined SpringBones.
#
#SpringBoneRubber: Inherites from SpringBoneClass. A particular SpringBone type. The joint is considered to
#consist of something like a waterhose draped over a small flexible metal antenna (with ball bearings in each end).
#So the water hose carries the torsion around the antenna, but the antenna carries the torque perpendicular to the antenna.
#The joint can  rotate in any direction from its rest state, up to 180 deg.
#
#SpringBoneHinge: Inherites from SpringBoneClass. A particular SpringBone type. The joint is considered to
#consist of a mechanical system similar to a Kardan joint, with mechanical springs and dampers at the three rotating axes.
#Note that in one of the directions, the joint can only (stably) rotate up to 90 degrees. This is a downside with this SpringBone type
#as compared to SpringBoneRubber type.
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from Blender import Draw									#For pop-up messages
from Blender.Mathutils import Vector, Matrix, CrossVecs		#For vectors and matrices
from math import pi,atan,cos,sin 

import ArmaturePhysicsTools
reload(ArmaturePhysicsTools)								#For the development phase
from ArmaturePhysicsTools import MidMat, RotVec4Vecs, RotateVec, Normalize, GetRotVec, RotateMat, AngleBetweenVecs

EPSILON=1e-10

#Note: The function AngleBetweenVecs that is imported and used in this module is not quite the same (returns radians)
#as the the function AngleBetweenVecs in Blender.Mathutils (returns degrees). T

#############################################################################################################
#############################################################################################################
######################################## PhysBone Class #####################################################

class PhysBoneClass:
	#The basic physical bone class. For evolving an armature object (consisting of several PhysBones),
	#as well as the basis for other bone classes for internal evolution and monitoring.
	#
	#Note: If you create an object of this type - all list objects
	#including vectors and matrixes, are just linked (even if you deepcopy the new object).
	#So if you have a method or function that for example adds to elements within an object list
	#make sure to first write to the list like b.dynChildren=[], or b.forceExternal=Vector(0,0,0) 
	#before you start adding to the list. Otherwise what you add to one object may be added to another as well.
	#One could optionally set all lists to None in this class definition - but that seems a bit clumsy as well. 
	
	type='PhysBone'		#Indicator for the type of an instance of this class
	Ob=None				#An instance of the PhysBaseClass. Attributes: arm_mat,pose,bodyBoneName,objectBody,dt
	
	#Static instances
	
	name=''
	mass=1											#The bone mass
	crel=Vector(0,0.5,0)							#The relative position of the center of mass relative to the HEAD of the bone in the bone reference frame
	inertiamatrix=Matrix([1,0,0],[0,1,0],[0,0,1]) 	#Moment of inertia matrix
		
	#Dynamic instances
	
	mat=Matrix([1,0,0],[0,1,0],[0,0,1])	#World space bone matrix, 1 frame before current
	c=Vector(0,0,0)						#World space center of mass, 1 frame before current 
	
	prel_mat=Matrix([1,0,0],[0,1,0],[0,0,1])	#An attribute where one can store a preliminary world space bone matrix 
	prel_c=None									#An attribute where one can store a preliminary world space center of mass vector
	
	#Extra attributes. Essentially meaningless when the base object is static (like if we simulate the branches of a tree).
	
	relc=Vector(0,0,0)					#Bone center of mass relative to object/bodybone space
	prel_relc=Vector(0,0,0)				#An instance where one can store a preliminary object/bodybone space center of mass vector
	
	#Safety attributes
	
	updateframe=0						#Keeping track of at what frame the last update_PhysBone took place. To make sure update_PhysBone is not called twice on the same frame
	updatejloop=0						#Keeping track of at what frame internal step the last update_PhysBone took place. To make sure update_PhysBone is not called twice on the same frame
	
	#################### Method definitions ##################################################
	
	def getC(self):
		#Returns current world space center of mass according to the current pose state and the current arm_mat.
		#You may want to update the object location and rotation and constraints according to arm_mat before calling this method.
			
		currmat4=self.Ob.pose[self.name].poseMatrix*self.Ob.arm_mat	#Note - this takes pose bone location into account	
		
		c_world=Vector(self.crel).resize4D()*currmat4
		return c_world.resize3D()	
		
	
	def getMat(self):
		#Returns the current world space matrix of the bone according to the current pose state and the current arm_mat.
		#You may want to update the object location and rotation and constraints according to arm_mat before calling this method.
		
		return (self.Ob.pose[self.name].poseMatrix*self.Ob.arm_mat).rotationPart() 				
	
	
	def getSpin(self):
		#Returns current world space spin of bone midframe before current
		#Note: Currently this method is not called by the corresponding PhysBoneDict,  but it could be and belongs here anyway.
		
		return self.getMatSpin(self.getMat())


	def getMatSpin(self,inmat):
		#Returns world space spin of bone midframe before current, given the (supposedly) current world space 3-matrix inmat.
		
		rotvec=GetRotVec(self.mat,inmat,self.name)
		mat=RotateMat(self.mat,rotvec/2)		#Approximate bone base vectors at midstep
		omega=rotvec/self.Ob.dt					#Rotation vector relative to world space
		omega_proj=mat*omega					#Rotation vector projected onto the momentary bone base vectors
		Sproj=self.inertiamatrix*omega_proj		#The spin angular momentum as projected on the momentary bone base vectors		
		return Sproj*mat						#The bone spin angular momentum relative to world space

	########################## Extra Methods ###########################################
	
	def getRelC(self):
		#Returns the bone center of mass relative to either the object space, or the bodyBone space,
		#depending on if Ob.objectBody is true or not.
		#You may want to update the object location and rotation, and bone chain constraints before calling this method.
		#Comment: This function is used when calculating the angular momentum of for example an entire armature object,
		#but could also be used to calculate the angular momentum of a non-dynamic bone subchain in a dynamic bones evolution.
		#If the armature object is not rotating  there is no real point of this method however.
		#And, if we want to make a dynamic evolution of a tree, rooted to ground - there is no point of this method.
		#So, it is not entirely minimalistic to include it here - but makes sense since one after bones are initialized,
		#one can set object evolution or not, and we do not want to add methods just because we turned on object evolution.
		
		if self.Ob.objectBody:
			return (Vector(self.crel).resize4D()  *  self.Ob.pose[self.name].poseMatrix  ).resize3D()
		
		else:
			vec4=Vector(self.crel).resize4D()  *  self.Ob.pose[self.name].poseMatrix 	#The bone center of mass vector relative to the object space 
			relvec4=vec4-self.Ob.pose[self.Ob.bodyBoneName].poseMatrix[3]							#The bone center of mass vector relative to the bodyBone origin, in object space
			return (self.Ob.pose[self.Ob.bodyBoneName].poseMatrix * relvec4 ).resize3D()			#Projecting relvec4 onto the bodyBone axes
	
	
	####################### Updates ###########################################
	
	def updatePrelMatAndC(self):
		#Sets the bones .prel_mat and .prel_c according to the current pose state and the current arm_mat.
		#You may want to update the object location and rotation and constraints according to arm_mat before calling this method.
		#The idea behind the method is to increase spead, so that these entities are not calculated unnecessarily many times
		
		currmat4=self.Ob.pose[self.name].poseMatrix*self.Ob.arm_mat		#Note - this takes pose bone location into account
		self.prel_mat=currmat4.rotationPart()
		c_world=Vector(self.crel).resize4D()*currmat4
		self.prel_c=c_world.resize3D()
		
		#Update related to evolving object 
		
		self.prel_relc=self.getRelC()		#Comment: I think this is only used for object evolution.


	def update_PhysBone(self):	
		#Updates: .mat .c and .relc	
		
		#A bit of safety - so one does not double update - thus messing with the spin and the velocity.
		#Comment: A bit inefficient to do this safety check here at the individual bone level. But maybe it is the safest option. 
		
		updateframe=self.Ob.iframe
		
		if self.updateframe==updateframe and self.updatejloop==self.Ob.jloop and not updateframe==1:		#If one starts iteration on frame one - there should be three updates on frame one in the initiation (to initiate angular acceleration)
			print "update_PhysBone: Already updated for this frame and frame step. Canceling update."
			return
		else:
			self.updateframe=updateframe
			self.updatejloop=self.Ob.jloop
		
		
		#The actual update
		
		self.mat=self.getMat()
		self.c=self.getC()
		
		#Update related to object evolution
		
		self.relc=self.getRelC()	#Comment: I think this is only used for object evolution.	
		
		
	def update(self):
		#Updates just .mat and .c and .relc.  
		
		self.update_PhysBone()



#############################################################################################################
#############################################################################################################
################################ InternalPhysBone Class #####################################################

		
class InternalPhysBoneClass(PhysBoneClass):
	#A bone class for monitoring internal joint forces and torques (between multiple InternalPhysBones) within an armature object.
	#For example this can be used to calculate the muscle forces necessary to create a specific user defined armature pose evolution.

	type='InternalPhysBone'		#Indicator for the type of an instance of this class
	
	#Spin and velocity
	
	v=Vector(0,0,0)						#World space velocity, 1.5 frames before current
	spin=Vector(0,0,0)					#World space spin (angular momentum about the bone center of mass) 1.5 frame before current
	
	#External influences
	
	forceExternal=Vector(0,0,0)			#External force acting on bone - not including gravity or parent child forces
	torqueExternal=Vector(0,0,0)		#External world space torque about the center of mass, at the previous frame
	
	#Structure instances
	
	parent=None							#Name of physical (or dynamical) parent bone. 
	dynChildren=None					#List of names of dynamical cildren.
	
	#Comment: Should perhaps change name (and meaning?) from dynChildren to physChildren, or maybe internalPhysChildren,
	#since for muscle monitoring monitoring, the bone needs not be dynamical.
	#I am not sure if it is meaningful to mix pure physical and internalphysical bones so the distinction matters.
	
	#Torques and forces coming from contact at the head and at the childrens heads
	
	forceHead=Vector(0,0,0)				#The world space force acting on the head - coming from a parent
	torqueDynamic=Vector(0,0,0)			#The world space torque about the center of mass, coming from the connection with parent and children dynamic bones, at the previous frame
	torqueApparent=Vector(0,0,0)		#The apparent net torque (based on the bones motion and rotation) on the bone minus the (dynamic) torque coming from contact forces with parents and children.
	
	torqueHead=Vector(0,0,0)			#The torque at the head of the bone. Note: This is not used by childclass DynBones - except if you want to monitor torques (for example to drive muscles)
	torqueHeadLocal=Vector(0,0,0)		#Head torque projected onto bone local axes. Note: This is not used by DynBones - except if you want to monitor torques (for example to drive muscles)
	
	############ Method definitions	################################################
	
	def getForce(self):
		#Returns the net force acting on the bone, at the previous frame, based on current position, previous position and 
		#previous velocity. 
		#Comment: At some point this may be updated so that the object motion and rotation is cared for 
		#separate from the motion relative to the object, in the style used in GetPrelL().   
		
		return self.mass*((self.getC()-self.c)/self.Ob.dt-self.v)/self.Ob.dt
	
		
	def getTorque(self):
		#Returns the net torque acting on the bone at the frame before the current frame
		
		return (self.getSpin()-self.spin)/self.Ob.dt

	
	def head(self):	
		#Returns the world space position of the head at the previous frame
		
		return self.c-self.crel*self.mat
		
			
	def tail(self):
		#Returns the world space position of the tail a the previous frame
		
		return self.c-self.crel*self.mat+self.length*self.mat[1]
		
	#Update methods

	def update_InternalPhysBone(self):
		#Updtates internal physics components of bone.
		
		self.spin=self.getSpin()	#This must be done before the .mat is updated.
		cold=self.c					#Storing since update_PhysBone() will overwrite it
		self.update_PhysBone()
		self.v=(self.c-cold)/self.Ob.dt
		

	def update(self):
		#Uppdates self
		
		self.update_InternalPhysBone()


#############################################################################################################
#############################################################################################################
#################################### DynBone Class ##########################################################

class DynBoneClass(InternalPhysBoneClass):
	#A bone class for dynamical evolution of chains (branches) of joined DynBones within an armature.
	#The bone joints can have constrained directions of rotation. 	
	#
	#Comment: Requires additional self.Ob attributes: startFrame, stepsPerFrame and dt0.
	
	type='DynBone'						#Indicator for the type of an instance of this class
	length=1							#The length of the bone (assumed fixed)
	restLocalMatrix=None				#The edit mode (rest) local (relative to parent) matrix.		
	
	#For interpolating an initial rotation of bone
	
	localMatrix=None					#Matrix relative to bones rest space (which in turn rigidly follows the parent), one frame before current
	omegaLocal=None						#Angular velocity of bone relative to the bone's own rest space, 1.5 frame before current
	omegaLocalDot=None					#Angular velocity derivative relative to the bone's own rest space, 2 frames before current
	
	#Net target torque coming from for example springs at head and at childrens heads
	#For a pure dynamical (non-spring) bone, this will be zero.
		
	torqueTarget=Vector(0,0,0)			#The target world space torque about the center of mass, coming from parents and children - but excluding the torque coming from contact forces with parents and children
	initialTorqueTarget=None			#An attribute to save the initial torqueTarget when making small variations. Used typically for springbones, but belongs here next to the torqueTarget attribute.

	#Settings related to either offset gravity torques, or the DynamicFastFlag=True option.
	
	massVec=None						#A vector to be precalculated related to torques from a small shift of children and parent locations
										
	#Settings related to DynamicFastFlag=True
	
	downstreamMass=0					#Mass of all dynamic children and their children and so on.
	massVecChildren=None				#A vector to be precalculated related to torques from a small shift of children locations
	
	allDynChildren=None					#List of names of dynamical cildren and all their children and so on
	allDynParents=None					#List of names of all upstream dynamical parents
	boneIndex=None						#An index telling at what slot in the DynBones dictionary the bone resides. Used in TorqueMatrixAlternative()
	
	#Constraint attributes
	
	freeIndexes=[0,1,2]					#List of rotational freedoms. Contains at most three, no-two alike, ordered least first, integers from 0,1,2. For example [0,2], [3] or []. Example that is not okey: [2,1].
	constrainedIndexes=None				#The complement to freeIndexes.
	hasFreedom=None						#Flag to indicate if there the bone angles should be varied (for the equation system)
	hasConstraint=None					#Flag to indicate in the bone has any rotation constraints
	keepKeyedRotations=None				#Flag to indicate that no variation, or overwriting, of the bone angles should take place.
	
	#Unsorted
	
	parentsHeadToOwnHead=None 			#Vector from parents head to own head.
	massVecToParent=None 				#A type of massVec (a convenient object).

	
	###################### Methods ##############################################################
	
	def getLocalMatrix(self):
		#Returns the bone matrix relative to its own restspace (which in turn follows the parent)
		#For some reason the blender inbuilt .localMatrix does not seem to work
			
		m0=self.Ob.pose[self.name].poseMatrix.rotationPart()		#The posebone matrix (3x3) including constraints rel armature space
		m1=self.restLocalMatrix								#Matrix relative to parent (constant matrix - independent of pose)
		
		if self.parent==None:
			m2=Matrix([1,0,0],[0,1,0],[0,0,1])				#Meaning that armature space counts as the parent
		else:
			m2=self.Ob.pose[self.parent].poseMatrix.rotationPart()	#The parent posebone matrix including constraints rel armature space
		
		m1a=m1*m2											#The restmatrix relative to armature space		
		return m0*(m1a.transpose())	
		
	
	def update_DynBone(self):				
		# Makes updates related to local rotations - used to interpolate an initial bone rotation for the next step for dynamic bones
		# !!Maybe it is unnecessary to have the 'None' if-cases below	
		
		if self.isRigid and self.Ob.iframe==self.Ob.startFrame and self.Ob.jloop==0 and self.Ob.dynamicFlag:
			self.localMatrix=self.getLocalMatrix()
			#This matrix will be used to overwrite possible pre-existing keyed
			#rotations at subsequent steps if isRigid==True.
		
		if self.hasFreedom and self.Ob.dynamicFlag:
			newLocalMatrix=self.getLocalMatrix()
			if not self.localMatrix==None:
				newOmegaLocal=GetRotVec(self.localMatrix, newLocalMatrix,self.name )/self.Ob.dt
			else:
				newOmegaLocal=None
			
			if not self.omegaLocal==None:
				newOmegaLocalDot=(newOmegaLocal-self.omegaLocal)/self.Ob.dt
			else:
				newOmegaLocalDot=None
			
			self.localMatrix=newLocalMatrix 
			self.omegaLocal=newOmegaLocal
			self.omegaLocalDot=newOmegaLocalDot
			v=self.v							#For the startFrame shift below
			
			self.update_InternalPhysBone()
			
			
			#A minor shift due to possible time step change at first frame - to improve interpolation for that frame
			#Comment:This is not really tested.
			acc=(self.v-v)/self.Ob.dt0
			if self.Ob.iframe==self.Ob.startFrame:
				if self.Ob.stepsPerFrame>1:
					self.omegaLocal+=self.omegaLocalDot*(0.5-0.5/(self.Ob.stepsPerFrame))*self.Ob.dt0      
					self.v+=acc*(0.5-0.5/(self.Ob.stepsPerFrame))*self.Ob.dt0
					
		else:
			self.update_InternalPhysBone()
		

	def update(self):
		self.update_DynBone()


#############################################################################################################
#############################################################################################################
################################ SpringBone Class ###########################################################


class SpringBoneClass(DynBoneClass):
	#A bone class for including spring and damping torques at the joints between dynamical bones.
	#This class serves as the basis for the subclasses RubberBone and HingeBone 
	
	type='SpringBone'					#Indicator for the type of an instance of this class
	
	#Internal torque springs and damping
	
	offsetTorqueFlag=False				#Set to true to activate offset torques
	localHeadTorqueRest=Vector(0,0,0)	#The local head torque when the bone is at rest
	
	torqueOffsetRest=Vector(0,0,0)	
	
	SpringHeadTorque=None			#The world space head torque coming from spring joint at the head of the bone	
	SpringTorque=None				#The net world space spring torques coming from the head of the bone and from the childrens heads 
	midframeDampHeadTorque=None
	midframeDampTorque=None
	
	springVec=Vector(0,0,0)	#Stiffness of head joint springs
	dampVec=Vector(0,0,0)	#Strength of head joint dampers
	springChildren=[]		#List of those first-children that are SpringBones
	
	ghostBone=None			#The name of a bone (with the same parent) that sets an alternative restspace for the spring bone.
							#The space of the ghostbone is used instead of the spring bone restspace, when evaluating relative rotation - which in turn is used to get the .
	
	matRefWorld=Matrix([1,0,0],[0,1,0],[0,0,1])   		#The world matrix of the bone reference space, either the restspace of the bone the space derived from the corresponding ghost bone 
	matRelReference=Matrix([1,0,0],[0,1,0],[0,0,1])		#The bone matrix relative to the reference space, given either by the restspace of the bone the space derived from the corresponding ghost bone
	
	boneGroup=None			#Link to the he dynamic bone group this bone is belongs to.
	rubberNewDampFlag=True	#For the rubber type spring/damp joint. True: New type of damping, independent on joint angles.
							#False is the option to get the old damping kind of inbuilt into the rubber joint.
							#Typically should always be set to True.
		
	################## Methods ########################################
	
	def getMatRelReference(self):
		if self.ghostBone==None:
			return self.getLocalMatrix()	#The matrix relative to the restspace of the bone itself
		else:
			return self.getRelGhostMatrix()
	
	
	def getMatRefWorld(self):
		#Returns the matrix of the bone reference frame, whether that is the bones rest frame
		#or the frame given by a ghost bone.
		
		if self.ghostBone==None:
			m1=self.restLocalMatrix
		else:
			m1=self.getMatGhostRelParent()
			
		if self.parent==None:
			m2=Matrix([1,0,0],[0,1,0],[0,0,1])				#Meaning that armature space counts as the parent
		else:
			m2=self.Ob.pose[self.parent].poseMatrix.rotationPart()	#The parent posebone matrix including constraints rel armature space
						
		m1a=m1*m2											#The restmatrix relative to armature space		
		return m1a*(self.Ob.arm_mat.rotationPart())					#The restmatrix relative to world space

		
	def getMatGhostRelParent(self):
		mGhost=self.Ob.pose[self.ghostBone].poseMatrix.rotationPart()	
		if self.Ob.pose[self.ghostBone].parent==None:
			
			mGhostParent=Matrix([1,0,0],[0,1,0],[0,0,1])				#Meaning that armature space counts as the parent
		else:
			mGhostParent=self.Ob.pose[self.ghostBone].parent.poseMatrix.rotationPart()	#The parent posebone matrix including constraints rel armature space
			
		return mGhost*(mGhostParent.transpose())
		

	def getRelGhostMatrix(self):
		#Similar to getLocalMatrix. But the restspace matrix of the bone (relative to the bone's parent)
		#is not used as the reference of zero rotation. Instead the corresponding
		#ghost bones pose matrix relative to its parent is used as the reference matrix of zero rotation.
		#The result is for the current frame.
		
		#Getting the matrix of the pose bone relative to its 'unrotated state' defined by self.getMatGhostRelParent()
		
		m0=self.Ob.pose[self.name].poseMatrix.rotationPart()	#The posebone matrix (3x3) including constraints rel armature space
		m1=self.getMatGhostRelParent()							#This is sort of the 'user-settable' rest space of the bone
		
		if self.parent==None:
			m2=Matrix([1,0,0],[0,1,0],[0,0,1])				#Meaning that armature space counts as the parent
		else:
			m2=self.Ob.pose[self.parent].poseMatrix.rotationPart()	#The parent posebone matrix including constraints rel armature space
		
		m1a=m1*m2											#The restmatrix relative to armature space		
		return m0*(m1a.transpose())	

	
	def getMidframeDampHeadTorque(self):
		#Returns the midframe torque at the head of the bone coming from the joint damping.
		
		do=None						#To be defined by subclass
	
	
	def getSpringHeadTorque(self):
		#Returns the torque at the head of the bone coming from the joint spring.
		
		do=None						#To be defined by the subclass
		
				
	def update_SpringBone(self):
		#Updates the SpringBone
		
		self.matRelReference=self.getMatRelReference()	
		self.matRefWorld=self.getMatRefWorld()	
		self.update_DynBone()	
	
	
	def update(self):
		#Updates the SpringBone
		
		self.update_SpringBone(self)


#############################################################################################################
#############################################################################################################
################################# SpringBoneRubber Class ####################################################

class SpringBoneRubberClass(SpringBoneClass):
	#A SpringBone subclass for assigning torques at the head joints.
	#The joint is considered to consist of something like a waterhose draped over a
	#small flexible metal antenna (with ball bearings in each end guiding the antenna).
	#So the water hose carries the torsion around the antenna,
	#but the antenna carries the torque perpendicular to the antenna.
	#The joint can  rotate in any direction, from its rest state, up to 180 deg.
	
	type='SpringBone'					#Indicator for the type of an instance of this class
	subType='SpringBoneRubber'			#Indicator for the type of an instance of this class
	
	roll=0								#The bone roll relative to the rest space (not the regular Blender roll I think).		
	roll_dot=0							#The bone roll time derivative
	omega_rhat=Vector(0,0,0)			#The angular velocity of the bone central axis relative to the rest space 
	
	############# Method definitions
	
	def getThetaPhiRoll(self,mat):
		#Getting the angles theta,phi and torsion roll for the matrix mat.
		#theta: Tilt angle of the y-axis from its relaxed direction Y in rest space.
		#phi: The angle of the projection of the y-axis onto the Y=0 plane, with the Z-axis. Counting positive around the Y vector.
		#roll: The angle of rotation around the bone y-axis after the tilt. (Not the same as the standard roll angle though)
		
		r_hat=mat[1]
		rho=Vector(r_hat[0],0,r_hat[2])			#Recall that it is Y that is the main axis (not Z as is standard)
		
		if rho.length<EPSILON:
			phi=0
		else:
			x=r_hat[0]
			z=r_hat[2]
			
			if abs(z)>abs(x):		#For numerical accuracy, and to not divide by zero.
				phi=atan(x/z)
				if z<0:
					phi+=pi
			else:
				phi=pi/2-atan(z/x)
				if x<0:
					phi+=pi
		
		if phi<0:
			phi=phi+2*pi
			
		theta=AngleBetweenVecs(Vector(0,1,0),r_hat)
	
		rotvecA=RotVec4Vecs(Vector(0,1,0), mat[1])  						#How the bone y-axis has tilted from its rest position
		rotvecB=RotVec4Vecs(RotateVec(Vector(1,0,0),rotvecA),mat[0])		#How the bone is rotated after the tilt	
		roll=rotvecB*mat[1]
		
		return theta,phi,roll
	
	
	def getThetaPhiRollR_hat(self):
		#Getting the angles theta,phi and torsion roll of the bone as well as the main bone axis
		
		mat=self.matRelReference
		r_hat=mat[1]
		theta,phi,roll=self.getThetaPhiRoll(mat)
		
		if theta>0.8*pi:
			if self.springVec.length>EPSILON:
				print self.Ob.iframe, self.name,":close to pointing opposite its rest direction"
				
				#Passing very close to the southpole will typically result in the torsion roll angle
				#making a jump, which will yield jumping torques as well. 
			
		return theta,phi,roll,r_hat
		
	
	def getMidframeDampHeadTorque(self):
		return self.getMidframeDampHeadTorqueArg(  self.getMatRelReference()  )
	
	
	def getAngleDiff(self,a,b):
		#Returns angular difference from a to b, assuming angles have a break point
		#at +-pi. It assumes that angles change by shortest possible delta (less than or equal to pi) from a to b.
		
		if b>a:
			delta=b-a
			if delta<pi:
				return delta
			else:
				return -(2*pi-delta)
		else:
			delta=a-b
			if delta<pi:
				return -delta
			else:
				return (2*pi-delta)		
	
			
	def getMidframeDampHeadTorqueArg(self,matB):
		#Returning the torque coming from the damping in the rubber spring joint at the head of the bone,
		#midframe before the current.
		
		#A checkup, to give warning if this method is called without need. 
		if not self.hasFreedom:
			print "----------Do not come here!!"
		
		matA=self.matRelReference
		#matB=self.getMatRelReference()	#Currently sent as argument, but kept as a comment her to illustrate a typical meaning. 
		
		mat=MidMat(matA,matB)			#The matrix relative to the reference half a frame before current
		
		#Comment: The damping lives between frames so to speak, so to use mat=MidMat(matA,matB) makes sense.
		#However, calculating MidMat is kind of time consuming so one might opt to use simply mat=matB.
		#Reminder: If one changes to mat=matB, there is a factor of 2* for roll_dot that needs to be removed.
		
		#Getting angles 
		
		theta,phi,roll=self.getThetaPhiRoll(mat)
		
		#Getting various normalized vectors
		
		r_hat=mat[1]
		rho_hat=Normalize(Vector(r_hat[0],0,r_hat[2]))								#Recall that it is Y that is the main axis (not Z as is standard)
		theta_hat=Vector(cos(theta)*sin(phi) ,-sin(theta), cos(theta)*cos(phi))		#Recall that it is Y that is the main axis (not Z as is standard)
		phi_hat=Vector(cos(phi),0,-sin(phi))
		
		#Getting the angular velocities that induce damping
		
		if self.rubberNewDampFlag:
			#The damping due to the axis rotating about itself in a viscous fluid (grease) on a unit sphere.
			
			omega=GetRotVec(matA,matB)/self.Ob.dt	#Local omega. Comment: Bit inefficient, does similar stuff to RotVec4Vecs above. Also will be recalculated for .omega if there is an update. But better to keep it simple and not maximally efficient.
			tau_damp=-self.dampVec[1]*(omega*r_hat)*r_hat
		else:
			#Here the damping is imagined to be due to a not quite elastic (under torsion) rubber hose
			#fitted on a springy antenna. This fits well with how the spring torque is calculated,
			#but there is no real need to be as careful as this when it comes to damping (unlike when concerning spring torques that need to be energy conserving)
			#and frankly the greaseDamping option is probably often better.
			
			roll_dot=2*self.getAngleDiff(self.roll,roll)/self.Ob.dt		#Insures that passing angle pi etc is properly taken care of. The factor of 2 is because roll is evaluated at midstep, and self.roll is evaluated one step before current (so the differens in time is half the step time).
																		#Note that passing pi is NOT handled by the corresponding spring torque however. 
			tau_Y_damp= -roll_dot*self.dampVec[1] 						#The roll torsion torque is assumed to be transported along the bent joint to the base
			tau_rho_damp=tau_Y_damp*sin(theta)/(1+cos(theta))			#This takes som paper and pen calculation, based on the joint iself being massless, and thus in torque and force equilibrium
			tau_damp=tau_Y_damp*Vector(0,1,0)+tau_rho_damp*rho_hat
		
						
		#The damping of the axis motion on an imagined unit sphere
		#The idea is that there is a force acting on the unit sphere, which in turn gives rise to a torque.
		#The force is given by -v_theta*(bx*cos(phi)*cos(phi) + bz*sin(phi)*sin(phi))*theta_hat
		#					   -v_phi  *(bz*cos(phi)*cos(phi) + bx*sin(phi)*sin(phi))*phi_hat
		
		omega_rhat=RotVec4Vecs(matA[1],matB[1])/self.Ob.dt
		
		bx=self.dampVec[0]
		bz=self.dampVec[2]
		
		v_rhat=CrossVecs(omega_rhat,r_hat)		#The main axis velocity where it cuts the unit sphere. Note that it is currently midstep whereas other vectors are on step.
		
		tau_damp+=	-(v_rhat*theta_hat)*(bx*pow(cos(phi),2) + bz*pow(sin(phi),2))*phi_hat
		tau_damp+=	-(v_rhat*phi_hat) * (bx*pow(sin(phi),2) + bz*pow(cos(phi),2))*(-theta_hat)
		
		#Transforming to world space and returning
		
		if self.boneGroup.plainBranchFlag:		
			if self.boneGroup.dampMode==0:
				matRef=self.getMatRefWorld()
			elif self.boneGroup.dampMode==2:
				matRef=self.matRefWorld
			else:
				mat0=self.matRefWorld
				mat1=self.getMatRefWorld()
				matRef=MidMat(mat0,mat1)
		else:
			mat0=self.matRefWorld
			mat1=self.getMatRefWorld()
			matRef=MidMat(mat0,mat1)
		
		#Comment: Originally the MidMat option immediately above was the only option for matRef, regardless of the dampMode.
		#However, calculating MidMat is a bit time consuming, so one can get a bit of speed gain by instead using either self.getMatRefWorld()
		#or self.matRefWorld. Note that this only affects the direction of the torque, not the magnitude. Also note that for
		#a typical two-dimensional scenario (where all bones move in a plane with zero rolling), the different options are equal.
		#
		#Furthermore, for the case of dampMode==2, there is during the creation of the main matrix for the dynamic evolution, 
		#a variation step. By using only the bone world matrix from the previous step, one insures that the damping torque
		#on bones downstream of the bone of variation is unaffected by the variation, and one can get
		#a speed gain by omitting the calculation of these torques. Note that this rests on the assumption of a plain branch.
		#In principle one could have for example ghost bones with point to constraints in the downstream chain,
		#so that the damping downstream is affected after all. 
		#
		#The point is that when there is damping, there is anyway no energy conservation,
		#and keeping track of precisely when damping is evaluated (midframe or on-frame or som mixture of the two) is not as important
		#as for energy conserving torques and forces (like spring torques).
		#However, as dampMode==2 is concerned it is probably a bit more stable for some scenarios to actually use the MidMat option
		#so that is left as an option (set plainBranch=False)
				
		return tau_damp*matRef

	
			
	def getSpringHeadTorque(self):
		#Returning the torque coming from the rubber spring joint at the head of the bone
		
		#A checkup, to give warning if this method is called without need.
		if not self.hasFreedom:
			print "---------- Should not be here!"	
		
		theta,phi,roll,r_hat=self.getThetaPhiRollR_hat()
		
		#Getting the relevant spherical and cylidrical base vectors
		
		rho_hat=Normalize(Vector(r_hat[0],0,r_hat[2]))								#Recall that it is Y that is the main axis (not Z as is standard)
		theta_hat=Vector(cos(theta)*sin(phi) ,-sin(theta), cos(theta)*cos(phi))		#Recall that it is Y that is the main axis (not Z as is standard)
		phi_hat=Vector(cos(phi),0,-sin(phi))
		
		#The torque coming from the roll angle, due to spring torques and offset torque but not including damping
		#The net spring torque about r_hat is assumed to be transported, by some rubber or joint mechanism, to be directed along the positive Y-axis,
		#at the joint base in the rest space (either ghost or bone rest space).
		#This torque then induces a torque about rho_hat as well.
		#A possible physical example would be a torsion bearing rubber hose around a flexible antenna. Where the antenna itself is not carrying any torsion because it is is torsion resistent and free to rotate at the base in the Y-direction.
		
		#Comment: One might think that for example roll_offset could be calculated in 
		#an initialization step rather than at every step. But for future versions
		#one may let the torque offset depend on how the character is leaning,
		#and hence self.localHeadTorqueRest is dynamical, and we need to calculate roll_offset here.
		#So, in anticipation, we calculate it here:-)
		
		if self.offsetTorqueFlag:
			tau_orig=self.localHeadTorqueRest
			if not self.springVec[1]<EPSILON:
				roll_offset=tau_orig[1]/self.springVec[1]		#tau original offset is relative to restspace	
			else:
				print self.name, ":You need a non-zero springVec[1] (y-direction along bone) to get torsion torque offset"
				roll_offset=0
			roll_net=roll-roll_offset
		else:
			roll_net=roll
		
		tau_Y=-roll_net*self.springVec[1] 								 	#The roll torsion is transported along the bent joint to the base
		tau_rho=tau_Y*sin(theta)/(1+cos(theta))								#This takes som paper and pen calculation, based on the joint iself being massless, and thus in torque and force equilibrium
		tau_roll=tau_Y*Vector(0,1,0)+tau_rho*rho_hat
			
		#The torque from a force as a gradient of a potential on a unit sphere acting perpendicular to the bone axis.
		#The potential is taken as PHI=kz*theta*theta*cos(phi)*cos(phi)/2   +  kx*theta*theta*sin(phi)*sin(phi)/2
		#Don't forget the factor of 1/sin(theta) that enters on the phi-part of the gradient:-)
		
		kx=self.springVec[0]
		kz=self.springVec[2]
		
		if theta<10*EPSILON:
			tau_pot=				 theta*(kz-kx)*sin(phi)*cos(phi)*theta_hat		#Approximating theta*theta/sin(theta)=theta for small angles
		else:
			tau_pot=theta/sin(theta)*theta*(kz-kx)*sin(phi)*cos(phi)*theta_hat
		
		tau_pot+=-theta*(kx*pow(cos(phi),2) + kz*pow(sin(phi),2))*phi_hat
				
		#Adding an offset torque due to a an offset force field (constant magnitude) pointing toward the north pole on a unit sphere,
		#(the force is tangent to the surface), acting on the axis of the bone. The unit sphere is rotated relative to rest space
		#such that the perpendicuar part of the offset head torque on the bone (the torque when the armature is relaxed and unrotated)
		#is accounted for by this force (the sphere is attached to the restspace).
		#There are plenty of other more advanced options, but at least this one is energy conserving,
		#and not affecting the frequencies of pure oscillations along the base axes.
		
		if self.offsetTorqueFlag:
			tau_perp=Vector(tau_orig[0],0,tau_orig[2])			#The part perpendicular to Y-hat of the offset torque
			n_hat=Normalize(CrossVecs(tau_perp,Vector(0,1,0)))	#The north pole direction of the offset potential sphere
			new_hat=Normalize(CrossVecs(r_hat,n_hat))			#The direction of the torque
			tau_offset=tau_perp.length*new_hat	
			
			#Letting the torque diminish away from the equator - while preserving energy and small amplitude frequencies
			#But then it is less analogous to the HingeBone offset torque.A matter of taste.
			
			if False:
				th=acos(n_hat*r_hat)	#The angle from the north pole	
				if th<(pi/2):			#If one would let it diminish also towards the south pole - it would less numerically stable
					tau_offset*=sin(th)*sin(th)		#Letting forces diminish towards the north pole
								
			if abs(r_hat*n_hat)>0.85:							#Passing the north or south pole and the torque changes sign
				print self.Ob.iframe,self.name,":Offset potential close to failing, or failing"
		else:
			tau_offset=Vector(0,0,0)
		
		#Adding up contributions
		
		tau=tau_roll+tau_pot+tau_offset
		
		#Transforming to world space and returning
		
		tauLocal=self.matRelReference*tau	#Transforming from bone rest space to bones own space  
		tauWorld=tauLocal*self.mat		#Transforming from bone space to world space				
		return tauWorld
	
	
	############ Updates related to internal torque springs and damping
	
	def update_SpringBoneRubber(self):
		#Updates .midframeDampHeadTorque and .roll, together with all parent class updates
		#and also updates the total spring head torque
		
		if self.hasFreedom:	
					
			if not self.boneGroup.dampMode==2:
				self.midframeDampHeadTorque=self.getMidframeDampHeadTorque()
				
			# Updating the roll angle. Necessary for joint damping.
			
			loc_mat=self.getMatRelReference()
			rotvecA=RotVec4Vecs(Vector(0,1,0), loc_mat[1])  						#How the bone y-axis has tilted from its rest position
			rotvecB=RotVec4Vecs(RotateVec(Vector(1,0,0),rotvecA),loc_mat[0])		#How the bone is rotated after the tilt (using the x-axis as indicator)	
			self.roll=rotvecB*loc_mat[1]											#The current roll angle - the dotted vectors should be aligned, and the dot product gives the correct sign
			
			#Comment: A bit inefficient. One could do the above roll stuff after the update_SpringBone()
			#and use .matRelReference instead of .getMatRelReference(). 
			
			# Updates from parent classes
			
			self.update_SpringBone()												#Calls the parent classes update methods as well
			
			# The spring head torque
			
			self.SpringHeadTorque=self.getSpringHeadTorque()						#Note that this must come after the update because it uses the updated bone matrices
			
			#Removing spring and damp torques along any constrained directions of the head joint.
			#Otherwise these torques are erroneously affecting the target net spring/damp torque of any dynamic parent bone,
			#when UpdatePrelMidframeDampTorqes() is called from the boneGroup instance this bone belongs to.
			#Comment: Note that if the whole matrix/target torque system is changed to work with head torques rather than net bone torques,
			#the need for these lines vanishes.
				
			if self.hasConstraint:
				
				for m in self.constrainedIndexes:
					self.SpringHeadTorque-=(self.SpringHeadTorque*self.mat[m])*self.mat[m]		#Note that self.mat has just been updated.
					
					if not self.boneGroup.dampMode==2:
						self.midframeDampHeadTorque-=(self.midframeDampHeadTorque*self.mat[m])*self.mat[m]
			
		else:
			self.SpringHeadTorque=Vector(0,0,0)			#Comment: Bit ineffective, could be set in an initialisation.
			self.midframeDampHeadTorque=Vector(0,0,0) 
			self.update_DynBone()
		
	def update(self):
		#Updates SpringBoneRubber-bone
		
		self.update_SpringBoneRubber()


#############################################################################################################
#############################################################################################################
####################################### SpringBoneHinge Class ###############################################

class SpringBoneHingeClass(SpringBoneClass):
	#A SpringBone subclass for assigning torques at the head joints.
	#The joint is considered to consist of two perpendicular hinges (like a Kardan joint) and a rolling freedom.
	#Each freedom is connected to a springload and a damping. The Z-hinge (for rotation around the Z-axis) is
	#fixed relative to restspace and the x-hinge axis is rotated according to the Z-rotation.
	#Note that in one of the directions, the joint can only (stably)
	#rotate up to 90 degrees. This is a downside with this SpringBone type
	#as compared to SpringBoneRubber type.
	
	type='SpringBone'					#Indicator for the type of an instance of this class
	subType='SpringBoneHinge'			#Indicator for the type of an instance of this class
	
	angleVec=			Vector(0,0,0)	#Collection of three angles that describe the local rotation of the bone
	angleVecDot=		Vector(0,0,0)	#Time derivative of angleVec

	############## Method definitions ###########################################################
		
	def getJointAngleVec(self,mat):
		#Returns three angles angleX, angleY and angleZ based on tha mat argument.
		#First make a rotation about the rest space Z axis, by angle angleZ. Then make a rotation about
		#the rotated bones x-axis by an angle X. Lastly rotate the bone about its own y-axis an angle angleY.
		#Note that the first two rotations are equivalent to a rotation about rest space X-axis by angleX,
		#and then a rotation about rest space Z-axis by angle Z.
		#In fact, relative to the restspace the three angles corrspond to rotation about Y,X,Z in succesion 
		#by angles angleY, angleX and angleZ.
		
		vec=Vector(mat[1][0],mat[1][1],0)			#The bone y-axis projected onto the rest space XY-plane 
		
		if vec.length<0.2:
			print "Close to the bone singular point in this representation of rotation"
		
		angleZ=Vector(0,0,1)*RotVec4Vecs(Vector(0,1,0),vec) #The dot product is just to get the sign right
		angleX=AngleBetweenVecs(vec,mat[1])
		if mat[1][2]<0:
			angleX*=-1
		
		vec=Vector(cos(angleZ),sin(angleZ),0)		#How the bone x-axis would be be if angleY=0
		angleY=mat[1]*RotVec4Vecs(vec,mat[0])		#The dot product is to get the sign of the angle right
		
		return Vector(angleX,angleY,angleZ)
	
	
	def getJointTorque(self,tau_x,tau_axis,tau_Z,angleZ,yhat):
		#Given three axial torques for the hinge joint, returns the net torque vector
		
		#Dealing with the roll torque
		yprim=Vector(-sin(angleZ),cos(angleZ),0)					#The bone y-axis after a pure rotation of angleZ about the Z-axis (i.e y projected onto the XY-plane)
		denom=yhat*yprim
		
		if denom>EPSILON:
			tau_yprim=(tau_axis -tau_Z*Vector(0,0,1)*yhat)  / denom 
		else:
			print self.Ob.iframe, self.name, "On joint singularity"
			if yhat[2]>0:
				tau_yprim=tau_Z
			else:
				tau_yprim=-tau_Z					
		
		#Putting together the three orthogonal torques in the joint
		
		return tau_yprim*yprim  +  tau_x*Vector(cos(angleZ),sin(angleZ),0)	+  tau_Z*Vector(0,0,1)
	
			
	def getSpringHeadTorque(self):
		#HINGEBONE: Returns the head torque on the bone due to springs in the joints.
		#Torques are relative to restspace XYZ (as opposed to bone space xyz) until they are transformed to world space in the end.
		
		mat=self.matRelReference
 		angleVec=self.getJointAngleVec(mat)
		springVec=self.springVec
		
		#Winding up springs to hold the neccessary rest torque
		
		if self.offsetTorqueFlag:
			tau_orig=self.localHeadTorqueRest	#Relative to rest space
			
			offsetAngleVec=Vector(0,0,0)
			for i in [0,1,2]:
				if not self.springVec[i]<EPSILON:
					angleVec[i]-=tau_orig[i]/self.springVec[i]
				else:
					print self.name, ":You need a non-zero springVec[",i,"]for the offset torque"
			
		#Getting the torques about x, axis and Z 
		
		tau_x=	 -angleVec[0]*springVec[0] 
		tau_axis=-angleVec[1]*springVec[1]   #The torque along the bone main axis coming from the roll-spring and damping
		tau_Z=   -angleVec[2]*springVec[2] 
		
		tau=self.getJointTorque(tau_x,tau_axis,tau_Z,angleVec[2],mat[1])
		
		#Transforming to world space and returning
		
		tauLocal=mat*tau				#Transforming from bone rest space to bones own space
		tauWorld=tauLocal*self.mat		#Transforming from bone space to world space
		return tauWorld
	
	
	def getMidframeDampHeadTorque(self):
		return self.getMidframeDampHeadTorqueArg(  self.getMatRelReference()  )
	
	
	def getMidframeDampHeadTorqueArg(self,matB):
		#HINGEBONE: Returns the current head damping torque on the hinge spring bone.
		#Torques are relative to restspace XYZ (as opposed to bone space xyz) until they are transformed to world space in the end.
		
		matA=self.matRelReference
		#matB=self.getMatRelReference()
		
		mat=MidMat(matA,matB)			#The matrix relative to the reference half a frame before current
		angleVec=self.getJointAngleVec(matB)
			
 		angleVec=self.getJointAngleVec(mat)
		angleVecDot=(angleVec-self.angleVec)/self.Ob.dt	 
		dampVec=self.dampVec
		
		angleVecDot*=2		#To account for the fact that mat is only halfway to the current matrix.
		
		#Getting the torques about x, axis and Z 
		
		tau_x=	 - angleVecDot[0]*dampVec[0]
		tau_axis=- angleVecDot[1]*dampVec[1]  #The torque along the bone main axis coming from the roll-spring and damping
		tau_Z=   - angleVecDot[2]*dampVec[2]
		
		tau=self.getJointTorque(tau_x,tau_axis,tau_Z,angleVec[2],mat[1])
		
		#Transforming to world space and returning
		
		if self.boneGroup.plainBranchFlag:		
			if self.boneGroup.dampMode==0:
				matRef=self.getMatRefWorld()
			elif self.boneGroup.dampMode==2:
				matRef=self.matRefWorld
			else:
				mat0=self.matRefWorld
				mat1=self.getMatRefWorld()
				matRef=MidMat(mat0,mat1)
		else:
			mat0=self.matRefWorld
			mat1=self.getMatRefWorld()
			matRef=MidMat(mat0,mat1)
		
		#Comment: Originally the MidMat option immediately above was the only option for matRef, regardless of the dampMode.
		#However, calculating MidMat is a bit time consuming, so one can get a bit of speed gain by instead using either self.getMatRefWorld()
		#or self.matRefWorld. Note that this only affects the direction of the torque, not the magnitude. Also note that for
		#a typical two-dimensional scenario (where all bones move in a plane with zero rolling), the different options are equal.
		#
		#Furthermore, for the case of dampMode==2, there is during the creation of the main matrix for the dynamic evolution, 
		#a variation step. By using only the bone world matrix from the previous step, one insures that the damping torque
		#on bones downstream of the bone of variation is unaffected by the variation, and one can get
		#a speed gain by omitting the calculation of these torques. Note that this rests on the assumption of a plain branch.
		#In principle one could have for example ghost bones with point to constraints in the downstream chain,
		#so that the damping downstream is affected after all. 
		#
		#The point is that when there is damping, there is anyway no energy conservation,
		#and keeping track of precisely when damping is evaluated (midframe or on-frame or som mixture of the two) is not as important
		#as for energy conserving torques and forces (like spring torques).
		#However, as dampMode==2 is concerned it is probably a bit more stable for some scenarios to actually use the MidMat option
		#so that is left as an option (set plainBranch=False)
			
		return tau*matRef


	def update_SpringBoneHinge(self):
		#Updates the SpringBoneHinge-bone
		
		if self.hasFreedom:
			
			if not self.boneGroup.dampMode==2:
				self.midframeDampHeadTorque=self.getMidframeDampHeadTorque()
			
			self.update_SpringBone()										#Calls the parent classes update methods as well
			mat=self.matRelReference
			self.angleVec=self.getJointAngleVec(mat)
			
			self.SpringHeadTorque=self.getSpringHeadTorque()
			
			#Removing spring and damp torques along any constrained directions of the head joint.
			#Otherwise these torques are erroneously affecting the target net spring/damp torque of any dynamic parent bone,
			#when UpdatePrelMidframeDampTorqes() is called from the boneGroup instance this bone belongs to.
			#Comment: If the hole matrix/target torque system is changed to work with head torques rather than net torques,
			#the need for these lines vanishes.
				
			if self.hasConstraint:
				
				for m in self.constrainedIndexes:
					self.SpringHeadTorque-=(self.SpringHeadTorque*self.mat[m])*self.mat[m]		#Note that self.mat has just been updated.
					
					if not self.boneGroup.dampMode==2:
						self.midframeDampHeadTorque-=(self.midframeDampHeadTorque*self.mat[m])*self.mat[m]
			
		else:
			self.SpringHeadTorque=Vector(0,0,0)				#Improvement: Bit ineffective, could be set in an initialisation.
			self.midframeDampHeadTorque=Vector(0,0,0)
			self.update_DynBone()
		
	def update(self):
		#Updates the SpringBoneHinge-bone
		
		self.update_SpringBoneHinge()