############################################################################
#Name: ArmaturePhysicsBoneDictClasses
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This modules contains class definitions for various levels of GROUPS of physical bones.
#There is a bone GROUP class for every INDIVIDUAL bone class found in ArmaturePhysicsBoneClasses, up to the level of the SpringBoneClass.  
#The module also contains a small class definition for a set of bone groups (list of branches) at the end of the module.
#Comment: For historical reasons the sub-name 'Dict' is used in the class names,
#but really 'Group' would be a better name ('Collection' is a bit long).
#
#PhysBoneDictClass: The basic physical bone group class. Contains attributes and methods for 
#				evolving an armature object. For example .getC() is a method that calculates the center of mass for the 
# 				the current blender pose of the bone collection. 
#
#InternalPhysBoneDictClass:  Note that it is NOT inheriting from PhysBoneDictClass. Contains attributes and methods that are basic for 
#				monitoring internal joint forces and torques, for a group of InternalPhysBones, within an armature object.
#				Currently there is no explicit such monitoring feature, although it is simple to add. 
#
#DynBoneDictClass: Inherites from InternalPhysBoneDictClass. Also contains attributes and methods for 
#				dynamical evolution of groups of joined DynBones within an armature.
#				The bone joints can have constrained directions of rotation.
#				At this level there is also support for holdBones and evolution of the armature object.
#				(via a bodyBone, which is kind of a master parent bone, if so desired).
#				The latter requires the setting of a dynAndObject=True flag.
#
#				The basics of the evolution is that first one interpolates rotations (and position in case of a bodyBone) of the dynamical bones in the group, 
#				based on previous frames. From the interpolated state, one varies all of the freedoms (typically rotations)
#				for the bones in the bone group. The resulting changes in various quantities, for example the the apparent extra ecternal torque of
#				a different bone, are registered in a matrix. The dynamical step is then solved by solving a linear equation system,
#				using the matrix just mentioned. From the resulting pose one can repeat the process to hopefully converge towards a better solution,
#				before moving on to the next step.
#
#SpringBoneDictClass: Inherites from DynBoneDictClass. Also contains attributes and methods for 
#				including spring and damping torques at the joints between joined SpringBones during the evolution.
#
#
#DynBoneBranchesClass: A small class for handling a set of separate dynamical branches.
#				For example, if one approximates that one tree branch is not affecting another tree branch,
#				one can set them up as as two elements in a DynBoneBranch object, which then evolves them separately 
#				leading (typically) to a faster evolution.
# 
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from Blender import Object, Draw				#Note: Object is used only to access a constant.
from Blender.Mathutils import Vector, Matrix, CrossVecs, Quaternion
import copy

import ArmaturePhysicsTools						#For the development phase.			
reload(ArmaturePhysicsTools)					#For the development phase.

from ArmaturePhysicsTools import GetRotVec, MidMat, RotateMat, TransposeList, sign, Normalize, RotVec4Vecs

from ArmaturePhysicsTools import printMatrix	#For the development phase (debugging).
import ArmaturePhysicsSolve						#For the development phase.
reload(ArmaturePhysicsSolve)					#For the development phase.

from ArmaturePhysicsSolve import rjsolve, invertMat, multMatVec	

EPSILON=1e-10
deltaangle=0.001		#Base value: 0.001. For the derivative with respect to rotations of dynamical bones.
						#Comment: Maybe this should be an object feature rather than a global variable.

from math import sqrt

#################################################################################################
#################################################################################################
###############################     PhysBoneDict       ##########################################						

class PhysBoneDictClass:
	#A class definition for a dictionary of physical bones (they may be dynamical as well)
	#Primarilly this is intendend to calculate the center of mass, and angular momentum,
	#of all the physical bones of an entire armature.
	#It can however also be used for sets of bones within an armature.
	#Note in particular that angular momentum is calculated by first calculating the motion relative to the
	#reference frame, and then adding the effect of the rotation of the reference frame.
	#The reference frame is either the armature object itself, or the armature bodybone,
	#depending on the objectBody flag.
	#
	#Comment: If this class was not intended to optionally be used for subgroups of 
	#physical bones, it would perhaps have been wiser to let this class inherite from
	#BasicPhysObjectClass. Then one could (from outside) write PhysOb.GetL(), rather than PhysOb.ObjectBones.GetL().
	#and within the methods one could use self.GetMat() instead of self.Ob.GetMat().
	#One can use this structure even if we use this class for subgroubs of physical bones. 
	#I am not sure which is the best structure, but the current setup sets this class
	#on a par with other BoneClasses, which I think makes sense.
	
	type=			'PhysBoneDict'	#Indicator for the type of an instance of this class
	Ob=				None			#An object of the BasicPhysObject class or its children
	Bones=			None			#Collection of physical bones (or links to physical bones, or its children bone classes), in dictionary form.
	M=				None			#The summed mass of the bones in self.Bones.
	
	PhysObFlag=		True			#True - then the center of mass at the previous frame for this bone collection is assumed to be PhysOb.C. 
	C=				None			#The bone collection center of mass. When self.PhysObFlag=True this is not used.
						 
	############ Method definitions	################################################
	
	def UpdatePrelMatAndC(self):
	#Updates the physicsbone .prel_mat and .prel_c for all bones
		
		for name in self.Bones:
			self.Bones[name].updatePrelMatAndC()
			
	
	def GetPrelC(self):
		#Returns the preliminary center of mass for the entire armature in question
		#Each bone center of mass is read from .prel_c
		
		cm_tot=Vector([0,0,0])
		for name in self.Bones:
			b=self.Bones[name]
			cm_tot+=b.mass*b.prel_c
		
		return cm_tot*1.0/self.M
	
	
	def GetPrelL(self,*args):
		#Returns the angular momentum of the collection of bones armature, with respect to the collection of bone center of mass,
		#midframe before the current frame. One may send the center of mass as an argument
		#to avoid recalculating the center of mass. The orbital angular momentum of the bones
		#with respect to the center of mass is calculated by first calculating the
		#motion relative to the object frame (displaced to be centered on the center of mass),
		#and then the motion due to the object rotation is added.
		#This way one can deal with large object angular velocities, with small relative motions and rotations
		#of the bones, and still get an accurate angular momentum estimate.
		#
		#Otherwise for a rigid object in 2D, with many bones far away from the center, the angular momentum calculation
		#typically goes completely wrong at object rotations of about 90 degrees per step. The reason is that 
		#the bone center of mass (determining bone angular momentum) is assumed to move rectilinearly in between frames.
		#This puts the bone midframe position closer to the center of mass than it actually is, and also slows down the orbital velocity
		#of the bone. The result is that the orbital angular momentum of the bone (about the object center of mass) is reduced.
		#One can show that at 90 degrees per frame, the orbital angular momentum reaches a maximum - so for example, for a 100 degrees object rotation per frame -
		#the rectilinear approximation yields smaller orbital angular momentum for any bone than for 90 degrees per frame rotation
		
		dt=self.Ob.dt
		
		if args==():
			C2=self.GetPrelC()
		else:
			C2=args[0]
		
		if self.PhysObFlag:
			C1=self.Ob.C
		else:
			C1=self.C
			if C==None:
				Draw.PupMenu('C (center of mass) for bone group has not been updated. Quitting.')
				10/0				
					
		
		obMat=self.Ob.GetMat()			#Get the current world 3-matrix of the reference bodybone/object relative to which individual bones .relc and .prel_relc is defined. 
		obLoc=self.Ob.GetLoc()			#Get the current world position of the reference bodybone/object origin.
		
		obOmega=GetRotVec(self.Ob.mat,obMat)/dt
		midMat=MidMat(self.Ob.mat,obMat)
		
		Crel1=self.Ob.mat*(C1-self.Ob.loc)			#Projecting difference between C and loc, onto local object axes. Previous frame
		Crel2=obMat*(C2-obLoc)					#Projecting difference between C and loc, onto local object axes. Current frame
		
		L=Vector(0,0,0)				#Initating the angular momentum about the bone collection center of mass
		
		for name in self.Bones:  
			b=self.Bones[name]
			
			#Midframe approximation
			
			relPos=		(  (b.prel_relc-Crel2)  +  (b.relc-Crel1)  )/2			#Average midframe position of the bone center of mass
																				#relative to the total bone collection center of mass, in object/bodybone axes
			relVelocity=(  (b.prel_relc-Crel2)  -  (b.relc-Crel1)  )/dt		#Average midframe velocity of the bone center of mass
																			#relative to the total bone collection center of mass, in object/bodybene axes		
			
			#Going to world space
			
			worldRelPos=relPos*midMat					
			worldRelVelocity=CrossVecs(obOmega,worldRelPos)+relVelocity*midMat
			
			#Adding up spin and orbital angular momentum
				
			L+=b.mass*CrossVecs(worldRelPos,worldRelVelocity)
			L+=b.getMatSpin(b.prel_mat)								#Note that as the spin is concerned, we consider only the world space matrix, not the matrix relative to the object/bodybone.
			
		return L
	
	
	def GetC(self):
		#Returns the center of mass for the entire armature in question
		#NOTE: If you simultaneously need the L of the armature - it is faster to use UpdatePrelMatAndC();PrelC=GetPrelC(),GetPrelL(PrelC)"
			
		cm_tot=Vector([0,0,0])		
		for name in self.Bones:
			cm_tot+=self.Bones[name].mass*self.Bones[name].getC()
		
		return cm_tot*1.0/self.M
	
	
	def GetL(self,*args):
		#Returns the angular momentum of the armature, with respect to object center of mass,
		#midframe before the current frame. If dt is negative - the regular midframe angular momentum
		#is still returned. Calls UpdatePrelMatAndC().
		#One can send the center of mass as an argument to speed things up. 
		
		self.UpdatePrelMatAndC()
		if args==():
			return self.GetPrelL() 
		else:
			return self.GetPrelL(args[0])
	
	
	def UpdatePhysBones(self):		
		#Updates .mat .c and .relc for all bones in PhysBones that are in the self.Bones list.
		#Note that it is not a complete update (unless the bones are purely physical - not dynamical),
		#but just the update of the basic PhysBone features.
		#Comment:  One could change the name to PhysBoneUpdate, to indicate (slightly) that it is not a complete update.
		
		for name in self.Bones:
			self.Bones[name].update_PhysBone()

	
	def Update(self):
		#Updates center of mass C. Note that if self.PhysObFlag=True this function is typically not called.
		#Instead the GetC() method is called from the physical object itself (rather than from this bone group).	
		
		self.C=self.GetC()
	
	
#################################################################################################
#################################################################################################
#########################      InternalPhysBoneDict       #######################################						

class InternalPhysBoneDictClass:
	#A class definition for a group of bones of the type InternalPhysBone (or its children derivatives).
	#Contains methods and attributes for monitoring internal joint forces and torques, for a group of InternalPhysBones within an armature object.
	#Note that this class is not inheriting from the PhysBoneDictClass.
	
	type='InternalPhysBoneDict'		#Indicator for the type of an instance of this class
	Bones=None						#A dictionary of physical bones. Can be sent as a argument when the class instance is created.
	Ob=None							#An object with attributes like .Mat and .ForceBones.
	ChildrenFirstList=None			#A list of bone names where the children names of self.Bones are listed before their parents
	ChildDynDict=None				#A dictionary of bone children. 
	
	########################### Initiation ######################################################
	
	def __init__(self,Bones,PhysOb):		#Bones should be of class InternalPhysBoneClass or its children.
											#Every bone needs a .parent and a .dynChildren attribute								
		self.Bones=Bones
		self.Ob=PhysOb
		
		self.init_InternalPhysBoneDict()	
		
		
	def init_InternalPhysBoneDict(self):
		
		self.init_Children()				#Writes to self.ChildDynDict and ChildrenFirstList
	
		
	def init_Children(self):
	
		# Entire class instance: .ChildDynDict 
		
		ChildDynDict={}
		for name in self.Bones:
			b=self.Bones[name]
			ChildDynDict[name]=b.dynChildren	
		
		self.ChildDynDict=ChildDynDict				#Comment: This is typically not used in this class (outside of this method) but in the children classes.
		
		
		# Entire class instance: .ChildrenFirstList (where children come before parents
		
		ChildrenFirstList=[]
		ChildDict=copy.deepcopy(ChildDynDict)	
		
		counter=0	#Just a counter to break the loop it it gets stuck
	
		while not ChildDict=={}:		#Looping through the dynamic bones. Note that ChildDict is changed during the loop.  
			
			counter+=1
			if counter>1000:
				Draw.PupMenu("Something is fishy in init_Children. Quitting")
				10/0
				
			donelist=[]	
			
			#The bones that are dealt with are removed from ChildDict before the next iteration.
			
			for name in ChildDict:
				list=ChildDict[name]
				if list==[]:
					ChildrenFirstList.append(name)
					b=self.Bones[name]				#Used to get hold of parent only
					donelist.append(name)
			
					#Removing the bone now that it is dealt with from the parents children
					
					try:
						ChildDict[b.parent].remove(name)	
					except:
						do=None	
					
			#Removing the bones that were dealt with from the ChildDict
		
			for name in donelist:
				del ChildDict[name]
		
		
		self.ChildrenFirstList=ChildrenFirstList
		
	############################ Methods ########################################################
	
	def SetExternalDynForceTorque(self):
		#From force bones in the armature sets the resulting forces and torques on the physical bones
		#that are parents to the forcebone. Also sets torques coming from torque bones?
		#
		#The force given by a force bone is:
		#
		#ScaleX * ScaleY * ScaleZ * RestLengthY * Mbody * g
		#
		#So smaller rest length yields smaller force. But, if you have a restlength (edit length)
		#of say 0.5, and you scale the entire pose bone by a factor of 2, making the pose bone
		#look like a pose bone that has restlength 1 and scale 1, then the small edit mode bone will
		#yield four times greater force than it how it may appear.
		#An option would be to multiply by length cubed instead of multiplying by just length.
		#Then the any two apparently equal pose shapes would yield the same resulting force. 
		#On the other hand one might want to have the freedom to adjust how apparent force relates
		#to actual force. For the moment we keep it as is.
		#
		#Note that there is some overlap between this method and the 
		#PhysOb.GetForceAndTorque() method.
			
		#Zeroing the external forces/torques - enabling the use of += later, in turn enabling several forcebones for the same force parent 
		
		for name in self.Bones:
			self.Bones[name].forceExternal=Vector(0,0,0)
			self.Bones[name].torqueExternal=Vector(0,0,0)
			#Comment: A bit time inefficient since one writes zero forces at every time step
			#also to those bones that has no force bone. Those that have no force bone
			#could be zeroed at the beginning of MainEvolve instead. 
			#Those with force bones however need to be zeroed here, to be able to use += later in this method.
			#One could introduce a "BoneWithForceBone" name list to deal with this a bit more efficiently.
			
		#Setting the external forces and torques from force bones
		
		for name in self.Ob.ForceBones:
			length=self.Ob.ForceBones[name]['length']
			forceparent=self.Ob.ForceBones[name]['forceparent']
			
			bone_mat=self.Ob.pose[name].poseMatrix				#The force bone matrix in armature space
			force_mat_world=bone_mat*self.Ob.arm_mat	
			fpos=Vector(force_mat_world[3][0:3])					#Head (thick end) of force bone position in scene space
			multfactor=Vector(force_mat_world[0][0:3]).length * Vector(force_mat_world[2][0:3]).length	#Comment: e.g. self.Ob.pose[name].size[0] gives only scale prior to scale constraints. Therefore not used.
			
			force_world=-self.Ob.forceFactor*Vector(force_mat_world[1][0:3])*length*multfactor 			#The y-axis is the force indicator
			
			# Interpolating force between frames.
			
			#Comment for below: It would probably generally work better to interpolate using the the local matrix (not the world matrix).
			#The difference arises when the force bone is a (non-hinged) child of dynamical bone. Then the force bone will rotate
			#along with its parent, between frames, even when stepsPerFrame>1. I believe it is more commmon to
			#use a hinged typ force bone however, and then the working with the world space matrix should work fine.
			#If one wants to go for the local matrix option, one should probably make force bones into a class objects, 
			#with methods like getLocalMatrix (like for physical bones).
			#
			#Comment: suppose evolution starts at frame 3. First frame to be affected by evolution is thus frame 4. 
			#Let stepsPerFrame=10. Note that the blender frame is changed from 3 to 4 already when  t=3.1 Dt.
			#Note also that this method, of setting the external forces, is called when jloop has just changed to 0,
			#but the frame has not yet been changed.
			#
			#Comment: Note that, with this setup, assigning a force only at the startFrame of evolution,
			#the net impulse given when stepsPerFrame is large is only about half of the impulse
			#given when stepsPerFrame=1 (think of a time-force diagram - only the right half of the force triangle is used sort of).
			#One could compensate for this and increase the initial force by some factor, but
			#this would not be a good option if we kind of restart the evolution in the middle of an already made 
			#evolution. For this case the left half of the force triangle has already had an effect on the initial velocity.
			#So for now, there will be no first frame compensation.
			
			if self.Ob.stepsPerFrame>1: 			
				if self.Ob.jloop==0:
					self.Ob.ForceBones[name]['force_world']=Vector(force_world)
				else:
					f1=self.Ob.ForceBones[name]['force_world']	#Stored just after one switched to the previous frame.
					f2=Vector(force_world)
					fraction=self.Ob.jloop*1.0/self.Ob.stepsPerFrame
					force_world=f1*(1-fraction) + f2*fraction	#Linear interpolation. Could make a more refined interpolation based on on a rotation and a scale, but this is faster.
			
			# Adding force and force torque to dynamical bone.
				
			if not forceparent==None:
				if forceparent in self.Bones:
					b=self.Bones[forceparent]		#The bone to affect
					cm=b.getC()
					b.forceExternal+=force_world
					b.torqueExternal+=CrossVecs(fpos-cm,force_world)			   				#Note that cm is the bone center of mass
		
		# Set torques from torque bones
		
		for name in self.Ob.TorqueBones:
			length=self.Ob.TorqueBones[name]['length']
			torqueparent=self.Ob.TorqueBones[name]['torqueparent']
			torqueparentParent=self.Ob.TorqueBones[name]['torqueparentParent']
			
			bone_mat=self.Ob.pose[name].poseMatrix	
			torque_mat_world=bone_mat*self.Ob.arm_mat	
			multfactor=Vector(torque_mat_world[0][0:3]).length * Vector(torque_mat_world[2][0:3]).length	#Comment: e.g. self.Ob.pose[name].size[0] gives only scale prior to scale constraints. Therefore not used.
			
			torque_world=self.Ob.torqueFactor*Vector(torque_mat_world[1][0:3])*length*multfactor   	#The y-axis is the torque indicator
			
			# Interpolating torque between frames.
			
			if self.Ob.stepsPerFrame>1: 			
				if self.Ob.jloop==0:
					self.Ob.TorqueBones[name]['torque_world']=Vector(torque_world)
				else:
					t1=self.Ob.TorqueBones[name]['torque_world']	#Stored just after one switched to the previous frame.
					t2=Vector(torque_world)
					fraction=self.Ob.jloop*1.0/self.Ob.stepsPerFrame
					torque_world=t1*(1-fraction) + t2*fraction
			
			# Adding torque to dynamical bone, and bone parent if these are in self.Bones. Note that thesa are internal torque/counter torques (not affecting object angular momentum).
			
			if not torqueparent==None:
				if torqueparent in self.Bones:
					b=self.Bones[torqueparent]		#The bone to affect
					b.torqueExternal+=-torque_world		

			if not torqueparentParent==None:
				if torqueparentParent in self.Bones:
					b=self.Bones[torqueparentParent]		#The bone to affect
					b.torqueExternal+=torque_world			#The y-axis is the torque indicator					
										
	
	def CalculateInternalForces(self):
		#For the current armature pose, calculates the internal head forces between parents and children,
		#at the frame before the current frame. The function assumes the external forces (and gravity) to be given,
		#as well as the pose of the armature, at current and previous frames. The results are written to .forceHead in self.Bones
		
		for name in self.ChildrenFirstList:	
			b=self.Bones[name]
			b.forceHead=b.getForce()-b.forceExternal
				
			for childname in b.dynChildren:
				b.forceHead+=self.Bones[childname].forceHead
			
			if self.Ob.gravity:
				b.forceHead+=b.mass*self.Ob.g*Vector(0,0,1)	 
	
	
	def UpdateTorqueApparent(self):
		self.InternalUpdateTorqueApparent()
					
	
	def InternalUpdateTorqueApparent(self):
		#Calculating the net torque on every dynamic bone,
		#coming for example from spring loaded parent child joints, or torques due to external forces,
		#that must be acting on the bone at the frame before the current 
		#to give current position and rotation of the bone.
		#Note that the so called dynamic torque coming from parent and children 
		#joint contact forces (as calculated based on the individual center of mass motion for all the bones)
		#is not included in the apparent torque. The result is written to the bones in self.Bones: .torqueApparent
		
		self.CalculateInternalForces()		#Calculates the internal head-forces at the frame before the current	
		
		for name in self.Bones:
			b=self.Bones[name]
			
			#Getting the torque coming from contact forces with parent and children 
			b.torqueDynamic=CrossVecs(b.head()-b.c,b.forceHead)				#Note that b.head, b.c are positions before the current frame
			for childname in b.dynChildren:
				bchild=self.Bones[childname]
				b.torqueDynamic+=CrossVecs(bchild.head()-b.c,-bchild.forceHead)
				
			#Getting net torque - excluding the torque from pull and pushing forces in the bone chain
			b.torqueApparent=(b.getSpin()-b.spin)/self.Ob.dt-b.torqueDynamic
	
					

#################################################################################################
#################################################################################################
#############################      DynBoneDict  	#############################################

class DynBoneDictClass(InternalPhysBoneDictClass):
	#A class definition for dynamical evolution of groups of joined DynBones within an armature.
	#The bone joints can have constrained directions of rotation.
	#There is also support for holdBones and evolution of the armature object (via a bodyBone).
	#The latter requires the setting of a dynAndObject=True flag.
	#The basics of the evolution is that first one interpolates rotations (and position)
	#based on previous frames. From the interpolated state, one varies all of the freedoms (typically rotations)
	#for the bones in the bone group. The resulting changes of various quantities, for example the the apparent torque of
	#a different bone, are registered in a matrix. The dynamical step is then solved by solving a linear equation system,
	#using the matrix just mentioned.
	
	
	type='DynBoneDict'			#Indicator for the type of an instance of this class
	Nbones=None					#The number of bones in self.Bones
	dynamicFastFlag=None
	dynamicNiteration=None
	
	rigidKaltInverseDict=None	#{'0':[None,None,None]}
	rigidKmatInverseDict=None	#{'0':[None,None,None]}
	
	#Dictionaries of lists for dynamicFastFlag=True and dynamicFastFlag=False. 
	#Each dictionary contains a list of TorqueMatrices for the the Almost Rigid option.
	#The dictionary key is an integer turned into a string, corresponding to stepsPerFrame-1.
	#The list items corresponds to dampMode=0,1 or 2. 
	
	#DynamicFast and torqueOffset
	
	parentsHeadToOwnHead=None		#Vector from dynamical parents head to own head, in parents editmode local space. 
	massVecToParent=None			#Bone mass + dyn children bone masses times vector from parent center of mass to own head, in parents editmode local space.				
	massVec=None					#See comments is code, where it is defined.
	massVecChildren=None			#Currently only use in initialization of massVec.
	
	#Almost rigid option
	
	firstStepFlag=None		#An indicator of whether it is the first frame of dynamic evolution
	
	#Constrained joints
	
	hasFreedom=None			#Indicates if the the entire branch has any joint degrees of freedom. If not it will typically not evolve.
	hasConstraints=None		#Indicates if the the entire branch has any joint constraints.
	isRigid=None			#Indicates if the bone local matrix from the first frame should overwrite subsequent local matrices. 
	
	# For hold bones
	
	HoldBoneDict=None		#A dictionary (hold bone names are the keys) with a lists of various hold bone features.
	
	# General
	
	interpolateAccFlag=None	#True: omega_dot is used in the interpolation of initial bone rotations.
	
	########################### Initiation ######################################################
	
	def __init__(self,Bones,PhysOb):
		
		self.Bones=Bones
		self.Ob=PhysOb
		self.init_DynBoneDict()					#This class initiation
		
			
	def init_DynBoneDict(self):
		
		self.init_InternalPhysBoneDict()		#Parent class initiation
		self.init_fastAndOffset()
		
	
	def init_fastAndOffset(self):
		#Initiation related to dynamicFastFlag and torqueOffset.
			
		# Each individual bone: .downstreamMass and .allDynChildren
		
		for name in self.Bones:
			
			b=self.Bones[name]
			b.allDynChildren=[]			#Needs to be done before appending, to unlink the instance
			b.downstreamMass=0
			
			list=self.Ob.arm_data.bones[name].getAllChildren()
			for childbone in list:
				if  childbone.name in self.Bones:						#Comment: At some point perhaps self.Bones -> self.Ob.AllPhysBones, to include non dynamical bones in the chain, if object self.Ob is attached. But I think this will be dealt with differently if it is dealt with.
					b.downstreamMass+=self.Bones[childbone.name].mass	#Comment: At some point perhaps self.Bones -> self.Ob.AllPhysBones, to include non dynamical bones in the chain, if object self.Ob is attached. But I think this will be dealt with differently if it is dealt with.
				if  childbone.name in self.Bones:
					b.allDynChildren.append(childbone.name)
			
			b.allDynChildrenAndSelf=b.allDynChildren[:]		#Making a copy
			b.allDynChildrenAndSelf.append(name)
		
		# Each inividual bone: .allDynParents
		
		for name in self.Bones:
			b=self.Bones[name]
			b.allDynParents=[]			#Needs to be done before appending, to unlink the instance
			newname=name
			b.dynParent=None
			
			while not newname==None:
				
				#Getting parent bone name
				
				pbone=self.Ob.arm_data.bones[newname].parent
				if not pbone==None:
					pname=pbone.name
				else:
					pname=None
				
				# The rest
				
				if pname in self.Bones:
					if b.dynParent==None:
						b.dynParent=pname
					
					b.allDynParents.append(pname)
					newname=pname
				else:
					newname=None	
		
		# Each individual bone:  .boneIndex. Entire class instance: .Nbones
		
		self.Nbones=0
		for name in self.Bones:
			b=self.Bones[name]
			b.boneIndex=self.Nbones	#Indicates what bone belongs at what row/column in the matrix from torqueMatrixAlternative()
			self.Nbones+=1			#The number of bones in self.Bones
		

		# Each individual bone:  .massVecChildren and .massVec
		
		for name in self.Bones:
			b=self.Bones[name]
			list=self.ChildDynDict[name]
			b.massVecChildren=Vector(0,0,0)		#Must be reset here to unlink it from the class definition connected to all dynamic bones
			
			for childname in list:	
				child=self.Bones[childname]
				vec=self.Ob.arm_data.bones[childname].head['BONESPACE']		#Head position relative to parent tail, in parent space, editmode
				deltavec=vec+b.length*Vector(0,1,0)-b.crel					#Child head position relative to parent center of mass in parent space
				
				child.parentsHeadToOwnHead=vec+b.length*Vector(0,1,0)				#To deal preperly with bones that have an offset (head is not attached to parents tail)
				child.massVecToParent=deltavec*(child.mass+child.downstreamMass)	#To deal preperly with bones that have an offset (head is not attached to parents tail)
				
				b.massVecChildren+=deltavec*(child.mass+child.downstreamMass)	
			
			b.massVec=b.crel*(b.mass+b.downstreamMass)+b.massVecChildren
		
		# Comment Above: Summing dynamic children downstream masses times vector from bone center of mass 
		# to child head, in bone space. Saved in .massVecChildren.
		# Also forming .massVec by subtracting the analogue vector for the bone itself.
		# The resulting vector, when subsequently transformed to world space and crossed with a displacement
		# vector, and divided by (dt*dt), will yield the extra internal torque on the bone coming from a 
		# pure displacement of the bone and all its children by 'dtail'. The displacement
		# in question is assumed to come from a small variation of an upstream bone.
		# The trick assumes that the offset of children is kept constant with respect to their parent space.
		
	
	############################### Methods ################################################
	
	def ExternalToTargetTorques(self):
		#Updating target torques for the dynamical bones.
	
		for name in self.Bones:
			b=self.Bones[name]
			b.torqueTarget=Vector(b.torqueExternal)		#Copying just to be safe

			
	def UpdateTargetTorques(self):
		#Note: If this class is inherited by a SpringBoneClass, then this method is overwritten, not inherited.
		
		self.ExternalToTargetTorques()
			
	
	def UpdateTorqueApparent(self):
		#Simple explanation: calls self.InternalUpdateTorqueApparent(). Then excludes the net constraint torques (like if a bone joint operates like the hinge of a door)
		#from each parent bones .torqueApparent. More complete explanation at the end of the method.
		
		self.InternalUpdateTorqueApparent()
		
		# Transporting torques due to constrained joints to the first parent
		
		if self.hasConstraints:
			for name in self.ChildrenFirstList:		#Comment: A bit inefficient. We could have a (in general) smaller constrainedChildrenFirstList instead.
				b=self.Bones[name]
				
				if b.parent in self.Bones and b.hasConstraint:				#Comment: A bit inefficient to search a list.
					constraintTorque=b.torqueApparent-b.torqueTarget
					for m in b.freeIndexes:		#Complement to constrainedIndexes. Selects only the torque in the constrained directions, by removing the torque the free directions.
						constraintTorque-=(constraintTorque*b.mat[m])*b.mat[m]	
					
					self.Bones[b.parent].torqueApparent+=constraintTorque
					
		
		#After evoking self.InternalUpdateTorqueApparent(), every bones original .torqueApparent  is the
		#net torque (as opposed to head torque) that needs to act on the bone,
		#apart from torques coming from joint contact forces, to bring the bone to its current
		#state of rotation (given the entire bone branch current configuration).
		#
		#The basic idea of this method is to put the torques due to constraints
		#(constrained directions of rotation) at the joints, on a par with 
		#the torques coming from the contact forces at the joints.
		#
		#So, the basic idea is that after evoking this script the .torqueApparent will be the 
		#net torque (as opposed to head torque) that needs to act on the bone,
		#apart from torques coming from joint contact forces AND joint constraint torques,
		#to bring the bone to its current state of rotation
		#(given the entire bone branch current configuration).
		#The .torqueApparent as defined above should be orthogonal to any constraint directions.
		#However, we do not bother to remove any constraint parallel 
		#parts in the resulting .torqueApparent, since these will fall away anyway
		#by removing the corresponding rows in the TorqueMatrix equation system.
		#
		#Now, the original .apparentTorque for every individual bone, can be divided into
		#three parts:
		#
		#o A target net torque. The real torque sort of. Known given the current configuration. For example due to external torques or parent/children spring_damp torques.
		#o A torque coming from any head constraints plus torques coming from children head constraints.
		#o An External extra (error) torque, that is always along any free direction of the bone.
		#
		#A definition: The "Reduced spring/damp head torque" is the spring head torque 
		#along the free directions of rotation for that bone.  
		#When we calculate the spring/damp contribution to the net target torque, we take the
		#bones own Reduced spring head torque, minus the reduced spring head torque for every child.
		#The net target torque we then get by adding the external target torques, to the 
		#net (reduced) spring torques.
		#
		#Now, what we do in this method is the following. Starting at the tips and moving upstream
		#in the tree, for the first constrained bone that we find - we take 
		#the original .torqueApparent and subtract the .torqueTarget. The remainder we dot
		#with the constraint directions. We take this to be the constraint torque.
		#This torque we propagate (actually with a plus sign)
		#onto the parent (if there is a parent in the branch). This propagation changes
		#the .torqueApparent for the parent. If there are any more constrained children 
		#of that parent we let their constraint torques propagate to the parents .torqueApparent as well.
		#Now, suppose that all of the children for the parent in question are dealt with,
		#then we deal with this parent as we did with the first constrained child.
		#
		#Now the above line of reasoning is a bit complex.  We could 
		#instead work with .torqueApparentHead (not yet existing), 
		#which can be calculated either after the regular .torqueApparent
		#are calculated, or instead of calculating the .torqueApparent
		#For the latter case we would propagate head torques at the same time as we
		#propagate contact forces (and the corresponding torques).
		#For this setup we would use the head torques as the targets,
		#for dampMode>0 the damping part of the target torque would need to be updated
		#for every bone variation, but we would not have to collect these into net torques using 
		#self.UpdatePrelMidframeDampTorques(). The speed gain
		#here can be more or less identical to the speed loss of calculating apparent 
		#head torques. If dampMode=0, or if there is no spring/damping, however, this 
		#suggested head based torque system would be a bit slower.
		#But, when there is damping, a speed advantage of working with the head torques
		#is that we do not have to calculate the Reduced spring/damp torque.
	
	################### Methods relating to dynamicFast=False ########################################
	
	def dynamicColumns(self,taulist,localFlag,InitC=None,InitL=None):
		#Note: If this class is inherited by a SpringBoneClass, then this method is overwritten
		#not inherited.
		#
		#Given a list of vectors representing the initial (from interpolation) apparent torques,
		#ordered in the same manner as self.Bones, small rotations are made of each bone about it's own
		#three base axes (in succesion). The downstream bones will be shifted and rotated accordingly.
		#For the new configuration, the new apparent torques are calculated. 
		#From the difference between the new apparent torques and the initial apparent
		#torques, a koefficient matrix is calculated giving the influence (derivative) of a change
		#in angle of bone i on the apparent torque on bone j.
		#
		#If localFlag is True then the vectorial coefficent of influence is rotated to local bone coordinates
		#before being added to the influence matrix. 
		#
		#If there are no joint constraints then for example, 
		#the effect of an x-rotation of the i'th bone (starting on 0) 
		#can be found in column nr 3*i. The effect on all bones of a y-rotation can be
		#found in column 3*i+1.
		#
		#When there are joint constraints the correseponding rows and columns  
		#are cropped away. So if there for example is just a single constraint, 
		#on the x-direction of bone number i, then row and columned index 3*i is removed and the matrix
		#is reduced accordingly.
		#
		#If there are hold bones, either parented to the bone group in question or not,
		#the submatri (list of columns) get added rows. These contains information
		#of how distance (vector) between the hold bone head and the target dynamic bone tip is changed
		#due to variations of the dynamical bones in the boneGroup. The columns contain
		#information of how the apparent torque is affected by a force (in any of three directions)
		#from the hold bone. If the hold bones parent is part of the bone group, one considers the effect
		#of a force on the target bone and a counter force on the parent of the hold bone.
		#
		#If the dynAndObject flag is set, the submatrix (list of columns) gets added rows at the beginning 
		#that indicates how the object center of mass and the object angular momentum depend on 
		#the variations of the dynamical bone angles.
		#
		#NOTE: It is assumed (for the moment) that the hold bone has no offset
		#relative to the parent bone (so the head of the hold bone is at the tip of the parent bone).
		
		
		# Setting up the submatrix (list of columns).
		
		columnlist=[]
		for name in self.Bones:
			
			b=self.Bones[name]
			
			if b.hasFreedom: 	
				
				#Initial conditions
				matInit=b.getLocalMatrix()				#The local matrix after constraints
				quatInit=Quaternion(matInit.toQuat())	#Note that this may be different from the bone quat - because the quat is not including constraints - but matInit is	
					
				#Making small variations of the bone angles and registering the effects. 
				
				for i in b.freeIndexes:
					column=[]
					
					self.Ob.pose[name].quat=RotateMat(matInit,matInit[i]*deltaangle).toQuat()
					self.Ob.arm_ob.getPose().update()
					
					#For object evolution. 
					
					if self.Ob.dynAndObjectFlag: 
						if name in self.Ob.ObjectBones.Bones: #The bone could still affect the center of mass via constraints and stuff though - but we assume it does not.
							self.Ob.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
							DeltaC=self.Ob.ObjectBones.GetPrelC()-InitC			
							DeltaL=self.Ob.ObjectBones.GetPrelL(InitC)-InitL
							
							if localFlag:		#Comment: Maybe this transformation should be made irrespective of the localFlag. Unsure.
								DeltaC=self.Ob.mat*DeltaC
								DeltaL=self.Ob.mat*DeltaL
								
							column.extend(DeltaC/deltaangle)
							column.extend(DeltaL/deltaangle)
				
							
						else:
							column.extend([0,0,0,0,0,0])
					
					##############################		
					
					if self.Ob.dynamicFlag:
						self.UpdateTorqueApparent()
						
						j=-1
						#column=[]
						for name2 in self.Bones:
							j=j+1
							b2=self.Bones[name2]
							
							if b2.hasFreedom:
								kvec=(b2.torqueApparent-taulist[j])/deltaangle
								
								if localFlag:
									kvec=self.Bones[name2].mat*kvec
									
								for m in b2.freeIndexes:
									column.append(kvec[m])
							
					# Hold Bones: Adding the influence of the variation on the tip location of any bone targeted by a HoldBone
					# This added at at the end of each matrix column.
					
					self.addHoldDeviations(column,deltaangle,localFlag)
						
					# Appending the column connected to the current variation 
					
					columnlist.append(column)
										
				self.Ob.pose[name].quat=quatInit	#Resetting bone rotation 
			
		# Finalizing
							
		self.Ob.arm_ob.getPose().update()			#Resetting configuration to the initial configuration. This insures that poseMatrix is updated as well, as I understand it
		return columnlist
	
	
	##################### Methods related to hold bones ########################################
	
	def addHoldDeviations(self, column,deltaFactor,localFlag):
		#For hold bones. Given a link to a column (list), adds factors (at the end of the list) related to the shift in position of the tail of
		#target bones relative to the heads of the corresponding hold bones. Uses global variable 'deltaangle'.  
		#The effect is to add rows to the linear equation system in question. These rows contains the information of how
		#a change of dynamical bone angles (or object(bodyBone angles) affect the distance between hold bone(s) and target(s). 
		
		for holdName in self.HoldBoneDict:
			
			initYscale=self.Ob.pose[holdName].size[1]
			if initYscale>0.6:	#If not, the bone is considered shut off as a hold bone.			
							
				targetName=self.HoldBoneDict[holdName][0]
				
				pos1=(self.Ob.pose[targetName].tail.resize4D()*self.Ob.arm_mat).resize3D()  #Current position of the dynamical target bone tip	
				pos2=(self.Ob.pose[holdName].poseMatrix*self.Ob.arm_mat).translationPart()	#The position of the head of the HoldBone
				
				initialDeltaToHold=self.HoldBoneDict[holdName][3]	#This has been stored previously
				deltaShift=-( (pos2-pos1) - initialDeltaToHold)
				
				kvec=deltaShift/deltaFactor
				
				if localFlag:
					if targetName in self.Bones: 
						kvec=self.Bones[targetName].mat*kvec
					else:
						kvec=self.Ob.mat*kvec	#When dynAndObjecFlag is True - we use the object matrix.
				
				column.extend(kvec)
	
	
	def getHoldApparentTorques(self,force,targetDynBone,localFlag):	
		#For hold bones. Given a force 'force' in world space acting at the tip of 'targetDynBone'
		#returns the "would be" effect of the force on the apparent torque for all dynamical parents of the
		#bone. Result is return as a list (column).
		
		# The effect on the first bone (the targetDynBone): 
		# Note that 'targetDynBone' can mean either the hold bone target, or the hold bone parent (if the parent is part of the bone group),
		# depending on what targetDynBone name is sent to this function. So the notation 'targetDynBone' is a bit misleading. 
		
		b=self.Bones[targetDynBone]
		b.forceApparentTorque=-CrossVecs(b.mat[1]*b.length,force)	#The minus is because the apparent (extra) torque is opposite to the
																	#actual torque from the hold bone.
																	#Note that this can be seen as the torque about the bone center of mass,
																	#because not only is there a force acting at the bone tip,
																	#but there is also a counter force acting at the bone head. 
		
		#Note: .forceApparentTorque is only an intermediate attribute, only used in this function. 
		
		# The effect on all the upstream bones:
		
		while b.parent in self.Bones:		#Comment: A bit bit time inefficient. while not b.dynParent==None would be an option.
			bc=b
			b=self.Bones[b.parent]
				
			torque=CrossVecs(bc.head()-b.head(),force) #The real torque coming from the hold bone force.
			b.forceApparentTorque=-torque 		#The result of the hold force on the apparent torque if there would be no constraints.
			
			# Adding the effect of constraints. Compare with UpdateTorqueApparent()
			
			for m in bc.constrainedIndexes:
				b.forceApparentTorque+=(bc.forceApparentTorque*bc.mat[m])*bc.mat[m]
		
		
		# Composing the list (column) of the the effect on all the bones in self.Bones:
		 
		column=[]
		
		for name in self.Bones:
			b=self.Bones[name]
			
			if name in self.Bones[targetDynBone].allDynParents or name==targetDynBone:
				
				kvec=b.forceApparentTorque
				
				if localFlag:
					kvec=b.mat*kvec
							
				for m in b.freeIndexes:
					column.append(kvec[m])
			else:
				for m in b.freeIndexes:
					column.append(0)
		
		return column		
			
	
	def setHoldBoneForce(self,forcelist):
		#Given a list of hold bone force components, 
		#writes to self.Ob.holdWorldForce and self.Ob.holdWorldTorque to store the net force and torque on the physical object from the hold bone forces.  
		#Also sets each hold bone scale and rotation to correspond to the hold bone
		#force. The hold bone can then be treated as a regular force bone.
		#Note however that the hold bone force that is set in this method is set at iframe-1.
		#After a bone and step update, the stored hold bone force is therefore at two frames before current.
		#It can thus only (with the current object scheme) act as an approximate external force
		#(ideally it would have been at just one frame before the current step in the evolution). 
		#It can however be used for accurate subsequent monitoring of muscle forces for example.
		#Note that if the hold bone has a dynamical parent within self.Bones, 
		#there are two internal forces to consider, and no net external (object) force.
		
		self.Ob.holdWorldForce=Vector(0,0,0)
		self.Ob.holdWorldTorque=Vector(0,0,0)
		
		counter=-1
		for holdName in self.HoldBoneDict:
			initYscale=self.Ob.pose[holdName].size[1]
			
			if initYscale>0.6:	#If not, the bone is considered shut off as a hold bone.			
				
				counter+=1
				force=Vector(forcelist[counter*3:counter*3+3])	#Local force in target (dynamic) bone space, frame before current.
				
				targetName=self.HoldBoneDict[holdName][0]
				if targetName in self.Bones:
					worldForce=force*self.Bones[targetName].mat		#Irrespective of the local flag, force lives in the target bone space.
				else:
					worldForce=force*self.Ob.mat
				
				worldRescaledForce=worldForce/self.Ob.forceFactor				#The force in terms of the basic force (self.Ob.M*self.Ob.g)
				
				#Updating object hold force and hold torque
				
				if self.Ob.dynAndObjectFlag and self.HoldBoneDict[holdName][1]==None:	#If the hold bone has a dynamical parent, the hold bone is assumed to give no net force.
					#Comment: If holdbone has no parent, or the parent is not part of the branch - then we go here (assuming dynAndObjectFlag==True).
					
					#Note that the targets for L and C for the body, given to the equation system
					#are the targets based on regular exterior forces and torques - not on hold bone forces and torques. 
					#Therefore we must store the net force and torque coming from hold bones here,
					#so that subsequently one can include their effect on the targets for L and C.
					
					pos=self.HoldBoneDict[holdName][2]
					self.Ob.holdWorldForce+=worldForce
					self.Ob.holdWorldTorque+=CrossVecs(pos-self.Ob.C,worldForce)	#Note that we use (as we should) the stored C (from the frame before) and not the current C. 
																					#One could opt to use a stored target C however (from the frame before), although that is currently not stored.
				
				###############################################################
				##### Storing the force in the pose-size of the hold bone #####
				
				if self.Ob.jloop==0:		
					#Getting the force relative to the hold bone rest space (frame before current)
					
					targetName=self.HoldBoneDict[holdName][0]
					parentName=self.HoldBoneDict[holdName][1]
					
					if parentName==None:
						if self.HoldBoneDict[holdName][5]==None:
							parentMat=Matrix(self.Ob.storedArmMat)
						else:
							parentMat=Matrix(self.HoldBoneDict[holdName][6])		#This is where the world matrix of any non-dynamic parent bone is stored. 	
					else:
						parentMat=self.Bones[parentName].mat
						
							
					refMat=self.Ob.arm_data.bones[holdName].matrix['BONESPACE']*parentMat	#The 'BONESPACE' matrix is 3x3
					refRescaledForce=worldRescaledForce*(refMat.transpose())		#The rescaled force relative to the hold bone rest space frame before current.
					
					#Setting hold bone quat and size
					
					initQuat=Quaternion(self.Ob.pose[holdName].quat)
					initSize=Vector(self.Ob.pose[holdName].size)					
					
					if refRescaledForce.length>EPSILON:
						normForce=Normalize(refRescaledForce)
						rotvec=RotVec4Vecs(Vector(0,1,0),-normForce)	#Minus sign is because force points from tail to head.
						newmat=RotateMat(Matrix([1,0,0],[0,1,0],[0,0,1]),rotvec)
						self.Ob.pose[holdName].quat=newmat.toQuat()
						f=sqrt(refRescaledForce.length)
						
						initYscale2=self.HoldBoneDict[holdName][4]
						
						if initYscale2>EPSILON:
							f*=1.0/sqrt(initYscale2)	#So if the hold bone is turned off, with yScale=0.5 for example, we compensate by making the hold bone wider (so the scale volume is correct, assuming unit rest length).
														#The thing is that the if the hold bone is active at the current frame, but not at the previous frame
														#we must not muck with the y-scale of the previous frame, but we still want it to work as a regular force arrow.					
						
						self.Ob.pose[holdName].size=Vector(f,initYscale2,f)
					else:
						self.Ob.pose[holdName].size=Vector(0,1,0)
					
					#Keying hold bone quat and size
								
					self.Ob.pose[holdName].insertKey(self.Ob.arm_ob,self.Ob.iframe-1,Object.Pose.ROT,True)	#Note that it is iframe-1
					self.Ob.pose[holdName].insertKey(self.Ob.arm_ob,self.Ob.iframe-1,Object.Pose.SIZE,True)	
					
					#Resetting pose at current frame
					
					self.Ob.pose[holdName].size=initSize
					self.Ob.pose[holdName].quat=initQuat
					#Comment: A bit inefficient. Should really be set just before changing frame (hence not every iteration and every step between frames).
					
		
	def TorqueMatrix(self,taulist,localFlag,InitC=None,InitL=None):
		columnList=[]
		
		if self.Ob.dynAndObjectFlag:
			columnList.extend(self.objectColumns(taulist,localFlag,InitC,InitL))
		
		if self.Ob.dynamicFlag:
			columnList.extend(self.dynamicColumns(taulist,localFlag,InitC=InitC,InitL=InitL))	
		
		columnList.extend(self.holdColumns(localFlag))
		
		return TransposeList(columnList)
		
	
	def holdColumns(self,localFlag):
		#Returns list of lists of how hold bone forces affect:
		#A. The entire body center of mass and angular momentum
		#B. The dynamical bones torque
		#C. The displacement from holdbone to target (this is just zeroes).
		#The matrix (list of lists) that is returned is typically going to be a part of a larger matrix
		#later on. The number of columns in the returned list is 3*nHoldActive where nHoldActive
		#is the number of active hold bones.
		
		columnList=[]
		for holdName in self.HoldBoneDict:
			
			initYscale=self.Ob.pose[holdName].size[1]
			if initYscale>0.6:	#If not, the bone is considered shut off as a hold bone.			

				targetName=self.HoldBoneDict[holdName][0]
				parentName=self.HoldBoneDict[holdName][1]	#This may well be None.	
				
				#if targetName in self.Bones:
				#	bTarget=self.Bones[targetName]
				
				for i in [0,1,2]:
					
					column=[]
					if targetName in self.Bones:
						#force=bTarget.mat[i]			#The test force coming from the hold bone, affecting the target bone.
						force=self.Bones[targetName].mat[i] 	#The test force coming from the hold bone, affecting the target bone.
					else:
						force=self.Ob.mat[i]	#Comment: One would perhaps have liked to use the matrix of the 
												#target bone, but when this is not a physical bone - that matrix is not stored,
												#so we use the body matrix instead.
										
					####################################################################
					#The hold force effect on the object center of mass and angular momentum
					
					if self.Ob.dynAndObjectFlag:
						if not parentName==None:					#If the holdbone has a parent in the (branch) we assume that there is no NET object force or torque effect from the hold bone force.
							column.extend([0,0,0,0,0,0])	#Three zeroes for the object center of mass, and three zeroes for the object angular momentum.
						else:							
							#poseHead=(self.Ob.pose[holdName].poseMatrix*self.Ob.arm_mat).translationPart()
							poseHead=self.HoldBoneDict[holdName][2]	
							Delta=poseHead-self.Ob.C	#Note that we use both holdBone head position and boneGroup center of mass are from the previous step - as they should.
														#Because that is where the holdForce, calculated at the current frame, resides.	
							
							#The hold bone force effect on the object force/center-of-mass (really it is the target that is affected sort of)
							
							kvec=-force * self.Ob.dt * self.Ob.dt / self.Ob.M	#Setting a hold force can be thought of as altering the object target center of mass.
																				#So TargetC depends linearly on the hold force.
																				#The corresponding components in TargetC-InitC on the right hand side
																				#of the equation system is shifted to the left hand side, which introduces a minus sign. 
							if localFlag:
								kvec=self.Ob.mat*kvec			#Converting kvec to the object/bodybone space.						
							
							column.extend(kvec)
							
							#The hold bone force effect on the object torque/angular momentum 
							
							Dtau=CrossVecs(Delta,force)
							kvec=-Dtau*self.Ob.dt			#Setting a hold force can be thought of as altering the object torque hence affecting the target angular momentum.
															#So TargetL depends linearly on the hold torque.
															#The corresponding components in TargetL-InitL on the right hand side
															#of the equation system is shifted to the left hand side, which introduces a minus sign.	
							
							if localFlag:
								kvec=self.Ob.mat*kvec			#Converting kvec to the object/bodybone space.
							
							column.extend(kvec)
					
					##################################################################
					#The hold force effect on the dynamical bones torque
					
					if self.Ob.dynamicFlag:
						
						#If the target bone is part of the dynamical branch in question.
						
						if targetName in self.Bones:	
							dynColumn=self.getHoldApparentTorques(force, targetName,localFlag)
										
							if parentName!=None:				#If the hold bone has a parent, it will be affected by a counter-force
								dynColumn2=self.getHoldApparentTorques(-force,parentName,localFlag)
								
								#Adding the effects of the force and the counterforce on the dynamical bones (this may result in cancellations).
								
								for ii in range(len(dynColumn)):
									dynColumn[ii]+=dynColumn2[ii] 	
							
							column.extend(dynColumn)
						
						
						#If the target bone is NOT part of the dynamical branch in question.
						
						else:
							if parentName!=None:				#If the hold bone has a parent, it will be affected by a counter-force
								dynColumn=self.getHoldApparentTorques(-force,parentName,localFlag)
							else:
								dynColumn=[]
								for name in self.Bones:
									b=self.Bones[name]
									for m in b.freeIndexes:
										dynColumn.append(0)
							
							column.extend(dynColumn)
						
					##################################################################
					#The hold force effect on the displacement (there is no immediate such effect so zeroes are added)
					
					for holdName2 in self.HoldBoneDict:
						column.extend([0,0,0])
					
					#Adding column to column list
					
					columnList.append(column)
				
		return columnList
	
	
	#Method concerning object evolution
	
	def objectColumns(self,taulist,localFlag,InitC=None,InitL=None):
		#Returns list of lists for how object rotations and translations affect object center of mass,
		#object angular momentum about the center of mass, apparent torques on dynamical bones, and 
		#holdBones distances to targets. For example, the first list gives the effect of a translation along the object/bodyBone
		#local x-direction, and the fourth list gives the the effect of a rotation of the object/bodyBone about the object/bodyBone
		#local x axis (for the initial object/bodyBone matrix)
		#
		#Comment: currently we need branchFlag=False for this to work properly.
		#Perhaps the body bone freedom should be a freedom of every branch however, 
		#can be handy for throwing stuff for example.
		#Then for example self.Ob.ObjectBones.UpdatePrelMatAndC()
		#goes to self.UpdatePrelMatAndC() for example.
		
		columnlist=[]
			
		self.Ob.UpdateLocRot()
		
		if InitC==None or InitL==None:
			self.Ob.ObjectBones.UpdatePrelMatAndC()
			InitC=self.Ob.ObjectBones.GetPrelC()
			InitL=self.Ob.ObjectBones.GetPrelL(InitC)
				
		InitLoc=self.Ob.GetLoc()
		InitMat=self.Ob.GetMat()
		
		for shift in ['Translation','Rotation']:
			
			for i in [0,1,2]:
				column=[]
				
				#Performing the shifts
				
				if shift=='Translation':
					deltaFactor=self.Ob.deltashift
					shiftvec=InitMat[i]*deltaFactor
					self.Ob.SetLoc(InitLoc+shiftvec)	
				else:
					deltaFactor=self.Ob.deltaangleOb
					rotvec=InitMat[i]*deltaFactor
					newmat=RotateMat(InitMat,rotvec)
					self.Ob.SetMat(newmat)
				
				self.Ob.UpdateLocRot()	
				
				#The effect of the shift on the center of mass and the angular momentum.
					
				self.Ob.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
				PrelC=self.Ob.ObjectBones.GetPrelC()
				PrelL=self.Ob.ObjectBones.GetPrelL(PrelC)
				
				kvecC=PrelC-InitC
				kvecL=PrelL-InitL
				
				if localFlag:
					kvecC=self.Ob.mat*kvecC
					kvecL=self.Ob.mat*kvecL
					
				column.extend(kvecC/deltaFactor) 
				column.extend(kvecL/deltaFactor) 
			
				#The effect on the dynamical bones		
						
				j=0
				self.UpdateTorqueApparent()	   #Based on current bone chain configuration, updates .torqueApparent=(b.getSpin()-b.spin)/dt-b.torqueDynamic, for all bones
											   #Comment: This step can be sped up if we assume that all dynamical bones just follows the translation.
											   #Identical to how the downstream calculation is made for dynamicFast=True option.
				for name2 in self.Bones:
					
					kvec=(self.Bones[name2].torqueApparent-taulist[j])/deltaFactor	#The net effect of the variation (the derivative)
					
					if localFlag:
						kvec=self.Bones[name2].mat*kvec
					
					#Adding the influence vector to the column
					
					for m in self.Bones[name2].freeIndexes:	
						column.append(kvec[m])
					
					j=j+1
				
				#The effect on the shift of any hold bones relative to their targets
				
				self.addHoldDeviations(column,deltaFactor,localFlag)
				
				# Appending column
				
				columnlist.append(column)
				
			if shift=='Translation':
				self.Ob.SetLoc(InitLoc)
			else:
				self.Ob.SetMat(InitMat)
			
		# Finalizing
		
		self.Ob.UpdateLocRot()		#Comment: This may be redundant.
		
		return columnlist

	
	############## Applying the rotations ###########################################################
	
	def ApplyRotations(self,anglelist):
		#Given a list of angles [bone1Xangle, bone1Yangle, bone1Zangle, bone2Xangle, ...]
		#where the rotations are relative to each bones own current space - rotates all dynamic bones accordingly.
		#Parent rotations will automatically propagate to children rotations (relative to world space).
		
		#Comment: A bit inefficient. Rotates (by zero) even bones that are rigid for example.
		
		j=0
		for name in self.Bones:
			b=self.Bones[name]

			matInit=b.getLocalMatrix()				#The local matrix after constraints	
			drotvec=anglelist[j]*matInit[0] + anglelist[j+1]*matInit[1] + anglelist[j+2]*matInit[2] 	
			self.Ob.pose[name].quat=RotateMat(matInit,drotvec).toQuat()
			j=j+3
			
		self.Ob.arm_ob.getPose().update()		#This insures that poseMatrix is updated as well, as I understand it
		
	
	################### Methods relating to dynamicFast=True ########################################
	
	def TorqueMatrixAlternative(self,localFlag):
		return self.DynTorqueMatrixAlternative(localFlag)
	
	
	def DynTorqueMatrixAlternative(self,localFlag):
		#This function is similar in purpose to TorqueMatrix() 
		#but much faster (for a 10 bone chain the total simulation speed goes up by roughly a factor of 2).
		#It is called when dynamicFastFlag=True
		# 
		#For every bone B one considers the effect of a small rotation around
		#each of he base axes of the bone. Bones that are downstream of B
		#are assumed to be shifted in world space as much as the tail of bone B is shifted,
		#but not rotated in any way. So one has kind of momentarilly shut off 
		#inherite rotation from parent. Note that this assumption may violate constraints in the chain 
		#of dynamical bones.
		#From the assumed shift the change of the dynamical forces 
		#(acting at children heads on their parent), both for bones downstream of B and
		#directly upstream from B can be deduced. From these changes the necessary
		#change of the internal torque on each bone (downstream or directly upstream)
		#is calculated. The effects, after dividing with the angle of change, are stored  
		#in a matrix that is returned.
		#
		#For example, the effect of an x-rotation of the i'th bone (starting on 0) 
		#can be found in column nr 3*i. The effect on all bones of a y-rotation can be
		#found in column 3*i+1.
		# 
		#Note that the matrix that is returned is not (generally) the same 
		#matrix as that returned by TorqueMatrix(), where children bones typically
		#rotate rigidly as one makes a small rotation of bone B, and constraints of 
		#various types are at least to some extent automatically acounted for.
		#Columns belonging to bones with no children should be identical if there
		#are no constraints though.
		#
		#Comment: Note that unlike TorqueMatrix() this method never calls for updates of the armature. 
		#With the correct input (of initial bone matrices among other things) this could
		#be done with compiled code fairly simply I would imagine. In combination with 
		#a compiled linear equation solver, dynamic evolution could be quite fast I imagine.
		
		dt=self.Ob.dt
		Kalt=[[0]*self.Nbones*3 for i in range(self.Nbones*3)]		#Initializing list-matrix of zeroes
																	#Strictly speaking, for a single chain of dynamical bones, this could be done just once before this loop is called.
		for name in self.Bones:
			b=self.Bones[name]
			
			init_mat=b.getMat()		
			init_spin=b.getMatSpin(init_mat)
			
			for i in [0,1,2]:
				
				mat=RotateMat(init_mat, init_mat[i]*deltaangle)
				dc=b.crel*(mat-init_mat)					#How much the center of mass moved in world space
				dforce=b.mass*dc/(dt*dt)					#Extra force on head necessary to shift own center of mass.
				for childname in b.dynChildren:
					bc=self.Bones[childname]				
					bc.deltaHead=bc.parentsHeadToOwnHead*(mat-init_mat)	#World delta vector for the head of the child.
					dforce+=(bc.mass+bc.downstreamMass)*bc.deltaHead/(dt*dt)
			
	
				#The influence on the bone itself
				
				dtau=(b.getMatSpin(mat)-init_spin)/dt
					
				for childname in b.dynChildren:		#Looping through the first dynamic children
					bc=self.Bones[childname]
					dtau+=CrossVecs(bc.massVecToParent*b.mat, bc.deltaHead/(dt*dt))
					
				dtau+=CrossVecs(b.crel*b.mat,dforce)							#Extra torque needed because the headforce was changed
				
				if localFlag:
					dtau=b.mat*dtau
				
				for vecindex in [0,1,2]:										#Looping through dtau_x, dtau_y and dtau_z
					Kalt[b.boneIndex*3+vecindex][b.boneIndex*3+i]=dtau[vecindex]/deltaangle
				
				#The influences downstream of the bone
				
				for childname0 in b.dynChildren:	#Looping through first children
					bc0=self.Bones[childname0]
					delta0=bc0.deltaHead
					
					for childname in bc0.allDynChildrenAndSelf:
						bc=self.Bones[childname]
						dtau=CrossVecs(bc.massVec*bc.mat,delta0/(dt*dt))	
					
						if localFlag:
							dtau=bc.mat*dtau
						
						for vecindex in [0,1,2]:									#Looping through dtau_x, dtau_y and dtau_z
							Kalt[bc.boneIndex*3+vecindex][b.boneIndex*3+i]=dtau[vecindex]/deltaangle
					
				#The influences upstream of the bone
				
				bp=b
				while not bp.dynParent==None:	#dynParent refers to dynamical parent within self.Bones
					bc=bp
					bp=self.Bones[bc.dynParent]
					
					head2head=bc.parentsHeadToOwnHead*bp.mat			#World space head to tail vector for parent in question. Note that .parentsHeadToOwnHead is in parents local rest space.
					dtau=CrossVecs(head2head,dforce)							
					
					if localFlag:
						dtau=bp.mat*dtau
					
					for vecindex in [0,1,2]:									#Looping through dtau_x, dtau_y and dtau_z
						Kalt[bp.boneIndex*3+vecindex][b.boneIndex*3+i]=dtau[vecindex]/deltaangle
						
		return Kalt
	
	
	def SetMatricesAlternative(self,anglelist):
		#Given a list of angles [bone1Xangle, bone1Yangle, bone1Zangle, bone2Xangle, ...]
		#where the rotations are relative to each bones own current space - 
		#rotates the current mat of each dynamic bone and writes the result to .prel_mat (both of which are relative to world space) 
		#It is called by ApplyRotationsAlternative() when dynamicFastFlag=True
		
		j=0
		for name in self.Bones:
			b=self.Bones[name]
			initMat=b.getMat()
			drotvec=anglelist[j]*initMat[0] + anglelist[j+1]*initMat[1] + anglelist[j+2]*initMat[2] 	
			b.prel_mat=RotateMat(initMat,drotvec)
			j=j+3
			
	
	def ApplyRotationsAlternative(self,anglelist):
		#Given a list of angles [bone1Xangle, bone1Yangle, bone1Zangle, bone2Xangle, ...],
		#where the rotations are relative to each bones own current space,
		#one rotates each bone, about its world rotation state given by.prel_mat,
		#which in turn is set by this funciton through calling SetMatricesAlternative(),
		#to find the new world space matrix of the bone. Then one calculates the resulting local matrices -
		#based on the new bone world space matrix and the new parent world space matrix.
		#The resulting local rotations are keyed, and the armature updated.
		#
		#In other words - you specify the desired rotations relative to world space,
		#and the rotations of parents are not propagated to their children.
		#
		#It is called when dynamicFastFlag=True
		
		self.SetMatricesAlternative(anglelist)		#Sets .prel_mat for the dynamic bones according to anglelist
			
		j=0
		for name in self.Bones:
			b=self.Bones[name]
				
			############# Getting the local matrix corresponding to the bone .prel_mat
			
			m0=b.prel_mat				#The desired world space matrix
			m1=b.restLocalMatrix		#The matrix of the bone restspace (3x3) relative to parent (this matrix is constant)
			
			# Getting the parent world matrix
			
			if b.parent==None:
				 m2=Matrix([1,0,0],[0,1,0],[0,0,1])*self.Ob.arm_mat.rotationPart()	#Meaning that armature space counts as the parent
			elif b.parent in self.Bones:
				m2=self.Bones[b.parent].prel_mat
			else:										    				#The parent is not a PhysBone.
				m2=(self.Ob.pose[b.parent].poseMatrix * self.Ob.arm_mat).rotationPart()
				
			# Going to local space
				
			m1a=m1*m2														#The bones restspace matrix relative to world space
			matLocal=m0*(m1a.transpose())
			
			######### Setting it up
			
			self.Ob.pose[name].quat=matLocal.toQuat()   
			
		self.Ob.arm_ob.getPose().update()	#This insures that poseMatrix is updated as well, as I understand it
		
	
	################ Remaining methods #############################################################################
	
	def SetInterpolatedRot(self):
		#Sets preliminary rotations of the bones in self.Bones, based on each bones .localMatrix, .omegaLocal and .omegaLocalDot
		
		dt=self.Ob.dt
			
		for name in self.Bones:
			b=self.Bones[name]
			
			#Note that omega_rel is the angular velocity relative to the restspace
			#of the bone - which in turn has a rotation if the parent rotates.
			#So omega_rel is Not just a projection of the world space bone rotation
			#onto the bone axes.
			
			if b.hasFreedom:
				
				if self.dynInterpolateAccFlag:
					rotVec=dt*(b.omegaLocal+dt*b.omegaLocalDot)		#Initial guesstimated local rotation
				else:
					rotVec=dt*b.omegaLocal							#Initial guesstimated local rotation
				
				#Comment: For many smooth scenarios, using dynInterpolateAccFlag=True will most of the time give a better estimate
				#of the correct rotations. However, if bones are kind of jumping about (like at the onset of a numerical instability),
				#dynInterpolateAccFlag=False will be a better option. Also, interpolateAccFlag=False has the advantage that complete initial conditions
				#can be given in two frames rather than three.
				
				if b.hasConstraint:
					#Insuring that the interpolation rotation is not violating constraints.
					#If there is a corresponding Blender constraint, 
					#there is no point to this step.
					#If there is not however, without this step, constraints may soon be violated during evolution
					#(presumably due to a numerical error that is allowed to grow).
					#Note that this is no guarantuee however, it only limits the local interpolation rotation of 
					#the current evolutionary step. A blender constraint will
					#limit the NET rotation, which is more restrictive. 
					
					mat=b.localMatrix					#NOTE: This has nothing to do with ghostbones, it is the bone matrix relative to the bone rest space.
					for m in b.constrainedIndexes:
						rotVec-=(rotVec*mat[m])*mat[m]
					
				prel_localMatrix=RotateMat(b.localMatrix,  rotVec)
				self.Ob.pose[name].quat=prel_localMatrix.toQuat()
			
			elif b.isRigid:
				self.Ob.pose[name].quat=b.localMatrix.toQuat()		#Then we overwrite the current rotation by the stored rotation from last update.
				#Note: The local matrix for a rigid bone is only updated at the start frame (and jloop=0)
				
		self.Ob.arm_ob.getPose().update()
	
	
	def SetDynamicKeys(self):
		for name in self.Bones:
			b=self.Bones[name]
			if b.hasFreedom or b.isRigid:	#If it is rigid, it overwrites existing keys
				self.Ob.pose[name].insertKey(self.Ob.arm_ob,self.Ob.iframe,Object.Pose.ROT,True)
			
			#Comment: The True argument, at the end, is a fast flag that insures that IPO's are not recalculated.
			#I think this means that bezier curves are not calculated. When StepsPerFrame=1, this should make
			#no difference for the operation of the script, because there is a key added for every frame, and
			#Bezier curves are esentilly not used. Using the fast option can speed up the entire script time by 30%,
			#which is kind of an astonishing improvement. As the script is working at the 
			#writing of this comment (100317),setting the True option seems to have no impact on 
			#the operation at all, even if StepsPerFrame>1, it just makes everything faster.
			#For future versions where one may do more refined in between frames calculations,
			#it might have an actual effect on the evolution if the fast flag is True or not however.
			#Comment: I wonder why one has to send the object as argument.
			
	
	def possiblyInitializeAlmostRigidMatrix(self,taulist,localFlag,InitC=None,InitL=None):
		#If appropriate, updates the fixed matrix used for the Almost Rigid option. 
		
		if self.rigidKaltInverseDict==None:
			self.rigidKaltInverseDict={}
		
		if self.rigidKmatInverseDict==None:
			self.rigidKmatInverseDict={}
		
		
		initializeFlag=False
		s=str(self.Ob.stepsPerFrame-1)
		
		if self.dynamicFastFlag:
			if not s in self.rigidKaltInverseDict:
				self.rigidKaltInverseDict[s]=[None,None,None]
				
			if self.rigidKaltInverseDict[s][self.dampMode]==None:
				initializeFlag=True
		
		if self.dynamicFastFlag==False:
			if not s in self.rigidKmatInverseDict:
				self.rigidKmatInverseDict[s]=[None,None,None]
				
			if self.rigidKmatInverseDict[s][self.dampMode]==None:
				initializeFlag=True
									
	
		if self.Ob.updateAlmostRigidFlag or initializeFlag:
			
			print "Branch ",str(self.branchIndex)+":"
			print "INITIALIZING ALMOST-RIGID MATRIX INVERSE."
			
			if self.dynamicFastFlag:							
				Kalt=self.TorqueMatrixAlternative(localFlag)
				self.rigidKaltInverseDict[s][self.dampMode]=invertMat(Kalt)
			else:
				Kmat=self.TorqueMatrix(taulist, localFlag,InitC=InitC,InitL=InitL)
				self.rigidKmatInverseDict[s][self.dampMode]=invertMat(Kmat)
			
			print "DONE: INITIALIZING ALMOST-RIGID MATRIX INVERSE."
			
			self.firstStepFlag=False	#Comment: we could set this ouside of the if case, but it is currently relevant for the stuff within the if-case.
		
		#Comment: One might feel that a call of the above method from within the Evolve method, is misplaced,
		#and belongs in an initialization phase, prior to the evolution. But it is kind of convenient to 
		#call it from within the Evolve method partially since one of thea TorqueMatrix methods needs a taulist - which is naturally 
		#evaluated at the first step of the evolution.
			
	
	def cutTheCrap(self, deltalistIn):
		#Given a list of torque components, pops out the elements that correspond to constrained 
		#bone directions and returns the reduced list of torque components.
		
		deltalistOut=[]
		boneIndex=0
		for name in self.Bones:
			b=self.Bones[name]
				
			for m in b.freeIndexes:
				deltalistOut.append(deltalistIn[boneIndex*3+m])
				
			boneIndex+=1
		
		return deltalistOut	
	
	
	def insertTheCrap(self,anglelistIn):
		#Given a list of (reduced) rotation angles, inserts zeroes at the positions corresponding
		#to constrained axles of rotation. It is sort of the opposite of the .cutTheCrap method.
		
		anglelistOut=[]
		index=0						#Initialize counter for index in anglelistIn
		for name in self.Bones:
			b=self.Bones[name]
			
			angleTriplet=[0,0,0]	#Initializing zero rotations
			
			for m in b.freeIndexes:
				angleTriplet[m]=anglelistIn[index]
				index+=1
				
			anglelistOut.extend(angleTriplet)
	
		return anglelistOut	
				
	
	def EvolveDynamics(self):
		#The main method of dynamic evolution (also evolves object if dynaAndObjectFlag=True).
		
		if not (self.Bones=={} and self.Ob.dynAndObjectFlag==False):
			
			localFlag=True		#True: Makes torque matrices and target torques refer to the 
								#local space of each bone (at the step before current).
			
			#Comment: localFlag=True makes Almost-Rigid evolution much more stable and is necessary
			#for constrained joints (temp txt). The extra matrix multiplications connected with localFlag=True 
			#has virtually no impact on the evolution time - so it is left as standard currently.
						
			################# One step forward evolution ################################
							
			for k in range(self.dynamicNiteration):
				
				self.Ob.arm_ob.getPose().update()		#Comment: This update may be redundant, at least for more than one iteration.	
				
				if k==0 or self.dampMode==1 or self.dampMode==2:
					self.UpdateTargetTorques()		#Updates target torques due to external torques, spring torques, damping torques etc.
				
				
				#For object evolution.
				
				objectDeltalist=[]	#Subsequently this is inserted at the beginning of deltalist
				
				if self.Ob.dynAndObjectFlag: 
					
					self.Ob.UpdateLocRot()
			
					InitLoc=self.Ob.GetLoc()
					InitMat=self.Ob.GetMat()
					
					self.Ob.ObjectBones.UpdatePrelMatAndC()	#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
					
					InitC=self.Ob.ObjectBones.GetPrelC()
					InitL=self.Ob.ObjectBones.GetPrelL(InitC)
					
					DeltaC=self.Ob.targetC-InitC
					DeltaL=self.Ob.targetL-InitL	
					
					if localFlag:
						DeltaC=self.Ob.mat*DeltaC
						DeltaL=self.Ob.mat*DeltaL
					
					objectDeltalist.extend(DeltaC)
					objectDeltalist.extend(DeltaL)
					
				else:
					InitC=None
					InitL=None
				
				#For the dynamical bones 
				
				self.UpdateTorqueApparent()		#Calculates the current apparent net torques (excluding torques from joint forces from parents and children), 
												#based on the current rotation and position of the bones (i.e the change of the spin)
												#and writes it to the individual dynamical bone's .torqueApparent
				
				#Calculation the necessary shifts of each bones current apparent torque (this torque excludes the torques due to joint forces), 
				#to each bones target torque. Also assembles the the current apparent torques in a list.
			
				deltalist=[]	#Inititing list of torque components. Will be right hand side of equation system
				taulist=[]		#Initiating list of torque vectors
					
				for name in self.Bones:
					b=self.Bones[name]
					DeltaTau=b.torqueTarget-b.torqueApparent
					
					deltalist.extend(DeltaTau)
					taulist.append(Vector(b.torqueApparent))	#Vector is to unlink	
			
				#Comment: Deltalist is a list of differences between the apparent internal torques
				#judged by the current position and rotation of each bone,
  				#and the net actual torque given by the spring joints etc.
				#Taulist is just a list of the apparent current torque vectors.	

				#Modifying deltalist to correspond to the torque errors in the local space of each bone.
			 	#Note that taulist should still be expressed relative to world space.
							
				if localFlag:
					for name in self.Bones:
						b=self.Bones[name]
						index=b.boneIndex
						delta=Vector(deltalist[index*3:index*3+3])
						deltaLocal=b.mat*delta
						deltalist[index*3:index*3+3]=deltaLocal
				
				####### Reductions due to given constraints
				
				deltalist=self.cutTheCrap(deltalist)	#Note that this is done after going to local coordinates - as it should.
				
				####### For holdBones. Writing position difference to self.HoldBoneDict[holdName][3], and adding to the deltalist 
					
				for holdName in self.HoldBoneDict:
					
					initYscale=self.Ob.pose[holdName].size[1]
					if initYscale>0.6:
													
						targetName=self.HoldBoneDict[holdName][0]
						
						pos1=(self.Ob.pose[targetName].tail.resize4D()*self.Ob.arm_mat).resize3D()
						pos2=(self.Ob.pose[holdName].poseMatrix*self.Ob.arm_mat).translationPart()	#The position of the head of the HoldBone
						deltaPos=pos2-pos1
						
						self.HoldBoneDict[holdName][3]=deltaPos		#The initial world delta to hold vector
						
						if localFlag:
							if targetName in self.Bones: 
								deltaPos=self.Bones[targetName].mat*deltaPos
							else:
								deltaPos=self.Ob.mat*deltaPos			#When dynAndObjecFlag is True, and the bone target is not spring/dynamic - we use the object matrix.
						
						deltalist.extend(deltaPos)
					
				######## Adding the object deltas on top of the other deltas
				
				objectDeltalist.extend(deltalist)
				deltalist=objectDeltalist
				
				####### Initializing matrixes for Almost Rigid mode if appropriate ##########
				
				if self.Ob.almostRigidFlag and self.firstStepFlag:
					self.possiblyInitializeAlmostRigidMatrix(taulist,localFlag,InitC=InitC,InitL=InitL)
					
				########## Getting angles and setting rotations #########################
				
				if self.dynamicFastFlag:
					if self.Ob.almostRigidFlag:
						s=str(self.Ob.stepsPerFrame-1)
						anglelistalt=multMatVec(self.rigidKaltInverseDict[s][self.dampMode],deltalist)						
					else:			 
						Kalt=self.TorqueMatrixAlternative(localFlag)
						anglelistalt=rjsolve(Kalt,deltalist)
						
					#Note: Here holdBoneCoumns and objectCoumns needs to be added for the and dynAndObjectFlag=True
					#to work with the dynamicFastFlag. Should be simple to implement - but it is not yet implemented.   
					
					forcelist=anglelistalt[-len(self.Ob.HoldBoneDict)*3:]	#For hold bones
			
					anglelistalt=self.insertTheCrap(anglelistalt)			#Inserting 0 rotations for the non-varied directions
					self.ApplyRotationsAlternative(anglelistalt)
					
				else:
									
					if self.Ob.almostRigidFlag:
						s=str(self.Ob.stepsPerFrame-1)
						anglelist=multMatVec(self.rigidKmatInverseDict[s][self.dampMode],deltalist)
						
					else:
						Kmat=self.TorqueMatrix(taulist,localFlag,InitC=InitC,InitL=InitL)
						anglelist=rjsolve(Kmat,deltalist)
									
						
					nHold=0	 								#Number of active hold bones.		
					for dummyName in self.HoldBoneDict:		#Comment: Unsure of of if it should be self.Ob.HoldBoneDict after all?
						initYscale=self.Ob.pose[dummyName].size[1]
						
						if initYscale>0.6:	#If not, the bone is considered shut off as a hold bone.			
							nHold+=1
					
					forcelist=anglelist[-nHold*3:]	#For hold bones
					
					if self.Ob.dynAndObjectFlag:
						
						shiftVec=Vector(anglelist[0:3])		#The local displacement (relative to object/bodyBone initial matrix)
						rotVec=Vector(anglelist[3:6])		#The local rotation 
						
						shiftVec=shiftVec*InitMat			#World displacement
						rotVec=rotVec*InitMat				#World rotation vector
					
						newmat=RotateMat(InitMat,rotVec)
						
						self.Ob.SetMat(newmat)
						self.Ob.SetLoc(InitLoc+shiftVec)		
						
						anglelist=anglelist[6:]			
					
					anglelist=self.insertTheCrap(anglelist)		#Inserting 0 rotations for the non-varied directions				
					self.ApplyRotations(anglelist)				
			
					#Comment: The name anglelist is misleading when dynAndObject=True.
				
				if k==self.dynamicNiteration-1:       #Last iteration
					self.setHoldBoneForce(forcelist)
				
	
#################################################################################################
#################################################################################################
##############################  SpringBoneDict  #################################################						

class SpringBoneDictClass(DynBoneDictClass):
	#Contains attributes and methods for including (into the evolution) spring and damping torques at the joints between 
	#SpringBones in a group of spring bones.
	
	type='SpringBoneDict'	#Indicator for the type of an instance of this class
	dampMode=None			#The mode of damping 0,1,2. 0 is the fastest, 2 is the most stable.


	############### Intialization ######################################################################
	
	def __init__(self,Bones,PhysOb):
		
		self.Bones=Bones
		self.Ob=PhysOb
		self.init_DynBoneDict()					#This class initiation
		self.init_SpringBoneDict()

	
	def init_SpringBoneDict(self):
		
		if not self.Bones=={}:
			if self.Ob.gravity:
				self.setOffsetGravityTorque()		#Writes to each bone .torqueOffsetRest, with respect to each bones rest space, the net bone spring torque needed to keep gravity at bay. 
				self.setLocalHeadTorqueRest()		#Writes to .localHeadTorqueRest, with respect to each bones rest space, the net bone spring torque needed to keep gravity at bay.
				

	############### Methods  ###########################################################################
		
	def UpdateTargetTorques(self):
		#Comment: Overwrites the parent class UpdateTargetTorques.
		#
		#Updating target torques for the dynamical bones. If the bones are pure dynamical bones
		#it writes the .externalTorque to the .targetTorque. If the bones are spring bones 
		#it includes spring torques and damp torques. Note that if dampMode=1 or dampMode=2
		#the damping at 1 frame before current depends directly on the bone pose at the current frame.
		
		
		if self.dampMode==0:
			do=None			#Meaning that the already set .midframeDampHeadTorque will be used 
			
		if self.dampMode==1 or self.dampMode==2:
			self.UpdateMidframeDampHeadTorques()
			self.UpdatePrelMidframeDampTorques()		#Writes net current damp torque for to .prelMidframeDampTorque for all springbones
														#Using prel to not overwrite .midframeDampTorque of the previous midframe just yet.
									
		#Comment: Here is where you would make an update to .torqueExternal if you have something like flight-bones
		#and you would like velocities to be evaluated 1 frame before current rather than 1.5 frame before current (as is the easy way).
		#Note however that typically - if the velocities are to be evaluated 1 frame before current, then this should affect the getForce()
		#wich affects the internal dynamical (push-pull between bones) torque. So it is not just here that 
		#one would change the code.
		
		for name in self.Bones:
			
			b=self.Bones[name]
					
			if self.dampMode==0:
				dampTorque=b.midframeDampTorque										#Using the damping torque from 1.5 frames before current.
			elif self.dampMode==1:
				dampTorque=(b.prelMidframeDampTorque+b.midframeDampTorque)/2		#Forming the average damping torque from 0.5 and 1.5 frames before curret to approximate the damping 1 frame before current
			elif self.dampMode==2:
				dampTorque=b.prelMidframeDampTorque									#Using the damping torque from 0.5 frames before current to approximate the damping 1 frame before current
							
			b.torqueTarget=b.SpringTorque + dampTorque + b.torqueExternal			#Note that .SpringTorque is the net spring torque from both the head of the bone and from the heads of the children.
			
			
	def dynamicColumns(self,taulist,localFlag,InitC=None,InitL=None):
		#Note: Currently this method overwrites the parent class corresponding method.		
		#
		#If localFlag is true, one uses the bone matrices from the previous step (.mat),
		#to rotate the kvec (torque influence on a bone from the rotation of some (other ) bone),
		#to local bone coordinates.
		#Note that taulist here is still the world space torques.
		#
		#For further understanding of the operation of this method see the TorqueMatrix() method of the parent class,
		#and further comments in this script and documentation.
		
		
		#Storing the initial midframe damp torque for all spring bones, if dampMode=1 or dampMode=2
		
		dampMode=self.dampMode
		
		if dampMode==1 or dampMode==2:
			for name in self.Bones:
				b=self.Bones[name]
				b.initialMidframeDampTorque=Vector(b.prelMidframeDampTorque)	#Just copying
				
				if self.plainBranchFlag:
					b.initialMidframeDampHeadTorque=Vector(b.midframeDampHeadTorque)
					
					if self.hasConstraints:
						b.initialTorqueTarget=b.torqueTarget
		
		columnlist=[]
		for name in self.Bones:
			b=self.Bones[name]
			
			#Initial conditions
					
			b=self.Bones[name]
			matInit=b.getLocalMatrix()				#The local matrix after constraints
			quatInit=Quaternion(matInit.toQuat())	#Note that this may be different from the bone quat - because the quat is not including constraints - but matInit is	
				
			#Making small variations of the bone angles and registering the effects. 
			
			for i in b.freeIndexes:  	#Comment: freeIndexes is typically [0,1,2] but can for example be [1,3] or [] as well.
				
				column=[]
				self.Ob.pose[name].quat=RotateMat(matInit,matInit[i]*deltaangle).toQuat()		#The variation rotation of the bone
				self.Ob.arm_ob.getPose().update()
				
				#For object evolution.
				
				if self.Ob.dynAndObjectFlag: 
					if name in self.Ob.ObjectBones.Bones: 			#The bone could still affect the center of mass via constraints and stuff though. 
						self.Ob.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
						DeltaC=self.Ob.ObjectBones.GetPrelC()-InitC			
						DeltaL=self.Ob.ObjectBones.GetPrelL(InitC)-InitL		
						
						if localFlag:
							DeltaC=self.Ob.mat*DeltaC
							DeltaL=self.Ob.mat*DeltaL
						
						column.extend(DeltaC/deltaangle)
						column.extend(DeltaL/deltaangle)
						
						
					else:
						column.extend([0,0,0,0,0,0])
				
				#Comment above: Some room for speedups. If one has stored the individual bones initial
				#contributions to  C and L, for most cases of interest, we need only consider
				#the bones downstream of the bone and not every bone.
				#But for now, this is a simple way of getting started.
				
				if self.Ob.dynamicFlag:
										
					if dampMode==1 or dampMode==2:
						
						if not self.plainBranchFlag:
							
							#If there are constraints, then .torqueTarget must be updated for all bones before calling UpdateTorqueApparent(). 
							#If there are no constraints, it is sufficient to just update the damping for all bones. 
							
							if self.hasConstraints:
								self.UpdateTargetTorques()
							else:
								self.UpdateMidframeDampHeadTorques()
								self.UpdatePrelMidframeDampTorques()		#Writes net current damp torque to .prelMidframeDampTorque for all springbones
												
						else:
							#Assuming that the branch in question is plain (no IK constraints etc.), then varying
							#the bone will only affect the head damping torque of the bone bone itself (when plainBranchFlag==True).
							#Hence it is only the bone and its first parent whose net damping torques are affected.
							#So to speed things up, we just calculate these two torques. 
							#Note that the first lines below are esentially copied from self.UpdateMidframeDampHeadtorques().
							
							if b.hasFreedom:
								b.midframeDampHeadTorque=b.getMidframeDampHeadTorque()
								if b.hasConstraint:
									for m in b.constrainedIndexes:
										b.midframeDampHeadTorque-=(b.midframeDampHeadTorque*b.mat[m])*b.mat[m]
							else:
								b.midframeDampHeadTorque=Vector(0,0,0)
							
							deltaDampTorque=b.midframeDampHeadTorque-b.initialMidframeDampHeadTorque
							
							#If there are constraints, then target torques must be updated before calling UpdateTorqueApparent(). 
							
							if self.hasConstraints:
								
								#Using a minimalistic update instead of running self.UpdateTargetTorques(), to speed things up.
								
								b.torqueTarget=b.initialTorqueTarget + deltaDampTorque
								if b.dynParent!=None:
									bParent=self.Bones[b.dynParent]
									bParent.torqueTarget=bParent.initialTorqueTarget - deltaDampTorque
									
								#When the plainBranchFlag is set, we assume that all that changed as a consequaence of the one rotaton variation 
								#as concerned the target torques, is the damp torque of the bone of variation
								#and its parent. Unless plainBranch flag is True, there is also an effect of rotation of the damp torques of the children
								#bones. But for the plainBranch option, one can certainly speed up
								#this step, and just write to .torqueTarget for the bone and its parent here.	
								
								
					
					self.UpdateTorqueApparent()	   	#Based on current bone chain configuration, updates .torqueApparent. When there are no contraints, this is simply (b.getSpin()-b.spin)/dt-b.torqueDynamic, for all bones.
													#But when there are constraints, .torqueApparent depends on the target torques.	
					
					if self.plainBranchFlag and (dampMode==1 or dampMode==2) and self.hasConstraints:
						
						#Resetting torqueTargets so the altered torqe targets are not lingering when we come to the next bone.
						#Since for these settings, UpdateTargetTorques() is not called to reset things for the next bone.
						
						b.torqueTarget=b.initialTorqueTarget
						if b.dynParent!=None:
							bParent=self.Bones[b.dynParent]
							bParent.torqueTarget=bParent.initialTorqueTarget
					

					####### Registering the effect on the other bones		
							
					j=0
					for name2 in self.Bones:
						
						kvec=(self.Bones[name2].torqueApparent-taulist[j])/deltaangle	#The net effect of the variation (the derivative)
						
						#When dampMode=1 or dampMode=2, the damping torque depends on the rotation of the bone in question
						#at the current frame (step). Thus as one shifts the bone about its guesstimated rotation, the
						#right hand side of the corresponding equation is altered. Assuming the shift is linear (which we do assume),
						#there is thus a factor times the delta angle in the right hand side of the equation. 
						#So, we just move this term to the left hand side of the equation system (which introduces a minus sign),
						#and let the factor be a part of the koefficient for the bone and angle in question. Quite neat.
						
						if dampMode==1 or dampMode==2:
							
							if not self.plainBranchFlag:
								deltaDamp=self.Bones[name2].prelMidframeDampTorque-self.Bones[name2].initialMidframeDampTorque
							else:	
								if name2==name:
									deltaDamp=Vector(deltaDampTorque)
								elif name2==b.dynParent:
									deltaDamp=-Vector(deltaDampTorque)
								else:
									deltaDamp=Vector(0,0,0)
							
							if self.dampMode==1:
								kvec-=0.5*deltaDamp/deltaangle		#The factor of 0.5 is because the damping torque on the right hand side of the equations system is an average of current and previous damping torques.
							if self.dampMode==2:
								kvec-=deltaDamp/deltaangle
									
						#Transforming to local coordinates
						
						if localFlag:
							kvec=self.Bones[name2].mat*kvec
						
						#Adding the influence vector to the matrix.
						
						for m in self.Bones[name2].freeIndexes:		#Comment: A bit time inefficient. When there is no variation, we need not calculate kvec at all.
							column.append(kvec[m])
						
						#Note that for a parent with no freedom, as we vary the child damp bone, the extra damping
						#induced on the parent is not used here for anything. But it is still included 
						#from previously running UpdateTargetTorques() and UpdateTorqueApparent.
						
						j=j+1
				
				# Hold Bones: Adding the influence of the variation on the tip location of any bone targeted by a HoldBone
				# This added at at the end of each matrix column.
				
				self.addHoldDeviations(column,deltaangle,localFlag)
				
				# Appending column
				
				columnlist.append(column)
				
			self.Ob.pose[name].quat=quatInit		#Resetting bone rotation 
		
		# Finalizing
		
		self.Ob.arm_ob.getPose().update()			#This insures that poseMatrix is updated as well, as I understand it
		
		return columnlist
			
	
	###### Initialization of offset torques ###################################################
	
	def setOffsetGravityTorque(self):
		#For each bone b in SpringBones, calculates the net spring torque
		#(head spring torque minus all torques from the children) needed to keep the bone from starting to rotate
		#(due to gravity acting on all dynamical bones) when the whole armature is in its rest state.
		#The resulting torque with respect to the bone rest space, is written to b.torqueOffsetRest.
		#In other words it calculates minus the torque coming from the forces on the head and
		#from the childrens head for each bone when the armature is in rest. 
		
		for name in self.Bones:
			b=self.Bones[name]
			if b.offsetTorqueFlag:	
				restMat=self.Ob.arm_data.bones[name].matrix['ARMATURESPACE']	#Matrix of the bone restspace, for the relaxed armature pose, relative to armature
				restGravity=restMat*(self.Ob.g*Vector(0,0,-1))					#Projecting gravitational vector onto bone rest space for the relaxed armature pose
				
				#Note: It is assumed that the armature z-axis and the world z-axis coincide when the armature is unrotated!
				
				b.torqueOffsetRest=-CrossVecs(b.massVec,restGravity)  	#The rest torque in rest space
				
	
	def setLocalHeadTorqueRest(self):
		#For each bone b in SpringBones, calculates the head spring torque needed to keep the bone from starting to rotate
		#(due to gravity acting on all dynamical bones) when the whole armature is in its rest state.
		#The calculation is based on the .torqueOffsetRest which should be calculated by setOffsetGravityTorque() function before this
		#function is called.
		
		for name in self.ChildrenFirstList:
			if name in self.Bones:
				b=self.Bones[name]
			else:
				continue
			
			if b.offsetTorqueFlag:
				b.localHeadTorqueRest=Vector(b.torqueOffsetRest)
					
				for childname in self.ChildDynDict[name]:
					rel_mat=self.Ob.arm_data.bones[childname].matrix['BONESPACE']	#The restspace matrix (3x3) relative to parent
					tau_child=self.Bones[childname].localHeadTorqueRest	#Note that non-springbones are excluded - these have no offset torque. Maybe not what one wants in the long run though.		
					b.localHeadTorqueRest+=tau_child * rel_mat				#Transforming children head tau to parent space
				
				if b.springVec[1]<EPSILON:
					Draw.PupMenu(b.name+":You need a non-zero springVec[1] (y-direction along bone) to get torsion torque offset")
				
				
	def TorqueMatrixAlternative(self,localFlag):
		#Makes an addition to the matrix coming from DynTorqueMatrixAlternative().
		#This addition allows the option dynamicFastFlag=True to still work,
		#for spring and damp bones  with the options dampMode=1 or dampMode==2.
		#
		#The idea is that we consider the effect of small rotations of a bone,
		#for dynamicFast, this affects the damping of the first children of the bone
		#the bone itself and the first parent.
		#It is important to realize that the effect of the rotation on the damping of the children
		#is also immediately affecting the net damping torque on the bone itself
		#(which is head damping torque minus childrens heda damping torques.)
		#
		#Just like DynTorqueMatrixAlternative() there is no call to update the 
		#armature. We just assume that the children bones are folowing the position
		#of the parent head but without rotating.
		#
		#One could have integrated the damping effects from this method into the 
		#DynTorqueMatrixAlternative(), using if cases to exclude the effects for dampMode=0,
		#(as is the case for dynFast=False), but since there anyway are no updates
		#the overhead of looping throught the bones again here is slight - and it makes
		#the code more readable.
		
		Kalt=self.DynTorqueMatrixAlternative(localFlag)	
		dampMode=self.dampMode
		
		if dampMode==0:
			return Kalt
		else:
			
			#Comment: self.UpdateMidframeDampHeadTorques(),	writing interpolated bone configuration dampTorques to .midframeDampHeadTorque 
			#is assumed to be called prior to calling this function.	
			
			for name in self.Bones:
			
				b=self.Bones[name]
				matInit=b.getMat()		#Current (interpolated) world matrix.
					
				for i in [0,1,2]:
					
					rotationVector=matInit[i]*deltaangle
					
					#The influences on the children of the bone of variation (these matter for the bone of variation itself also)
					
					dtauChildren=Vector(0,0,0)	#Initiating the sum total of the variation of the torques on the children
					
					for childname in b.dynChildren:		#Not allDynChildren
						bc=self.Bones[childname]
						
						matChildInit=bc.getMat()		#World matrix
						matChild=RotateMat(matChildInit, -rotationVector)		#Note rotation by MINUS the variation bone rotation
						matChildRelRef=matChild * (bc.getMatRefWorld().transpose())
						
						dtau1=bc.getMidframeDampHeadTorqueArg(matChildRelRef) - bc.midframeDampHeadTorque 		
						dtauChildren+=Vector(dtau1)
						
						if localFlag:
							dtau1=bc.mat*dtau1
							
						for vecindex in [0,1,2]:
							if dampMode==1:
								Kalt[bc.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau1[vecindex]/deltaangle*0.5
							if dampMode==2:
								Kalt[bc.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau1[vecindex]/deltaangle
					
			
					#The influence on the variation bone itself
					
					mat=RotateMat(matInit, rotationVector) 
					matRelRef=mat * (b.getMatRefWorld().transpose()) 	#Projecting onto world reference matrix, to get the relative reference matrix		
					dtau0head=b.getMidframeDampHeadTorqueArg(matRelRef)-b.midframeDampHeadTorque
					dtau0=dtau0head-dtauChildren
					
					
					if localFlag:
						dtau0=b.mat*dtau0
					
					for vecindex in [0,1,2]:
						if dampMode==1:		
							Kalt[b.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau0[vecindex]/deltaangle*0.5		#Note minus sign, due to shifting sides of the equation system.
						if dampMode==2:
							Kalt[b.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau0[vecindex]/deltaangle			#Note minus sign, due to shifting sides of the equation system
					
							
					#The influence on the parent
					
					if b.parent in self.Bones:
						bp=self.Bones[b.parent]
						dtau2=-Vector(dtau0head)
						
						if localFlag:
							dtau2=bp.mat*dtau2
						
						for vecindex in [0,1,2]:
							if dampMode==1:
								Kalt[bp.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau2[vecindex]/deltaangle*0.5
							if dampMode==2:
								Kalt[bp.boneIndex*3+vecindex][b.boneIndex*3+i]-=dtau2[vecindex]/deltaangle
			
			###### Returning final modified alternative (dynamicFastFlag=True) torque matrix
			
			return Kalt
			

	########### Updates ##################################################################

	def UpdateMidframeDampHeadTorques(self):		
		for name in self.Bones:		#Commment: A bit time inefficient. "for name in self.notFullyConstrainedBoneList:" would have fewer if-case evaluations.
			b=self.Bones[name]
			
			if b.hasFreedom:
				b.midframeDampHeadTorque=b.getMidframeDampHeadTorque()
			else:
				b.midframeDampHeadTorque=Vector(0,0,0)
				continue	
				
			if b.hasConstraint:
				for m in b.constrainedIndexes:
					b.midframeDampHeadTorque-=(b.midframeDampHeadTorque*b.mat[m])*b.mat[m]
			
	
	def UpdateSpringAndDampTorques(self):
		#Updates the net internal joint spring and damping torques for the spring bones.
		#Writes to .midframeDampTorque and .SpringTorque.
		#It is assumed that .midframeDampHeadTorque and .springHeadTorque has just been updated.
				
		#Update spring torques
		
		for name in self.Bones:
			b=self.Bones[name]
			b.SpringTorque=Vector(b.SpringHeadTorque)			#Starting with the head spring torque
			
			for childname in b.springChildren:					#Subtracting spring torques from children. 
				b.SpringTorque-=self.Bones[childname].SpringHeadTorque	
		
		
		#Update damp torques, unless dampMode==2 (then there is no need)
		
		if not self.dampMode==2:
			for name in self.Bones:
				b=self.Bones[name]
				b.midframeDampTorque=Vector(b.midframeDampHeadTorque)		#Starting with the head damp torque
				
				for childname in b.springChildren:					#Subtracting spring torques from children
					b.midframeDampTorque-=self.Bones[childname].midframeDampHeadTorque	
		
								
	def UpdatePrelMidframeDampTorques(self):
		#Updates the net internal damping torques for the spring bones.
		#Writes to .prelMidframeDampTorque
		#Note that .midframeDampHeadTorque should be updated before this function is called
		
		for name in self.Bones:
			b=self.Bones[name]	
			b.prelMidframeDampTorque=Vector(b.midframeDampHeadTorque) 	#Starting with the head damp torque
			
			for childname in b.springChildren:					#Subtracting damp torques from children
				b.prelMidframeDampTorque-=self.Bones[childname].midframeDampHeadTorque
				


#################################################################################################
#################################################################################################
##############################  DynBoneBranches  ################################################						


class DynBoneBranchesClass:
	#A class for handling a set of separate dynamical branches (bone groups).
	#For example, if one approximates that one tree branch is not affecting another tree branch,
	#one can set them up as as two elements in a DynBoneBranch object (list), which then evolves them separately 
	#leading (typically) to a faster evolution.
	
	BoneDictList= None		#A list of (links to) instances of the DynBoneDictClass or its children. 
							#Comment: Poor naming, it is not a list of Dictionaries. It is a list of BoneGroup objects.
							
	############# Methods ########################################
	
	def EvolveDynamics(self):
		
		for branch in self.BoneDictList:
			if branch.hasFreedom or branch.Ob.dynAndObjectFlag:
				branch.EvolveDynamics()
	
	def SetDynamicKeys(self):
		
		for branch in self.BoneDictList:
			if branch.hasFreedom:
				branch.SetDynamicKeys()
	
	
	def SetInterpolatedRot(self):
		
		for branch in self.BoneDictList:
			if branch.hasFreedom:
				branch.SetInterpolatedRot()
			
	
	def UpdateSpringAndDampTorques(self):
		
		for branch in self.BoneDictList:
			if branch.hasFreedom:
				if branch.type=='SpringBoneDict':
					branch.UpdateSpringAndDampTorques()
	
		
	def InitiationBeforeEvolution(self):
		for branch in self.BoneDictList:
			if branch.hasFreedom or branch.Ob.dynAndObjectFlag:
				branch.firstStepFlag=True