############################################################################
#Name: ArmaturePhysicsGraphics
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This module mainly contains functions related to creating graphical mesh objects (such as arrows)
#in the Blender scene in question. It also contains a function for manipulating the scale and rotation of an
#object (like a force arrow), as well as a couple of functions that shuts off and turns on object modifiers
#(which helps speed up the simulation).
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from math import *          
from Blender import *   
from Blender.Mathutils import *   #For vectors and matrices
from ArmaturePhysicsTools import Normalize,SetRotMat,RotateMat

###########################################################################################
#################### Making a rotational surface ##########################################

def flipAxes(invec,string,plusAxis):
	#Makes a permutation of the vector components of a given vector.
	
	if plusAxis:
		vec=Vector(invec)
	else:
		vec=-Vector(invec)
		
	if string=='y':
			return Vector(vec.y,vec.z,vec.x)
	elif string=='x':	
		return Vector(vec.z,vec.x,vec.y)
	else:
		return Vector(vec)


def rotsurf(scn,radiusList,axialList,axialCaps=[None,None],N=20,name='Arrow',col=[1,0,0],symaxis='z',inmesh=None,plusAxis=True):
	#Creates a rotational surface based on a list radiusList that gives the radii, and axialList that gives the 
	#corresponding axial coordinates (distance along the symmetry axis).
	#If axialCaps[0] is set to a numeric value, this interpreted as an axial coordinate where the radius should go to zero (before the first element of the lists of radiusList and axialList).
	#If axialCaps[1] is set to a numeric value, this interpreted as an axial coordinate where the radius should go to zero (after the last element of the lists of radiusList and axialList).
	#The point of the axialCaps argument is that one can make the faces of any caps as triangles, saving a few vertices compared to making quads
	#and squeezing them to make them look like triangles.
	
	if inmesh==None:
		i0=0					#Index of first vertex
	else:
		i0=len(inmesh.verts)	#Index of first vertex
		
	list=range(N)
	philist=[]
	for i in list:
		philist.append(2*pi/N*i)
	
	vertexList=[]
	faceList=[]
	
	if len(radiusList)!=len(axialList):
		print "Radius list and axial list must be the same length"
		10/0
	
	n=len(radiusList)
	
	for i in range(len(philist)):
		
		phi=philist[i]	
		j=i*n
		
		for k in range(n):
			z=axialList[k]
			r=radiusList[k]
			x=r*cos(phi)
			y=r*sin(phi)
			
			vertexList.append(flipAxes(Vector(x,y,z),symaxis,plusAxis))
			
			#Adding faces
			
			if i<(N-1):
				if k<n-1:
					faceList.append([i0+j+k,i0+j+1+k,i0+j+n+1+k,i0+j+n+k])
			else:
				if k<n-1:
					faceList.append([i0+j+k,i0+j+k+1,i0+k+1,i0+k])		
	
	
	#Capping the ends 
	
	if not axialCaps[0]==None:
		vertexList.append(flipAxes(Vector(0,0,axialCaps[0] ),symaxis,plusAxis))
		j1=len(vertexList)-1	
		
		for i in range(len(philist)):
			j=i*n
			
			if i<(N-1):
				faceList.append([i0+j1,i0+j,i0+j+n])				
			else:
				faceList.append([i0+j1,i0+j,0])
	
				
	if not axialCaps[1]==None:
		vertexList.append(flipAxes(Vector(0,0,axialCaps[1] ),symaxis,plusAxis))
		j2=len(vertexList)-1
		
		for i in range(len(philist)):
			j=i*n
			
			if i<(N-1):
				faceList.append([i0+j2,i0+j+n-1+n,i0+j+n-1])				
			else:
				faceList.append([i0+j2,i0+n-1,i0+j+n-1])
							
	#Outputting the surface
	
	if inmesh==None:	
		rjmesh=Mesh.New()
	else:
		rjmesh=inmesh
		
	rjmesh.verts.extend(vertexList)
	rjmesh.faces.extend(faceList)
		
	mat=Material.New('Material')
	mat.rgbCol=col
	
	if inmesh==None:
		rjmesh.materials=[mat]
	else:
		rjmesh.materials.append(mat)
	
	if inmesh==None:
		ob=scn.objects.new(rjmesh,name)
		return ob,rjmesh	
	else:
		return None,rjmesh
	


###################################################################################
######################   Manipulating objects   ###################################

def SetArrow(arrowvec,posvec,ArrowObject):
	#Sets a new location, length and direction for the negative y-axis of a given non-scaled object, based on vectors arrowvec and posvec.
	
	mag=sqrt(arrowvec*arrowvec)
	ArrowObject.setLocation(posvec)		#Location
	mainVec=Vector(0,-1,0)  			#Note that the arguments for the SetRotMat command below must be set by hand still.
	
	#Set the direction of the force arrow
	
	arr_mat=ArrowObject.matrixWorld
	Axis=CrossVecs(mainVec,arrowvec)
	Axis=Normalize(Axis)
	angle=acos(mainVec*Normalize(arrowvec))
	
	#Using acos is very inacurate for some angles - using asin instead	
	
	if 0<=angle<0.5:	
		angle=asin(CrossVecs(mainVec,Normalize(arrowvec)).length)
	
	if (pi-0.5)<angle<=pi:
		angle=pi-asin(CrossVecs(mainVec,Normalize(arrowvec)).length)
	
	SetRotMat(arr_mat,  RotateMat(  Matrix([1,0,0],[0,1*mag,0],[0,0,1]) ,Axis*angle ))
	ArrowObject.setMatrix(arr_mat)


###################################################################################
################   Disabling/Enabling modifiers   #################################

def DisableMod():
	#Disables interactive mode of all the modifiers for all the scene objects.
	#This speeds up frame changing a good deal.
	#Returns a dictionary of object names containing lists of modifier names that were
	#in interactive display mode "On".
	#Note that in the 3D-screen - nothing changes. It is not updated.
	#But if you look in the modifier list, or tab from object mode into edit mode
	#for each object after running this - you should see the effect.
	
	Dict={}		#Initializing dict of modifier values
	
	for ob in Scene.GetCurrent().objects:
		modnamelist=[]
		mods=ob.modifiers            # get the object's modifiers
		
		for mod in mods:
	   		if mod[Modifier.Settings.REALTIME]:
				modnamelist.append(mod.name)
				mod[Modifier.Settings.REALTIME]=0
		
		Dict[ob.name]=modnamelist
		
	print "Disabled modifiers"
	return Dict


def EnableMod(Dict):
	#Given a dictionary 
	
	for name in Dict:
		mods=Object.Get(name).modifiers
		for mod in mods:
			if mod.name in Dict[name]:
				mod[Modifier.Settings.REALTIME]=1
	
	print "Enabled modifiers"


###################################################################################
######################    Creating objects    #####################################

def makearrow(scn,N=20,name='Arrow',col=[1,0,0],symaxis='y',plusAxis=True,shift=False):	
	#Creates an object shaped like an arrow, pointing at the origin in the negative y-axis direction, and named 'Arrow' unless a second argument is sent,
	#which then becomes the name of the arrow object.
		
	r1=0.01			#The shaft radius
	r2=0.02			#The cone base radius
	h=0.75			#The shaft height
	
	if shift:
		offset=-1
	else:
		offset=0
	
	rList=[r2,r1,r1]
	zList=[offset+1-h,offset+1-h,offset+1]
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=[offset+0,offset+1],N=N,symaxis=symaxis,name=name,col=col,plusAxis=plusAxis)
	
	return ob

def makearrowline(scn,N=20,name='Arrow',col=[1,0,0],symaxis='y'):
	#Creates an object shaped like an arrow, pointing at the origin in the negative y-axis direction, and named 'Arrow' unless a second argument is sent,
	#which then becomes the name of the arrow object.
	#This is not maximally efficient - there are typically more vertices than are necessary.
		
	r1=0.01			#The shaft radius
	r2=0.02			#The cone base radius
	h=0.75			#The shaft height
	
	rList=[r1,r1,r2,0]
	zList=[1,1-h, 1-h,0]
	
	r=0.004
	
	#Pure cylinder
	#rList.extend([r,r,0])
	#zList.extend([0,-5,-5])
	
	# Making a dashed line
	
	dl=0.1
	l=-dl
	for i in range(15):
		l+=2*dl
		rList.extend([0,r,r,0])
		zList.extend([-l,-l,-(l+dl),-(l+dl)])
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=[1,None],N=N,symaxis=symaxis,name=name,col=col)
	
	return ob


def pinarrow(scn,N=20,name='PinArrow',col=[1,0,0],symaxis='y'):
	#Creates an object shaped like a pin, pointing at the origin in the negative y-axis direction, and named 'Arrow' unless a second argument is sent,
	#which then becomes the name of the arrow object.
		
	r=0.01	
	b=0.02			#Base disc height
	br=0.02			#Base disc radius
	
	rList=[br,br,r]
	zList=[1,1-b,1-b]	
	axialCaps=[1,0]
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis=symaxis,name=name,col=col)
	
	return ob


def masscenter(scn,N=4,name='MassCenter',col=[1,0,0],symaxis='z'):
	#Creates an object shaped like a stylized sphere.
	
	r=0.02
	n=20
	list=range(1,n)
	
	thetalist=[]
	for i in list:
		thetalist.append(pi/n*i)
		
	rList=[]
	zList=[]
	
	for theta in thetalist:
		rList.append(r*sin(theta))
		zList.append(r*cos(theta))

	axialCaps=[r,-r]
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis=symaxis,name=name,col=col)
	ob.xRay=True
	
	return ob


def grade(scn,N=20,name='MassCenter',col=[1,0,0],symaxis='z'):
	#Makes a cylinder.
	
	r1=0.015
	rList=[r1,r1]
	zList=[0,1]
		
	axialCaps=[0,1]
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis=symaxis,name=name,col=col)
	ob.xRay=True
	return ob


def gradeStick(scn,N=20,name='MassCenter',col=[1,0,0]):
	#Makes a stick with grade markings on it
	
	rList=[]
	zList=[]
	
	r1=0.01
	r2=0.02
	
	n=5
	d1=0.01
	d2=0.2
	
	z=-d2
	for i in range(n):
		z+=d2
		rList.extend([r1,r1,r2,r2])
		zList.extend([z+d1*0.5,z+d2-d1*0.5,z+d2-d1*0.5,z+d2+d1*0.5])
		
	axialCaps=[d1*0.5,d2*n+d1*0.5]
	
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis='z',name=name,col=col)
	rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis='z',name=name,col=col,inmesh=rjmesh,plusAxis=False)
	rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis='x',name=name,col=col,inmesh=rjmesh)
	
	ob.xRay=True
	return ob


def boneforcetarget(scn,N=20,name='BoneForceTarget',col=[1,0,0],symaxis='z'):
	#Makes a geometrical shape corresponding to a target object for a bone fores.
	#Note: One can just use a regular (scaled down) blender cube instead.
	
	r=0.02
	n=20
	list=range(1,n)
	
	thetalist=[]
	for i in list:
		thetalist.append(pi/n*i)
		
	print thetalist
		
	rList=[]
	zList=[]
	
	for theta in thetalist:
		rList.append(r*sin(theta))
		zList.append(r*cos(theta))
	
	axialCaps=[r,-r]
	
	ob,rjmesh=rotsurf(scn,rList,zList,axialCaps=axialCaps,N=N,symaxis=symaxis,name=name,col=col)
	ob.xRay=True
	
	return ob


##########################################################################################
############## Create a set of objects for Armature physics ##############################

def SetupForceGraphics():
	#Creates a set of objects useful for ArmaturePhysics
	
	#Arrow to use as bone shape for force arrows
	
	name='Arrow0'
	try:
		temp=Object.Get(name)
	except:
		temp=makearrow(Scene.GetCurrent(),name=name,col=[1,0,0])
		temp.xRay=True
		temp.loc=Vector(0,0,0)
		Object.Get(name).layers=[17]
	
	
	name='Arrow1'
	try:
		temp=Object.Get(name)
	except:
		temp=makearrowline(Scene.GetCurrent(),name=name,col=[1,0,0])
		temp.xRay=True
		temp.loc=Vector(1,0,0)
		Object.Get(name).layers=[17]
	
	
	name='Arrow2'
	try:
		temp=Object.Get(name)
	except:
		temp=makearrow(Scene.GetCurrent(),name=name,col=[1,0,0],shift=True)
		temp.xRay=True
		temp.loc=Vector(2,0,0)
		Object.Get(name).layers=[17]
	
	
	#The object monitor arrows and pins
	
	name='F_Arrow'
	try:
		temp=Object.Get(name)	#For some reason ForceArrow does not work as name.
	except:
		makearrow(Scene.GetCurrent(),name=name,col=[1,0,1])
		Object.Get(name).layers=[18]	
	
	
	name='T_Arrow'
	try:
		temp=Object.Get(name)
	except:
		makearrow(Scene.GetCurrent(),name=name,col=[0,1,1])
		Object.Get(name).layers=[18]
	
	name='T_Target'
	try:
		temp=Object.Get(name)
	except:
		temp=pinarrow(Scene.GetCurrent(),name=name,col=[0,1,1])
		Object.Get(name).layers=[18]
	
	
	name='F_Target'
	try:
		temp=Object.Get(name)
	except:
		pinarrow(Scene.GetCurrent(),name=name,col=[1,0,1])
		Object.Get(name).layers=[18]
	
	
	#The small center of mass dot 
	
	try:
		temp=Object.Get('CMdot')
	except:
		masscenter(Scene.GetCurrent(),name='CMdot',col=[1,1,0])
		Object.Get('CMdot').layers=[19]
	
	
	#The velocity gauge at the center of mass
	
	try:
		temp=Object.Get('CM')
	except:
		gradeStick(Scene.GetCurrent(),N=20,name='CM',col=[0,0,0])
		Object.Get('CM').layers=[20]
	
	
	name='xyVelocity'
	try:
		temp=Object.Get(name)
	except:
		grade(Scene.GetCurrent(),N=20,name=name,col=[1,1,0],symaxis='x')
		Object.Get(name).layers=[20]
	
	
	name='zVelocity'
	try:
		temp=Object.Get(name)
	except:
		grade(Scene.GetCurrent(),N=20,name=name,col=[1,1,0],symaxis='z')
		Object.Get(name).layers=[20]
	
	
	# The boneforce stretch-to object. Not used at the moment.
	
	try:
		temp=Object.Get('BoneForceTarget')
	except:
		boneforcetarget(Scene.GetCurrent(),name='BoneForceTarget',col=[1,0,0])
		Object.Get('BoneForceTarget').layers=[17]
	


#################################################################################
##################### Testing module ############################################

#scn=Scene.GetCurrent()
#gradeStick(scn,N=20,name='MassCenter',col=[1,0,0])
#grade(scn,N=20,name='MassCenter',col=[1,1,0],symaxis='z')

#Testing
#pinarrow(scn)
#pinarrow(scn,symaxis='y')
#masscenter(scn)
#makearrowline(scn)
#Redraw()