############################################################################
#Name: ArmaturePhysicsObjects
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This module contains class definitions for various levels of physical objects.
#Note that while one connects a group of physical bones to the physical object (or several groups), the physical
#bones still needs to reference some object features (such as the object matrix).
#So there is a kind of cross reference going on. In practise, the bones (via bone groups) will be attributes to the object,
#but the object will also be an attribute of the bones themselves.
#
#If I recall correctly, the idea behind the three levels of classes in this modules is (essentially):
#
#PhysBaseClass: 		The most basic object class. Contains what is needed to be referenced by the individual physical bone methods.
#
#BasicPhysObjectClass: 	Inherites from PhysBaseClass. Contains what is needed to be referenced by the PhysBoneDictClass,
#					   	which in turn has methods for calculating for example angular momentum (can be used for monitoring).
#
#PhysObjectClass: 		Inherites from BasicPhysObjectClass. Contains what is needed for evolving the physical object (translation and rotation) as such.
#						Note that when the dynAndObjectFlag=True, some of the methods of the PhysObjectClass (like iteratLocRot1 and iterateLocRot2),
#						will not be used.
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from Blender import Object			#Used to reference external forcearmatures. Comment: One could optionally attach the armature objects and not just the names to the PhysOb.
from Blender import Set,Get			#For object (not bodybone) evolution
from Blender.Mathutils import Vector, Matrix, CrossVecs		#For Vectors and matrices

import ArmaturePhysicsTools
reload(ArmaturePhysicsTools)		#For the development phase.
from ArmaturePhysicsTools import GetRotVec, RotateMat, RotateVec, SetRotMat

from ArmaturePhysicsSolve import rjsolve	

#Global variables

veclist=[Vector(1,0,0),Vector(0,1,0),Vector(0,0,1)]
deltashift=0.001		#Base value: 0.001. For the derivative with respect to shifts. Comment 0.0001 is to small - gives visible numerical errors. And 0.01 can yield errors because the disp is too large. But 0.001 can yield errors (matrix Kmat in IterateLocRot1 gets error of the order of 1/1000) also.
deltaangleOb=0.001		#Base value: 0.001. For the derivative with respect to rotations of object.


#########################################################################################################
#################### The most basic object class ########################################################

class PhysBaseClass:
	#The most basic physical object class.
	#Contains what is needed to be referenced by the individual phuysical bones classes (found in ArmaturePhysicsBoneClasses.py)
		
	# Necessary attributes for PhysBoneClass instances
	
	arm_mat= 		None		#A link to the Blender armature (object) matrix of the armature		e.g: BlenderOb.matrixWorld
	pose=			None		#A link to the Blender armature pose 								e.g: BlenderOb.getPose().bones
	dt=				None		#The time step for the evolution. Can change during evolution (in particular if self.stepsPerFrame is greater than 1, dt will change after the initial updates, before the evolution.)
	bodyBoneName=	None		#The name of the bodybone, used as bone motion reference for angular momentum calculations of physbone collections when self.objectBody=False 
	objectBody=		None		#True: 	Object  		  is used as reference of relative motion for calculating angular momentum (L) of physbone collections
								#False: Armature bodyBone is used as reference of relative motion for calculating angular momentum (L) of physbone collections
	
	# Necessary attributes for DynBoneClass instances
	
	dt0=			None		#The time step between Blender frames
	stepsPerFrame=	None		#Number of steps per frame
	startFrame=		None		#The Blender frame number of the first frame of evolution (which is not affected by evolution if keyed in advance)
	
	# Necessary attributes for DynBonesClass instances
	
	arm_ob=			None		#A link to the Blender armature object, e.g: Blender.Object.Get('ArmatureName') 
								#Needed for evolving dynamical bones (via arm_ob.getPose().update()), and also for keying, where the object is sent as an argument.
	gravity=		None		#Flag for indicating if gravity affects the object or not.  Comment: Possibly this is necessary also for PhysBoneClass instances
	g=				None		#The gravitional constant (typically 9.81 m/s/s). 			Comment: Possibly this is necessary also for PhysBoneClass instances
	
	iframe= 		None		#Used to set bone keys in DynBonesClass.
	jloop=			None		#Just for diagnosis
	
	# General
	
	forbidOject=	None		#A flag to indicate that, if the user has set the parameter objectBody=False but has given no valid bodyBone, object evolution will not be allowed.
	existBodyBone=	None		#A flag to indicate if a valid bodyBone exists or not. 
	
	AllPhysBones=	None		#A dictionary containing all physical bones related to the object
	UpdateBoneList=	None		#A list with all the physical bone names that need updating before a new time step  
	
	dynamicFlag= 	None		#Comment: Unsure of if this belongs here.
	boneforce=		None		#Comment: Change name to boneForceFlag
	
	storedArmMat=	None		#The arm_mat rotational part. Used for hold bones.
	
	# For springBones
	
	arm_data= 		None 
	
	InternalPhysBones=None		#An instance of the InternalPhysBoneDictClass.
								#For all bones that require SetExternalDynForceTorque().
	
	##################### Bone related object methods #########################
	
	def UpdateAppropriateBones(self): 	
		#Updates all the appropriate bones.
		
		if self.dynamicFlag:		#This if case may be removed if one wants to do monitoring of torques for non-dynamical bones in future versions.
			if self.boneforce:
				self.InternalPhysBones.SetExternalDynForceTorque()
				
				#Comment: One could instead loop through the branches in DynBoneDictList
				#But actually, that list may not be defined at this level,
				#where we may want to monitor internal torques in user keyed bones.
			
			
		for name in self.UpdateBoneList:			
			self.AllPhysBones[name].update()		#So this updates also the dynamic (including spring) bones.
		
		#Comment: Line below does not really belong here, since the line is an object update, in a bone update method.
		self.storedArmMat=self.arm_mat.rotationPart()	#Used for hold bones.


#############################################################################################################
#############################################################################################################
########################## Definitions related to the object as a whole #####################################

class BasicPhysObjectClass(PhysBaseClass):
	#The basic class for dealing with objects that may move and rotate,
	#on which we may have bone chains that we want to determine the angular momentum for.
	#Note that in calculating the angular momentum of the bone chain (typically the whole object)
	#one evaluates the individual bone motion relative to the reference frame 
	#(object or bodybone, translated to have the origin coincide with the center of mass) as an intermediate step.
	#This improves accuracy by a lot if the reference frame is rotating rapidly and the internal bones are not 
	#moving so rapidly relative to the reference frame.
	#
	#Note also that there are no attributes or methods related to evolving the object as such.
	#It contains the attributes needed for a PhysBoneDictClass instance.
	#
	#Comment: Due to the limited usefulness of this class as such, it might be better to incorporate it
	#into the PhysObClass (as it originally was). 
	
	loc=			Vector(0,0,0)						#Object/bodybone location in worldspace Comment: One might argue that this should be an ObjectBones attribute rather than an Object attribute.
	mat=			Matrix([1,0,0],[0,1,0],[0,0,1])		#Object/bodybone 3x3 matrix in worldspace.
	C=				Vector(0,0,0)						#Object center of mass			
	bodyBoneRestMat=None								#The rest matrix of the bodybone.
	
	########### Methods ##################################################
	
	def GetLoc(self):
		#Returns the object/bodybone location in world space.
	
		if self.objectBody:
			return self.arm_mat.translationPart()
		else:
			return (self.pose[self.bodyBoneName].poseMatrix*self.arm_mat).translationPart()	#The poseMatrix is already in armature space, so there is no need to use the posebone_restmat.


	def GetMat(self):
		#Returns the object/bodybone matrix in world space.
		
		if self.objectBody:
			return self.arm_mat.rotationPart()
		else:
			return (self.pose[self.bodyBoneName].poseMatrix*self.arm_mat).rotationPart()		#The poseMatrix is already in armature space, so there is no need to use the posebone_restmat.
	
	
	#Comment: Currently this is just overwritten by the child class update method.
	def update(self,C):	 			#Comment: Should be capital U for Update
		self.C=C
		self.mat=self.GetMat()
		self.loc=self.GetLoc()
		
		#For hold bones
		
		self.storedArmMat=self.arm_mat.rotationPart()
		
		
#############################################################################################################
#############################################################################################################
#################################### PhysObjectClass	 ####################################################

class PhysObjectClass(BasicPhysObjectClass):
	#Contains what is needed for evolving the physical object (translation and rotation) as such.
	#Note that when the dynAndObjectFlag=True, some of the methods of the PhysObjectClass (like iteratLocRot1 and iterateLocRot2),
	#will not be used.
		
	v=Vector(0,0,0)							#Object location velocity in worldspace
	omega=Vector(0,0,0)						#Object angular velocity about location in worldspace
	a=Vector(0,0,0)							#Object acceleration of location in worldspace
	omegadot=Vector(0,0,0)					#Object time derivative of angular velocity about location
		
	relVec=Vector(0,0,0)					#Vector from target object center of mass to object location in object space
	relVecV=Vector(0,0,0)					#First time derivative of relVec
	relVecA=Vector(0,0,0)					#Second time derivative of relVec
	
	ForceBones=None					#A dictionary with forcebone names, and rest lengths?
	TorqueBones=None				#A dictionary with torquebone names.
	ForceArmatureDict=None			#A dictionary of forcebone dictionaries 
	M=None							#The total mass
	forceFactor=None				#The scale factor of the forces. forceFactor = M*g
	
	ObjectBones=None				#An instance of the PhysBonesClass
	dynamicDerivative= None			#Comment: Could change name to dynamicDerivativeFlag, but with the dynAndObjectFlag option - this is obsolete anyway.
	
	DynBoneDictList=None			#Comment: Not sure if it belongs at this level. But I think it does.
	
	storedForce=None				#Force  stored for in between frames linearization when stepsPerFrame>1
	storedTorque=None				#Torque stored for in between frames linearization when stepsPerFrame>1
	
	holdWorldForce=None				#Net world force from hold bones.
	holdWorldTorque=None			#Net world force from hold bones.
	
	deltashift=None					#Base value: 0.001. For the derivative with respect to shifts. Comment 0.0001 is to small - gives visible numerical errors. And 0.01 can yield errors because the disp is too large. But 0.001 can yield errors (matrix Kmat in IterateLocRot1 gets error of the order of 1/1000) also.
	deltaangleOb=None				#Base value: 0.001. For the derivative with respect to rotations of object.

	################# Methods #############################################
				
	def RelInterpolateLoc(self, newMat, newC):
		#newMat is the guesstimated object/bodybone world 3-matrix for the current step.
		#newC is the guesstimated world position of the entire armature center of mass.
		#
		#Returns the new world object/bodybone location we would get if the discrete acceleration of the vector 
		#from the target center of mass to the object location (relative to the object space) were constant.
		#(How the center of mass moves is known since the onframe force is known).
		# 
		#If the object center of mass is far separated from the object position and the object is rotating fast,
		#then the object location acceleration will be far from constant (even for simple parabolic motion),
		#an this guesstimation may be better than a regular guesstimation based only on previous object locations.
		#The argument newMat is the guesstimation of the new object 3-matrix, and newC is the new target center of mass.
		#For rigid objects moving under the influence of a constant force - the guesstimation of this
		#method should be exact modulo numerical errors.
		
		dt=self.dt
		newRelVec=self.relVec+(self.relVecV+self.relVecA*dt)*dt		#Note that intentionally there is no division by 2 on the acceleration term: Velocity is considered to lie between frames
		return newC+newRelVec*newMat								#Returning the guesstimated world object location
	
		
	def InterpolateLoc(self):
		#Currently this is not in use. See RelInterpolateLoc instead.
		#Returns the new object location we would get if the discrete acceleration were constant 
		
		dt=self.dt
		return self.loc+(self.v+self.a*dt)*dt	#Note that intentionally there is no division by 2 on the acceleration term: Velocity is considered to lie between frames

	
	def InterpolateMat(self):
		#Returns the new object matrix we would get if the discrete angular velocity derivative were constant  
		
		dt=self.dt
		return RotateMat(self.mat,(self.omega+self.omegadot*dt)*dt)


	def update(self,C):	 	#Comment: Should be capital U for Update really.
		#Updates .loc .mat .v .omega .a and .omegadot according to current self.arm_mat,
		#which may or may not correspond to the actual current object matrix
		
		dt=self.dt
		mat=self.GetMat()
		loc=self.GetLoc()
		
		omega=GetRotVec(self.mat,mat,'Whole Object')/dt
		self.omegadot=(omega-self.omega)/dt
		self.omega=omega
		self.mat=mat
		self.loc=loc 	#Used in GetPrelL() and InterpolateLoc
		self.C=C*1.0
		
		#For hold bones
		
		self.storedArmMat=self.arm_mat.rotationPart()
		
		#Updates for InterpolateLoc - Not used currently.
		
		#v=(self.arm_mat.translationPart()-self.loc)/dt
		#self.a=(v-self.v)/dt
		#self.v=v
		
		#Updates for RelInterpolateLoc 
		
		deltaVec=(loc-C)					#This uses the target C which is faster than GetC().
		relVec=self.mat*deltaVec			#Projecting onto the local axes of the object
		relVecV=(relVec-self.relVec)/dt
		self.relVecA=(relVecV-self.relVecV)/dt
		self.relVecV=Vector(relVecV)     	#Copying just to be safe. Vector() can probably be omitted.
		self.relVec=Vector(relVec)			#Copying just to be safe. Vector() can probably be omitted.
		
		#Comment: This is not really tested - but of marginal importance anyway
		if self.iframe==self.startFrame:
			if self.stepsPerFrame>1:	
				self.relVecV+=self.relVecA*(0.5-0.5/(self.stepsPerFrame))*self.dt0 
				self.omega+=self.omegadot*(0.5-0.5/(self.stepsPerFrame))*self.dt0
	
	
	#################### Getting the object forces and torques from force bones and force armatures ###################### 
	
	def GetForceAndTorque(self,C):
		#Given the object center of mass location C, returns the net force and torque about C
		#coming from ForceBones, ForceArmatures and gravity
		  
		f=Vector(0,0,0)		#Force initiation
		tau=Vector(0,0,0)	#Torque about the center of mass, initiation
		
		#Getting the forces and corresponding torques from force bones in the object armature
		
		if self.boneforce:
			for name in self.ForceBones:
				fbone=self.ForceBones[name]
				
				length=fbone['length']
			
				try:
					bone_mat=self.pose[name].poseMatrix
					
				except AttributeError:
					print 'Listed force bone is not in armature. Omitting'
					continue
			
				force_mat_world=bone_mat*self.arm_mat
				multfactor=Vector(force_mat_world[0][0:3]).length * Vector(force_mat_world[2][0:3]).length	#Comment: e.g. self.Ob.pose[name].size[0] gives only scale prior to scale constraints. Therefore not used.
				
				fpos=Vector(force_mat_world[3][0:3])				#Head (thick end) of force bone position
				
				ff=-self.forceFactor*Vector(force_mat_world[1][0:3])*multfactor*length	 	#The y-axis is the force indicator
				tau+=CrossVecs(fpos-C,ff)							#Note that C is the Target center of mass.
				f+=ff
		
		#Getting the forces and corresponding torques from external force armatures 
		
		if self.boneforce:
			for armname in self.ForceArmatureDict:
				
				force_arm_ob=Object.Get(armname)
				force_arm_mat=force_arm_ob.matrixWorld
				force_pose=force_arm_ob.getPose().bones
				
				ExternalForceBoneDict=self.ForceArmatureDict[armname]	#Dict of the bones with names starting with "Force" in the armature in question
			
				for name in ExternalForceBoneDict:
					force_bone_mat=force_pose[name].poseMatrix
					force_mat_world=force_bone_mat*force_arm_mat
					multfactor=Vector(force_mat_world[0][0:3]).length * Vector(force_mat_world[2][0:3]).length	#Comment: e.g. self.Ob.pose[name].size[0] gives only scale prior to scale constraints. Therefore not used.	
					
					length=ExternalForceBoneDict[name]['length']
					fpos=Vector(force_mat_world[3][0:3])
					force=-self.forceFactor*Vector(force_mat_world[1][0:3])*multfactor*length
					
					f+=force
					tau+=CrossVecs(fpos-C,force)	#Note that C is the target center of mass.			
		
		if self.gravity:			#Best to add the effect of gravity here, afterwards - gravity should give no net torque - but might in the algorithm due to C not being the exact object center of mass.
			f+=-self.M*self.g*Vector(0,0,1)
		
		#  Interpolating force and torque between frames.

		if self.stepsPerFrame>1: 			
			if self.jloop==0:
				self.storedForce=Vector(f)
				self.storedTorque=Vector(tau)
			else:
				fraction=self.jloop*1.0/self.stepsPerFrame
				f=  self.storedForce* (1-fraction) + f*fraction
				tau=self.storedTorque*(1-fraction) + tau*fraction
				
		# Returning the result	
				
		return f,tau
		
		
	################ Blender specific methods ##############################
	
	def UpdateLocRot(self):
		#Makes the location and rotation of the object/bodybone real (constraints etc are updated).
		#Note that for self.objectBody=True, any individual non-keyed bone states will be lost.
		
		if self.objectBody:
			self.arm_ob.setMatrix(self.arm_mat)
			Set('curframe',Get('curframe'))
		else:		
			self.arm_ob.getPose().update()		#Here non-keyed states are preserved.
	

	def SetLoc(self,vec):
		#Sets the object/bodybone location in world space as vec.
		
		if self.objectBody:
			self.arm_mat[3][0:3]=Vector(vec)
		else:
			#Note that the bone .poseMatrix is not updated until you run self.arm_ob.getPose().update() however!
			
			try:
				netmat=self.pose[self.bodyBoneName].parent.poseMatrix*self.arm_mat	#The world reference matrix relative to which the bodyBone local matrix lives.
			except:
				netmat=self.arm_mat		#If there is no parent - we use this.
			
			refmat=self.bodyBoneRestMat*netmat		#The world space matrix of the rest (editmode) body bone.
			self.pose[self.bodyBoneName].loc=(vec.resize4D()*refmat.invert()).resize3D()
		
	
	def SetMat(self,mat):
		#Sets the object/bodybone 3-matrix in world space as mat.
		
		if self.objectBody:
			SetRotMat(self.arm_mat,mat)
		else:		
			#Note that the bone .poseMatrix is not updated until you run self.arm_ob.getPose().update() however!
			try:
				netmat=self.pose[self.bodyBoneName].parent.poseMatrix*self.arm_mat
			except:
				netmat=self.arm_mat	
			refmat=self.bodyBoneRestMat.rotationPart()*netmat.rotationPart()		
			self.pose[self.bodyBoneName].quat=( mat * (refmat.invert()) ).toQuat()		
			

	def SetKey(self):
		if self.objectBody:
			self.arm_ob.insertIpoKey(3)		#Integer 3 means LOCROT
		else:
			self.pose[self.bodyBoneName].insertKey(self.arm_ob,self.iframe,[Object.Pose.ROT,Object.Pose.LOC],True)
			#Comment: The True argument, at the end, is a fast flag that insures that IPO's are not recalculated.
			#I think this means that bezier curves are not calculated. When StepsPerFrame=1, this should make
			#no difference for the operation of the script, because there is a key added for every frame, and
			#Bezier curves are esentilly not used. Using the fast option can speed up the entire script time by 30%,
			#which is kind of an astonishing improvement. As the script is working at the 
			#writing of this comment (100317),setting the True option seems to have no impact on 
			#the operation at all - it just makes everything faster.
			#For future versions where one may do more refined in between frames calculations,
			#it might have an actual effect on the evolution if the fast flag is True or not however.
			

	########### The main iteration functions for the whole object ###############################
	
	def IterateLocRot1(self,TargetC,TargetL):
		#Given a target center of mass position TargetC at the current frame, and a target angular
		#momentum TargetL, with respect to the world origin, midframe before the current frame, this function shifts 
		#and rotates the object about the current center of mass of the object (as the algorithm is called),
		#to try and achieve the targets. 
		#
		#The idea is to first make small rotations and shifts of the object to numerically determine the first order  
		#derivatives of C and L, with respect to rotations and shifts. From these derivatives 
		#the net shift and rotation is then determined, and set. First the object position is shifted, to try and achieve the 
		#target center, without taking into account the effect of the shift on the object angular momentum.
		#Then, starting from the new position, the object is rotated to try and achieve the angular momentum target.
		#Now assuming that the pose of the object is independent of how the object is positioned or rotated
		#this should work well.
		#
		#Comment: Could change name to BodyIterate1, so the name explains that it is not primarilly a dynamic iteration.
		
		
		self.UpdateLocRot()
		InitC=self.ObjectBones.GetC()
		DeltaC=TargetC-InitC
		InitLoc=self.GetLoc()
		
		K_C=[]					#Initiating the vector derivatives for the center of mass
			
		# Getting the derivatives of how small shifts of the object location affects C.
		
		for i in [0,1,2]:
			shiftvec=veclist[i]*deltashift
			vec=InitLoc+shiftvec
			self.SetLoc(vec)
			self.UpdateLocRot()
			K_C.append( (self.ObjectBones.GetC()-InitC)/deltashift )			#Derivative, how much does the center of mass move as the object is moved
			
			
		#Note that for small deltashift values (like 0.001) - one can get errors (due to Blender numerics) of 1 part per thousand in the matrix components.
		
		Kmat=Matrix(K_C[0],K_C[1],K_C[2])
		Kmat.transpose()	
		deltashiftvec=Kmat.invert()*DeltaC
			
		self.SetLoc(InitLoc+deltashiftvec)
		
		#Setting the new object location as the basis for the coming rotation part
		
		InitLoc+=deltashiftvec
		
		self.UpdateLocRot()
		self.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
		InitC=self.ObjectBones.GetPrelC()
		InitL=self.ObjectBones.GetPrelL(InitC)		
			
		DeltaL=TargetL-InitL  	#The necessary shift from InitL to TargetL in the midstep before current frame
			
		# Getting the derivatives of how small object rotations about InitC affects L.
		
		InitMat=self.GetMat()
		K_L=[]					#Initiating the vector derivatives for the angular momentum
	
		for i in [0,1,2]:
			rotvec=veclist[i]*deltaangleOb
			newmat=RotateMat(InitMat,rotvec)
			self.SetMat(newmat)
			
			#Shifting object position to yield a net rotation about InitC
			
			vecA=InitC-InitLoc				
			vecB=RotateVec(vecA,rotvec)
			self.SetLoc(InitLoc+vecA-vecB)
			
			# Appending derivative	
			
			self.UpdateLocRot()			
			self.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
			PrelC=self.ObjectBones.GetPrelC()
			PrelL=self.ObjectBones.GetPrelL(PrelC)		
		
			K_L.append(  (PrelL-InitL)/deltaangleOb )
				
		#Solving the remaining equations
		
		Kmat=Matrix(K_L[0],K_L[1],K_L[2])
		Kmat.transpose()
		deltaanglevec=Kmat.invert()*DeltaL
			
		#Making the net rotation in self.arm_mat or in the bodybone
		
		rotvec=deltaanglevec
		newmat=RotateMat(InitMat,rotvec)
		self.SetMat(newmat)
		
		#Shifting object position to keep rotation about InitC
		
		vecA=InitC-InitLoc				
		vecB=RotateVec(vecA,rotvec)				 
		self.SetLoc(InitLoc+vecA-vecB)
	
	
	def IterateLocRot2(self,TargetC,TargetL):
		#Given a target center of mass position TargetC at the current frame, and a target angular
		#momentum TargetL, with respect to the center of mass, midframe before the current frame, this function shifts 
		#and rotates the object about the current center of mass of the object (as the algorithm is called),
		#to try and achieve the targets.
		#The derivatives of L and C with respect to shifts and rotations are determined, and then
		#by solving a 6x6 equation system the net shift and rotation are determined in a single step. 
		#
		#This method is more time consuming than IteratLocRot1, because more derivatives are calculated, but it may be more 
		#efficient when the pose of the object is affected by the rotation and the position of the object,
		#like when a leg in an IK chain is touching the ground. Because then a shift of the object position 
		#will affect the object intrinsic angular momentum.
		
		self.UpdateLocRot()
		if self.dynamicFlag and self.dynamicDerivative:	
			self.DynBoneBranches.EvolveDynamics()	#Evolving around interpolated state 
			if self.objectBody:
				self.DynBoneBranches.SetDynamicKeys()
				#Comment: when objectBody is False, UpdateLocRot() will not destroy non-keyed bone rotations. Then there is no need to key them here.
			
		self.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.
		InitC=self.ObjectBones.GetPrelC()
		InitL=self.ObjectBones.GetPrelL(InitC)
				
		DeltaL=TargetL-InitL
		DeltaC=TargetC-InitC
		
		InitLoc=self.GetLoc()
		InitMat=self.GetMat()
		
		K_C=[]					#Initiating the vector derivatives for the center of mass
		K_L=[]					#Initiating the vector derivatives for the angular momentum	
		
		# Getting the derivatives of how small shifts of the object location affects C and L.
		
		for i in [0,1,2]:
			shiftvec=veclist[i]*deltashift
			self.SetLoc(InitLoc+shiftvec)
			
			self.UpdateLocRot()
			if self.dynamicFlag and self.dynamicDerivative:	
				self.DynBoneBranches.EvolveDynamics()	#Evolving around interpolated state 	
				
			self.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
			PrelC=self.ObjectBones.GetPrelC()
			PrelL=self.ObjectBones.GetPrelL(PrelC)
			
			K_L.append( (PrelL-InitL)/deltashift )
			K_C.append( (PrelC-InitC)/deltashift )	
				
		# Getting the derivatives of how small object rotations about InitC affects C and L.
			
		for i in [0,1,2]:	
			rotvec=veclist[i]*deltaangleOb
			newmat=RotateMat(InitMat,rotvec)
			self.SetMat(newmat)
			
			#Shifting object position to yield a net rotation about InitC
			
			vecA=InitC-InitLoc				
			vecB=RotateVec(vecA,rotvec)
			self.SetLoc(InitLoc+vecA-vecB)
			
			#Evolving dynamical bones
			
			self.UpdateLocRot()
			if self.dynamicFlag and self.dynamicDerivative:	
				self.DynBoneBranches.EvolveDynamics()	#Evolving around interpolated state 
				
			#Adding derivatives
			
			self.ObjectBones.UpdatePrelMatAndC()		#Updates preliminary .prel_mat and .prel_c according to current bone states relative to the world space.		
			PrelC=self.ObjectBones.GetPrelC()
			PrelL=self.ObjectBones.GetPrelL(PrelC)
			
			K_L.append( (PrelL-InitL)/deltaangleOb )
			K_C.append(	(PrelC-InitC)/deltaangleOb )	
			
		#Solving the 6x6 equation system 
		
		Kmat=[[0]*6 for i in range(6)]		#Initializing 6x6 list-matrix of zeroes
		for j in range(6):
			for i in range(3):
				Kmat[i][j]=K_C[j][i]
				Kmat[i+3][j]=K_L[j][i]
		
		b=DeltaC[:]
		b.extend(DeltaL)
		
		deltas=rjsolve(Kmat,b)
		deltashiftvec=Vector(deltas[0:3])
		deltaanglevec=Vector(deltas[3:])	
	
		#Making the net rotation about InitC, and the net shift, in self.arm_mat
		
		rotvec=deltaanglevec
		newmat=RotateMat(InitMat,rotvec)
		self.SetMat(newmat)
		
		#Shifting object position to keep rotation about InitC, and adding the desired shift
		
		vecA=InitC-InitLoc				
		vecB=RotateVec(vecA,rotvec)				 
		self.SetLoc(InitLoc+deltashiftvec+vecA-vecB)