############################################################################
#Name: ArmaturePhysicsSolve
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This module contains a couple of functions related to solving linear equation systems.
#
#Note: The functions of this module should be part of the numPy package.
#So if one can acces numPy, it should be considerably faster to use these functions.
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


import copy

def rjsolve(mat_in,b,EPSILON=0.000001):
	#Given row koefficents listed in mat, and the right hand side terms listed in b,
	#returns the solution to the equation system.
	#EPSILON is currently not in use.
	
	mat=copy.deepcopy(mat_in) 
	b=copy.deepcopy(b)	
	
	list=range(len(mat))
	
	#Putting the equation system on triangular form (effectively see note about '+1' below)
	
	for i in list:
		
		#Finding the index of the major element in the remaining rows at column i
		
		col=[]
		for q in list[i:]:
			col.append(abs(mat[q][i]))
		m=col.index(max(col))+i
		
		#Shifting the rows
		
		row=mat[m]
		bval=b[m]
		if not i==m:
			mat.pop(m)
			mat.insert(i,row)
			b.pop(m)
			b.insert(i,bval)

		#Making the gaussian elimination
		
		for k in list[i+1:]:			#Looping through the rows
			fac=mat[k][i]*1.0/row[i]
			for j in list[i+1:]:		#Note: The '+1' skips setting zeroes straight down the column. Will not be used anyway.
				mat[k][j]-=fac*row[j]
			b[k]-=bval*fac
	
	#Substituting from the bottom and up to get the solution
	
	revlist=list[:]
	revlist.reverse()
	x=list[:]				#A dummy initialization
	
	for i in revlist:
		sum=0
		for j in list[i+1:]:
			sum+=x[j]*mat[i][j]
	
		x[i]=(b[i]-sum)*1.0/mat[i][i]
	
	return x


def invertMat(mat_in,EPSILON=0.000001):
	#Given row koefficents listed in mat, returns the inverted matrix as a list of row lists.
	#EPSILON is currently not in use.
	
	mat=copy.deepcopy(mat_in) 		
	list=range(len(mat))
	
	#Initializing a unit matrix Imat, of the same size as mat.
	
	Imat=[]
	for i in list:
		row=[]
		for j in list:
			if i==j:
				row.append(1)
			else:
				row.append(0)
		Imat.append(row)
	
	UnitMat=copy.deepcopy(Imat)	#Only used to return if there is an error
	
	#Putting the equation system on triangular form
	
	for i in list:
		
		#Finding the index of the major element in the remaining rows at column i
		
		col=[]
		for q in list[i:]:
			col.append(abs(mat[q][i]))
		m=col.index(max(col))+i
		
		#Shifting the rows
		
		row=mat[m]
		Irow=Imat[m]
		if not i==m:
			mat.pop(m)
			mat.insert(i,row)
			
			Imat.pop(m)
			Imat.insert(i,Irow)	

		#Making the gaussian elimination
		
		for k in list[i+1:]:			#Looping through the rows
			fac=mat[k][i]*1.0/row[i]
			
			for j in list[i+1:]:			#Looping through the elements of the row
				mat[k][j]-=fac*row[j]
			
			for j in list:	
				Imat[k][j]-=fac*Irow[j]
							
	#Substituting from the bottom and up to get the solution
	
	revlist=list[:]
	revlist.reverse()

	for i in revlist:
		
		#Subracting from the upward rows
		
		if not i==0:
			for k in list[0:i]:	#Note that i-1 is tha largest element in that sublist
				fac=mat[k][i]/mat[i][i]
				for j in list:
					Imat[k][j]-=fac*Imat[i][j]
		
		#Multiplying the row by a factor to get unit diagonal element
		
		fac=1.0/mat[i][i]
		for j in list:
			Imat[i][j]*=fac
		
	return Imat


def multMatVec(mat,vec):
	#Multiplies matrix (list of lists) with vector (list) to get new list.
	
	n=len(mat[0])
	list=range(n)
	out=[]
	
	for i in list:
		sum=0
		for j in list:
			sum+=mat[i][j]*vec[j]

		out.append(sum)
			
	return out
