############################################################################
#Name: ArmaturePhysicsTools
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This module contains some functions related to vector and matrix operations.
#It also contains some other (diverse) functions that are used by the armature physics package.
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


from Blender import Draw
from math import *          
from Blender.Mathutils import *   #For vectors and matrices

EPSILON=1e-10			#A limit below wich numbers may be considered zero

#Comment: Note that the AngleBetweenVecs function defined below is almost identical to a function within Mathutils,
#which returns degrees instead of radians.

#################################################################################
###############    Regular math functions     ###################################

def sign(x):
	#Returns the sign of x.
	
	if x>0:
		return 1
	if x<0:
		return -1
	else:
		return 0


def is_nan(num):
	#Returns True if num is a nan, else returns False.
	
	return str(num)=='nan'


#################################################################################
###############    Print math functions     #####################################

def roundString(number):
    #Given a decimal number, with a comma separator somewhere, returns the most significant number
    #and the corresponding exponential. For example 0.027 yields 2e-2.
    
    string=str(number)
    
    counter=-1
    foundPoint=False
    foundNumber=False
    foundExp=False
    eCounter=0
    
    
    for i in string:
        counter+=1

        if foundNumber==False and i!="0" and i!="-" and i!='.':
            firstNumber=i
            foundNumber=True
            numberCounter=counter
            
        
        if i=='.':
            foundPoint=True
            pointCounter=counter

        if i=='e':
            eCounter=counter
    

    if foundPoint:
        distNumberToPoint=pointCounter-numberCounter
        if distNumberToPoint>0:
            distNumberToPoint-=1
    else:
        return "NOPO"
    
    if foundNumber:
        returnString=firstNumber
    else:
        returnString="0"

    if string[0]=='-':
        returnString='-'+returnString

    if not 'e' in string:
        returnString+='e'+str(distNumberToPoint)
    else:
        eString=string[eCounter+1:]
        eStringNum=int(eString)
        returnString+='e'+str(distNumberToPoint+eStringNum)

    return returnString


def printMatrix(mat,EPSILON=1e-3):
    #Prints the first significant number, with the proper exponential, of all the
	#float values of a matrix (list of lists) sent as argument 'mat'.
	#Note that for the method to work a comma separator in each matrix element (value) is necessary.
	 
    for i in mat:
        for j in i:
            if abs(j)<EPSILON:
                print "0    ",
            else:
                nstr=roundString(j)
                while len(nstr)<5:
                    nstr+=" "
                print nstr,
                
        print ""

#################################################################################
###############     Vector functions    #########################################

def Normalize(vec):
	#Returns a normalized vector.
	
	mag=sqrt(vec*vec)
	if mag>EPSILON:
		return vec/mag		#Makes a copy and returns it
	else:
		#print 'Can not normalize zero vector - returns vector'
		return Vector(vec)	#Makes a copy and returns it


def MagnitudeVec(vec):
	#Returns the magnitude of the vector.
		
	return sqrt(vec*vec)


def AngleBetweenVecs(vecA,vecB):
	#Given two vectors, returns the angle between the vectors in radians
	#
	#Note that there is an almost identical function built into the Mathutils, but
	#that returns the angle in degrees.
	
	try:
		vec1=Vector(vecA)	#Making a copy
		vec2=Vector(vecB)	#Making a copy
	except:
		Draw.PupMenu("AngleBetweenVecs: Needs two vectors. Quitting.")
		10/0	
	
	angle=asin(CrossVecs(vec1.normalize(),vec2.normalize()).length) #One could do this with dot product and acos - but for small angles that is not a numerically good option
	if is_nan(angle):	
		if is_nan(vec1[0]) or is_nan(vec2[0]):
			Draw.PupMenu("Something went wrong in AngleBetweenVecs(). This happens. Just try again, it usually works then. Quitting.")
			print "Something went wrong in AngleBetweenVecs(). Quitting."
			print "invec1",vecA
			print "invec2",vecB
			print "normalized vec1", vec1
			print "normalized vec2", vec2
			10/0
		else:
			angle=pi/2	#Thing is that if vec1 and vec2 are near ortogonal, due to numerical error
						#the length of the cross product may exceed 1, which causes asin()
						#to return 'nan' - this takes care of that.
		
	if vec1*vec2<0:
		angle=pi-angle
	
	return angle


def RotVec4Vecs(vec1,vec2):
	#Given two vectors, returns the rotation vector that brings the first one to be aligned to the second one
	#(by an as small angle as possible).
	
	Axis=CrossVecs(vec1,vec2)
	if not Axis.length < EPSILON*EPSILON:
		Axis.normalize()
		angle=AngleBetweenVecs(vec1,vec2)
		return angle*Axis
	else:
		return Vector(0,0,0)


def RotateVec(vec,rotvec):
	#Rotates vector vec about the rotation vector rotvec.
	#The direction of rotvec is the rotation axis, the magnitude of rotvec is the angle.
	
	angle=rotvec.length
	if not angle<EPSILON:
		Axis=Normalize(rotvec)
		vecA=(vec*Axis)*Axis
		vecB=vec-vecA
		vecC=CrossVecs(Axis,vecB)
		return vecA+cos(angle)*vecB+sin(angle)*vecC
	else:
		return Vector(vec) 	#Returns copy


#################################################################################
###############     Matrix functions    #########################################

def TransposeList(list):
	#Transposes a list of equally long lists. Maybe there is a built in feature to do this.
	
	newlist=[]
	for j in range(len(list[0])):
		row=[]
		for i in range(len(list)):
			row.append(list[i][j])
		newlist.append(row)
	return newlist


def GetRotVec(Mat0,Mat1,*args):
	#Given two matrices containing orthonormal base-vectors in the rows
	#this function returns a vector whose direction corresponds to the axis of rotation
	#that brings the first matrix into the second. The magnitude of the vector corresponds to the
	#rotation angle in radians.
	
	D=Mat1-Mat0		#The delta-vectors
	
	#Find the two largest deltas
	
	list=[D[0].length, D[1].length, D[2].length]
	mindex=list.index(min(list))
	ilist=[0,1,2]
	ilist.pop(mindex)
	m=ilist[0]		#Index to first large delta
	n=ilist[1]		#Index to second large delta
	
	#Getting the direction of the rotation axis based on the two largest deltas
	
	Axis=CrossVecs(D[m],D[n])					
	if CrossVecs(Mat0[m], D[m])*Axis < 0:
		Axis*=-1
	
	#Setting the length of the rotation vec to the angle, based on the angle base vector index 'm' has rotated
	
	BIGEPSILON=1e-3       #1e-3. See comment about BIGEPSILON below.
	
	if Axis.length<EPSILON*EPSILON:
		rotvec=Vector([0,0,0])
	
	elif Axis.length< BIGEPSILON*BIGEPSILON:
		#Using a linear approximation that works well for small angles,
		#and that allows for small numerical deviations from
		#ideal orthonormal matrixes (and small numerical errors).	
		#We use Mat0 as an approximate orthonormal basis for the rotation
		#as an intermediate step, and get the average rotation components by dot product.
		#
		#Comment: Using different methods in different intervals will yield a small jump in the output of the rotation
		#vector as one passes from one interval to another.
		#This should not pose a problem if only the jump (of any component)
		#of the rotation vector is considerably smaller than the deltaangle used 
		#as one varies the physical bones (as far as I can see).
		#
		#Comment about BIGEPSILON: In some tests that led to adding this elif case, Axis.length was about 1e-14,
		#and then the below (else case) would at least in one case yield a rotation of 50 degrees,
		#when it should have been very close to zero.
		#So we should definetely have a boundary that is smaller than 1e-7*1e-7.
		#Choosing 1e-3 is quite arbitrary though.
		#
		#This method is also faster than the below method that calls AngleBetweenVecs.
		
		rotvecX=Mat0[2]*Mat1[1]-Mat0[1]*Mat1[2]
		rotvecY=Mat0[0]*Mat1[2]-Mat0[2]*Mat1[0]
		rotvecZ=Mat0[1]*Mat1[0]-Mat0[0]*Mat1[1]
		rotvec=0.5*(rotvecX*Mat0[0] + rotvecY*Mat0[1] + rotvecZ*Mat0[2]) 	
	
	else:
		#Comment: When Mat0 and Mat1 are both almost unit matrices, it can happen that
		#Axis is parallell (or very close to parallel) to Mat0[m] and Mat1[m], in which case vec1 and vec2 below become very very small vectors,
		#that can have a rather large angle between them. That is why the above elif case is inserted.
		
		Axis.normalize()	
		
		vec1=Mat0[m]-(Mat0[m]*Axis)*Axis								#Vector from axis and out to Mat0[m]
		vec2=Mat1[m]-(Mat1[m]*Axis)*Axis								#Vector from axis and out to Mat1[m]
		
		angle=AngleBetweenVecs(vec1,vec2)

		rotvec=Axis*angle
	
	return rotvec


def RotateMat(mat,rotvec):
	#Given a vector rotvec whose direction indicates the axis of rotation and whose length indicates the angle
	#the rotates each row of the matrix mat and returns these as matrix.
	
	if not rotvec.length<EPSILON:
		vec0=RotateVec(mat[0],rotvec)
		vec1=RotateVec(mat[1],rotvec)
		vec2=RotateVec(mat[2],rotvec)
		return Matrix(vec0,vec1,vec2)
	else:
		return Matrix(mat)	#Returns copy


def MidMat(mat0,mat1):
	#Given two orthogonal normed matrices, returns a matrix that is rotated halfway from the first matrix to the second matrix.
			
	rotvec=GetRotVec(mat0,mat1)
	return RotateMat(mat0,rotvec/2)


def SetRotMat(mat_link,newmat):
	#Given a link mat_link to an object matrix, sets the rotation part equal to 3x3 matrix mat.
	
	mat_link[0][0:3]=newmat[0]
	mat_link[1][0:3]=newmat[1]
	mat_link[2][0:3]=newmat[2]


#############################################################################
############# Functions related to bone initiation ##########################

def ModifyDefaultDict(ParameterDict,inParameterDict,callName):
	#Modifies the linked default ParameterDict dictionary according to modDict.
	#string is the name of the function calling this function (for diagnosis).
	
	for name in inParameterDict:
		val=inParameterDict[name]
		if name in ParameterDict:
			valDef=ParameterDict[name]
			
			okFlag=False
			
			#Insuring that floats and integers count the same
			
			if ( type(val)==type(1) or type(val)==type(1.0) ) and ( type(valDef)==type(1) or type(valDef)==type(1.0) ):
				okFlag=True
			
			#Insuring that 0 and 1 can be used in stead of boolean. Note that the default should use boolean when it is boolean (not 0 and 1).
			
			if (val==0 or val==1) and type(valDef)==type(True):
				okFlag=True	
			
			#Setting the new parameter if the type is okey.
			
			if okFlag or type(val)==type(valDef):	#0 and 1 are okey as booleans.
				ParameterDict[name]=inParameterDict[name]
			else:
				Draw.PupMenu(callName + ' Parameter: ' +name+ '=' +str(ParameterDict[name])+ ' should be of type '+str(type(ParameterDict[name]))+ '. Using default setting.' )	
		else:
			Draw.PupMenu(callName + 'Parameter: '+name+' not in default parameter list. Omitting it.' )	
	

#############################################################################
############# Functions related to bone initiation ##########################

def AddSimpleBones(SimpleBoneDict,BDict,arm_data):
	#Given a dictionary of dictionaries of the following type
	#SimpleBoneDict={'BoneName':{'density':1,'basediameter':	0.25, 'tipdiameter': 0.20, 'Object':True}, ...}
	#this function considers a frustum of a cone along the armature bone, whose height is the length of the armature bone in 
	#question, whose base diameter is 'basediameter', and whose tip diameter is 'tipdiameter'. For the given density
	#the function calculates the corresponding mass, center of mass and moment of inertia matrix and adds these
	#to the BDict argument (which is just a link - so the dictionary that was sent will be updated).
	
	for name in SimpleBoneDict:
		
		# Default parameters
		
		bdic={'density':1.0,	'basediameter':	1.0,	'tipdiameter': 1.0,"DiameterFactor":1.0,"Object":True}
		inDict=SimpleBoneDict[name]
		
		if type(inDict['basediameter'])==type('envelope'):
			inDict['basediameter']=-1.0
		
		if type(inDict['tipdiameter'])==type('envelope'):
			inDict['tipdiameter']=-1.0
		
		ModifyDefaultDict(bdic,inDict,'AddSimpleBones')
		
		# The rest
		
		try:
			h=arm_data.bones[name].length
		except:
			print "Simple bone: ", name ," has no corresponding bone in armature"
			continue			#In case the listed bone has no corresponding armature bone, try the next bone
			
		density=bdic['density']
		
		if bdic['basediameter']<-0.00001:				#If basediameter is given as a string, or negative value - use the envelope radiusHead.
			r1=arm_data.bones[name].headRadius
		else:
			r1=bdic['basediameter']*1.0/2
		
		if bdic['tipdiameter']<-0.00001:		#If basedimeter is given as a string - use the envelope radiusTail.
			r2=arm_data.bones[name].tailRadius
		else:
			r2=bdic['tipdiameter']*1.0/2
					
		#Simple bones may be rather slender, giving very low Iyy values. This means that to avoid numerical instability
		#one must have a very low y-spring constant (or low dt - but that is not currently settable).
		#For damping mode 0 (and 1), one must also have a low y-damping for numerical reasons.
		#In combination with the low inertia, this makes the bone twist very easily.
		#To modulate this we introduce a factor to multiply the bone radial dimensions without affecting the mass
		#(so we increas radial sizes and decrease density).
		#Note however that for large diameter factors - the entire object inertia matrix may be notably affected.
		#So use this cheat with some caution.
		
		df=bdic['DiameterFactor']
		density*=1.0/(df*df)
		r1*=df
		r2*=df
		
		#######
		
		mass=density*pi*h/3*(r1*r1+r1*r2+r2*r2)											#From wikipedia I think
		k=(r1-r2)/h																		#So the radius is given by r=r1-k*y - where y is distance along the cone axis from the base
		masscenter=pi*density*1.0/mass*(r1*r1*h*h/2 - 2*k*r1*h*h*h/3 + k*k*h*h*h*h/4)	#Just integrate the frustrated cone
		
		if abs(r1-r2)/max([r1,r2])<0.001:
			Iyy=mass*pow((r1+r2)*0.5,2)*0.5												#Treating it like a cylinder	
		else:
			Iyy=-density*pi/(k*10)*(  pow(r2,5) - pow(r1,5)  )							#Treating it like a frustum of a cone. Result of integration
			
		Ixx=Iyy/2 + density*pi*(r1*r1*pow(h,3)*1.0/3-2*r1*k*pow(h,4)*1.0/4+k*k*pow(h,5)*1.0/5) - mass*pow(masscenter,2)		#Result of integration
		Izz=Ixx
		
		if name in BDict:
			print "Replacing previous bone:", name, "by simple bone"
		
		BDict[name]={'mass':mass, 'masscenter':	Vector(0,masscenter,0), 'inertiamatrix':	Matrix([Ixx,0,0],[0,Iyy,0],[0,0,Izz]), 'Object': bdic['Object']}


def getSpringVecElastic(elasticVec,bone_data):
	#Returning a spring vector based on elastic properties of the material.
	#
	#If the shear modulus (for torsion) equals the the Young's modulus,
	#The torsion (y) component of the springvector would be twice that of the bending components
	#(x or z). However, for example for steel, the shear modulus is about 40% of the Young's
	#modulus, so setting the spring components equal is fairly accurate.
	#In this simple method, if the elasticVec is uniform (like (1,1,1)), 
	#then so will the output from this mehtod.
	#For wood, the shear modulus is higher relative to the Young's modulus, 
	#so we could opt to send an elasticVec of the type=(1,2,1) for example.
	
	baseDiameter=bone_data.headRadius*2
	boneLength=bone_data.length
	springVec=elasticVec*pow(baseDiameter,4)/boneLength*800
	
	return springVec
	
	#The factor of 800 is included to get some similarity with the results from getSpringVec().
	#For a bone with head diameter 0.6, tail diameter 0.4, length 1 and density 1,
	#the spring vec returned from getSpringVecElastic is similar (in x,z) to the one returned from
	#getSpringVec()
	#
	#Note that the longer the bone, the softer the head spring needs to be.
	#The idea is that even if you split a bone into several sub bones, the net bending
	#for given torques at the end (or the original bone) should be roughly the same.
	#
	#Note however that as you make the bones shorter and shorter - their eigenfrequencies
	#will go up due to both decrasing moment of inertia (du partially to decreasing mass)
	#but also to increasing head spring vector. So, if you want to use short bones,
	#be prepared to crank up the stepsPerFrame to avoid spring instability.


def getSpringVec(simpleVec,inertiamatrix,crel,mass,DeltaT):
	#Translates the components of simpleVec to an actual springVec. A value
	#of 1 in the simpleVec gives the numerically highest possible (sort of)
	#actual springVec value, to retain numerical stability for the given other parameters.  
	
	kvec=Vector(0,0,0)	#Just initiating
	for i in [0,1,2]:
		vec=Vector(crel)
		vec[i]=0
		s=vec.length								#The distance from the axis of rotation in question to the bone center of mass
		I=inertiamatrix[i][i]+mass*s*s				#Net axial inertia at the head
		kvec[i]=3*I/(DeltaT*DeltaT)*simpleVec[i]	#The factor of 3 is kind of empirical.
	
	return kvec
	
	
def getDampVec(simpleVec,inertiamatrix,crel,mass,DeltaT):
	#Translates the components of simpleVec to an actual springVec. A value
	#of 1 in the simpleVec gives the numerically highest possible (sort of)
	#actual springVec value, to retain numerical stability for the given other parameters.  
	
	bvec=Vector(0,0,0)	#Just initiating
	for i in [0,1,2]:
		vec=Vector(crel)
		vec[i]=0
		s=vec.length								#The distance from the axis of rotation in question to the bone center of mass
		I=inertiamatrix[i][i]+mass*s*s				#Net axial inertia at the head
		bvec[i]=I/DeltaT*simpleVec[i]	#The factor of 3 is kind of empirical.
		
	return bvec


####################################################################################
##############   Functions related to frame steps   ################################

def makelist(arg1,arg2):
	#Making a list that starts with arg1 and ends with arg2 in increments of +1 or -1 depending on if arg2>arg1 or not.
	
	if arg2>=arg1:
		return range(arg1,arg2+1)
	else:
		list=range(arg2,arg1+1)
		list.reverse()
		return list


def GetFramelist(FrameStartEnd, initframe):
	#Makes a list of frames for either evolution or monitoring.
	 
	if FrameStartEnd[0]=='currframe':
		FrameStartEnd[0]=initframe
		FrameStartEnd[1]+=initframe
	
	if FrameStartEnd[1]=='currframe':
		start=initframe-FrameStartEnd[0]
		if start<3:	#This is so one does not by mistake overwrite the initial conditions for the entire animation
			start=3
			print "Auto-setting startframe to 3"
		FrameStartEnd[0]=start
		FrameStartEnd[1]=initframe
	
	framelist=makelist(FrameStartEnd[0],FrameStartEnd[1]) 	#NB first frame to be changed is framelist[1] (not framelist[0]). Note that the first argument of the list may be larger than the second.
	
	return framelist


def GetDt(framelist,DeltaT):
	#Returns the time step dt.
	
	if framelist[1]>framelist[0]:
		return DeltaT
	else:
		return -DeltaT
		